"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CandyApiDraftClient = exports.CandyApiClient = exports.ServerErrors = exports.CandyResourceZones = void 0;
const TypingUtils_1 = require("../TypingUtils");
const HttpClient_1 = require("../HttpClient/HttpClient");
const AuthenticationMode_1 = require("../HttpClient/AuthenticationMode");
const ModelPath_1 = require("../ModelPath/ModelPath");
const TypeConversionHelper_1 = require("../TypeConversionHelper");
const FocusTypes_1 = require("./Types/Focus/FocusTypes");
const DraftPresentationType_1 = require("./Types/Presentations/DraftPresentationType");
const IsBuffer_1 = require("./IsBuffer");
const FufSizeTooLargeError_1 = require("./Types/DraftActions/FufSizeTooLargeError");
const NeedNormalizationError_1 = require("./Types/DraftActions/NeedNormalizationError");
const strangeOptions = { src: "changeSet", dataVersion: "0" };
exports.CandyResourceZones = {
    TestForms: "candy.resources.test-forms",
};
var ServerErrors;
(function (ServerErrors) {
    ServerErrors["FufSizeTooLarge"] = "FufSizeTooLarge";
    ServerErrors["BadRequest"] = "BadRequest";
    ServerErrors["UnknownError"] = "UnknownError";
})(ServerErrors || (exports.ServerErrors = ServerErrors = {}));
function convertPathLoadSpecificationToString(spec) {
    return ((spec.skip ? "-" : "") +
        (spec.pathOrPathPrefix === "*" ? spec.pathOrPathPrefix : spec.pathOrPathPrefix.toLegacyPath()));
}
class CandyApiClient {
    httpClient;
    constructor(fetchImpl, baseUrl, resourceZone, authentication, exceptionLogger, httpStatusHandlers) {
        this.httpClient = new HttpClient_1.HttpClient(fetchImpl, baseUrl, (0, AuthenticationMode_1.createAuthenticationProvider)(authentication), {
            resourceZone: resourceZone,
        }, exceptionLogger, httpStatusHandlers);
    }
    forDraft(draftFullId) {
        return new CandyApiDraftClient(this.httpClient, draftFullId);
    }
    async getInfoByInns(draftFullId, inns) {
        const paths = inns.map(inn => `@focus/${inn}`);
        const response = await this.httpClient
            .post(this.getDraftUrl(draftFullId, "get-inner"))
            .withJsonBody(paths)
            .json();
        const result = {};
        for (const inn of inns) {
            const innServerValue = response[`@focus/${inn}`]?.value;
            if (innServerValue == undefined) {
                result[inn] = {
                    value: undefined,
                    error: "Server does not return inn info",
                    errorCode: FocusTypes_1.FocusServerErrorCode.NoInnError,
                };
            }
            else {
                result[inn] = {
                    value: innServerValue.value ?? undefined,
                    errorCode: innServerValue.errorCode ?? undefined,
                    error: innServerValue.error ?? undefined,
                };
            }
        }
        return result;
    }
    async getAllGfvs(abortSignal) {
        return this.httpClient.get(`build-server/gfvs`).json();
    }
    async forceGetAllGfvs() {
        return this.httpClient.get(`build-server/force-gfvs`).json();
    }
    async createDraftFromDrive(ns, fufDriveUri, abortSignal) {
        return await this.httpClient
            .post(`v1/ns/${ns}/create-draft-from-drive`)
            .withQueryParams({
            ns: ns,
            fufDriveUri: fufDriveUri,
        })
            .withAbortSignal(abortSignal)
            .json();
    }
    async createEmptyDraft(ns, gfv, abortSignal) {
        return await this.httpClient
            .post(`v1/ns/${ns}/create-draft`)
            .withQueryParams({ gfv: gfv })
            .withAbortSignal(abortSignal)
            .json();
    }
    async createDraft(ns, gfv, fufContent) {
        if (gfv == undefined && fufContent != undefined) {
            return await this.httpClient
                .post(`v1/ns/${ns}/create-draft`)
                .withContentType("application/octet-stream")
                .withBodyFromBuffer(fufContent)
                .json();
        }
        if (gfv != undefined && fufContent == undefined) {
            return await this.httpClient
                .post(`v1/ns/${ns}/create-draft`)
                .withQueryParams({ gfv: gfv })
                .withContentType("application/octet-stream")
                .withBodyFromBuffer(Buffer.from([]))
                .json();
        }
        if (gfv == undefined && fufContent == undefined) {
            throw new Error("Cannot create draft: no gfv and no fuf specified");
        }
        return await this.httpClient
            .post(`v1/ns/${ns}/create-draft`)
            .withQueryParams({ gfv: gfv })
            .withContentType("application/octet-stream")
            .withBodyFromBuffer(fufContent)
            .json();
    }
    async checkAlive() {
        await this.httpClient.get(`_status/ping`);
    }
    getDraftUrl(draftFullId, draftRelatedUrl) {
        return `v1/ns/${draftFullId.ns}/drafts/${draftFullId.draftId}/${draftRelatedUrl}`;
    }
    async getFeatureFlags(abonId, orgId, userId) {
        return await this.httpClient
            .get("v1/features/flags")
            .withQueryParams({
            abon: abonId,
            org: orgId,
            user: userId,
        })
            .json();
    }
}
exports.CandyApiClient = CandyApiClient;
class CandyApiDraftClient {
    httpClient;
    draftFullId;
    normalizeDownloadLink = (uri) => uri.includes("response-content-disposition") ? uri : `${uri}?response-content-disposition=attachment`;
    constructor(httpClient, draftFullId) {
        this.httpClient = httpClient;
        this.draftFullId = draftFullId;
    }
    getHrefToInnerEditor() {
        return this.httpClient.url(`inner-editor/ns/${this.draftFullId.ns}/drafts/${this.draftFullId.draftId}`);
    }
    getHrefToConvertationLog() {
        return this.httpClient.url(this.getDraftUrl("conversion"));
    }
    async getInnerFuf() {
        return (await this.httpClient.get(this.getDraftUrl(DraftPresentationType_1.DraftPresentationType.innerFuf))).text();
    }
    getHrefToDownloadInnerFuf() {
        return this.getHrefDownloadDraftPresentation(DraftPresentationType_1.DraftPresentationType.innerFuf);
    }
    getHrefToDownloadClientInfo() {
        return this.getHrefDownloadDraftPresentation(DraftPresentationType_1.DraftPresentationType.clientInfo);
    }
    getHrefToDownloadFormsClientInfo() {
        return this.getHrefDownloadDraftPresentation(DraftPresentationType_1.DraftPresentationType.formsClientInfo);
    }
    async getErrorLog() {
        return this.httpClient.get(this.getDraftPresentationUrl(DraftPresentationType_1.DraftPresentationType.errorLog)).json();
    }
    async getErrorLogWithCheckScenarios(checkScenarios) {
        return this.httpClient
            .post(this.getDraftUrl("errorlog"))
            .withJsonBody({ checkScenarios: checkScenarios })
            .json();
    }
    async getTaskById(taskId) {
        const tasks = await this.httpClient
            .post(this.getDraftUrl("get-inner"))
            .withJsonBody([`@task/*`])
            .json();
        return tasks[`@task/${taskId}`]?.value;
    }
    async getFufWithAttachments() {
        const response = await this.httpClient
            .get(this.getDraftUrl("get-fuf-with-attachments"))
            .json();
        return response;
    }
    async getPartialFuf(rules) {
        const response = await this.httpClient
            .post(this.getDraftUrl("get-partial-fuf"))
            .withJsonBody(rules)
            .json();
        return response;
    }
    async getAllTasks() {
        const tasks = await this.httpClient
            .post(this.getDraftUrl("get-inner"))
            .withJsonBody([`@task/*`])
            .json();
        return Object.values(tasks).map(x => x.value);
    }
    async downloadFuf() {
        const response = await this.httpClient.get(this.getDraftUrl("get-fuf-link")).json();
        if (!response.resultUri) {
            (0, TypingUtils_1.reject)("Failed to receive FUF link");
        }
        window.location.href = this.normalizeDownloadLink(response.resultUri);
    }
    async makeDownloadDriveLink(uri, fileName) {
        const url = fileName
            ? `${uri}?response-content-disposition=attachment;filename%3D%22${fileName}%22`
            : this.normalizeDownloadLink(uri);
        window.location.href = url;
    }
    async downloadInner() {
        const response = await this.httpClient.get(this.getDraftPresentationUrl(DraftPresentationType_1.DraftPresentationType.inner));
        const responseBlob = await response.blob();
        const innerFileBlob = new Blob([responseBlob], {
            type: (0, HttpClient_1.getContentType)(response) ?? "application/json",
        });
        const innerFileUrl = window.URL.createObjectURL(innerFileBlob);
        const link = document.createElement("a");
        link.href = innerFileUrl;
        link.download = (0, HttpClient_1.getFileName)(response) ?? "inner.json";
        link.click();
        link.remove();
        setTimeout(() => {
            window.URL.revokeObjectURL(innerFileUrl);
        }, 250);
    }
    async enqueueTask(taskId, taskInfo) {
        await this.httpClient
            .post(this.getDraftPresentationUrl(DraftPresentationType_1.DraftPresentationType.inner))
            .withQueryParams(strangeOptions)
            .withJsonBody({
            added: [],
            changed: {
                [`@task/${taskId}`]: {
                    ...taskInfo,
                    id: taskId,
                },
            },
            removed: [],
        });
    }
    async cancelTask(taskId) {
        await this.httpClient
            .post(this.getDraftPresentationUrl(DraftPresentationType_1.DraftPresentationType.inner))
            .withQueryParams(strangeOptions)
            .withJsonBody({
            added: [],
            changed: {},
            removed: [`@task/${taskId}`],
        });
    }
    getExpertErrorLog() {
        return this.httpClient
            .get(this.getDraftPresentationUrl(DraftPresentationType_1.DraftPresentationType.errorlogExpert))
            .json();
    }
    getHrefToErrorLogPdf(errorLogName, propsName) {
        return this.httpClient.url(this.getDraftUrl("presentations/errorLog/print"), {
            errorLogName: errorLogName,
            propsDriveName: propsName,
        });
    }
    async clearDraft() {
        await this.httpClient.post(this.getDraftUrl("clear"));
    }
    async convertDraftFromPrevGfv({ prevGfvDraftId, gfv }) {
        await this.httpClient.post(this.getDraftUrl("convert")).withQueryParams({
            id: prevGfvDraftId,
            gfv: gfv,
        });
    }
    async sendChangeWithNormalizationSet(data, currentDataVersion, normalizerId, operationName) {
        const result = await this.httpClient
            .post(this.getDraftUrl("inner/patch-with-normalization"))
            .withQueryParams({
            dataVersion: currentDataVersion,
            normalizerId: normalizerId,
            operationName: operationName ?? undefined,
        })
            .withJsonBody(data)
            .acceptCode(409);
        return result.status == 200 ? await result.json() : result.status;
    }
    async sendChangeSet(data, currentDataVersion, operationName) {
        const result = await this.httpClient
            .post(this.getDraftUrl("inner"))
            .withQueryParams({ dataVersion: currentDataVersion, operationName: operationName ?? undefined })
            .withJsonBody(data)
            .acceptCode(409);
        const getSuccessResult = async () => {
            const dataResult = await result.json();
            return {
                dataVersion: dataResult?.dataVersion ?? (0, TypingUtils_1.reject)("Invalid dataVersion after applying change set."),
            };
        };
        return result.status == 200 ? await getSuccessResult() : result.status;
    }
    sendChangeSetSync(data, currentDataVersion, normalizerId) {
        const result = this.httpClient.postSync(this.getDraftUrl("inner"), { dataVersion: currentDataVersion }, data);
        const getSuccessResultSync = () => {
            const dataResult = result.json();
            return {
                dataVersion: dataResult?.dataVersion ?? (0, TypingUtils_1.reject)("Invalid dataVersion after applying change set."),
            };
        };
        return result.status == 200 ? getSuccessResultSync() : result.status;
    }
    async getRequisites(requisiteList) {
        return await this.httpClient
            .post(this.getDraftUrl("get-inner"))
            .withJsonBody(requisiteList)
            .json();
    }
    async getInner(paths) {
        return await this.httpClient
            .post(this.getDraftUrl("get-inner"))
            .withJsonBody(paths)
            .json();
    }
    getPartialInner(rules, maxInstanceCount) {
        const convertedRules = rules.map(convertPathLoadSpecificationToString);
        return this.httpClient
            .post(this.getDraftUrl("get-partial-inner"))
            .withQueryParams(maxInstanceCount != undefined && { maxInstanceCount: maxInstanceCount.toString() })
            .withJsonBody(convertedRules.length > 0 ? convertedRules : ["*"])
            .json();
    }
    async getPartialInnerAsJsonTree(rules, maxInstanceCount, skipNormalization) {
        const convertedRules = rules.map(convertPathLoadSpecificationToString);
        const response = await this.httpClient
            .post(this.getDraftUrl("get-partial-inner"))
            .withQueryParams(maxInstanceCount != undefined && { maxInstanceCount: maxInstanceCount.toString() })
            .withQueryParams({ format: "JsonTree", useDraftNormalization: !skipNormalization })
            .withJsonBody(convertedRules.length > 0 ? convertedRules : ["*"])
            .acceptCode(424);
        if (response.status === 424) {
            throw new NeedNormalizationError_1.NeedNormalizationError();
        }
        return response.json();
    }
    async getRawPartialInnerAsJsonTree(rules, maxInstanceCount, skipNormalization) {
        const convertedRules = rules.map(convertPathLoadSpecificationToString);
        const response = await this.httpClient
            .post(this.getDraftUrl("get-partial-inner2"))
            .withQueryParams(maxInstanceCount != undefined && { maxInstanceCount: maxInstanceCount.toString() })
            .withQueryParams({ useDraftNormalization: !skipNormalization })
            .withJsonBody(convertedRules.length > 0 ? convertedRules : ["*"])
            .acceptCode(424);
        if (response.status === 424) {
            throw new NeedNormalizationError_1.NeedNormalizationError();
        }
        return response.json();
    }
    getDataVersion() {
        return this.httpClient.get(this.getDraftUrl("inner/get-data-version")).json();
    }
    startNormalizationTask() {
        return this.httpClient.post(this.getDraftUrl("normalization-task")).json();
    }
    getNormalizationTask() {
        return this.httpClient.get(this.getDraftUrl("normalization-task")).json();
    }
    async getExplanatoryNote() {
        return await this.httpClient
            .get(this.getDraftPresentationUrl(DraftPresentationType_1.DraftPresentationType.explanatoryNote))
            .json();
    }
    async getAttachmentById(attachmentId) {
        const result = await this.httpClient
            .post(this.getDraftUrl("get-inner"))
            .withJsonBody([`@attachment/${attachmentId}`])
            .json();
        return result[`@attachment/${attachmentId}`]?.value ?? (0, TypingUtils_1.reject)();
    }
    async getAllAttachments() {
        const result = await this.httpClient
            .post(this.getDraftUrl("get-inner"))
            .withJsonBody([`@attachment/*`])
            .json();
        return Object.values(result).map(x => x.value);
    }
    async removeAttachments(attachmentIds) {
        await this.httpClient
            .post(this.getDraftPresentationUrl(DraftPresentationType_1.DraftPresentationType.inner))
            .withQueryParams(strangeOptions)
            .withJsonBody({
            added: [],
            changed: {},
            removed: attachmentIds.map(id => `@attachment/${id}`),
        });
    }
    async createOrUpdateAttachments(attachments) {
        const changed = {};
        for (const attachment of attachments) {
            changed[`@attachment/${attachment.id}`] = {
                contentUri: attachment.contentUri,
                id: attachment.id,
                meta: attachment.meta,
            };
        }
        await this.httpClient
            .post(this.getDraftPresentationUrl(DraftPresentationType_1.DraftPresentationType.inner))
            .withQueryParams(strangeOptions)
            .withJsonBody({
            added: [],
            changed: changed,
            removed: [],
        });
    }
    async createDraftAttachment(gfv, path) {
        return await this.httpClient
            .post(this.getDraftUrl("external/create-attachment-draft"))
            .withQueryParams({ gfv: gfv, path: path })
            .json();
    }
    async openLoadedAttachmentAsDraft(id) {
        const response = await this.httpClient
            .post(this.getDraftUrl("external/open-loaded-attachment-as-draft"))
            .withQueryParams({ attachmentId: id })
            .acceptCode(413);
        if (response.status === 413) {
            const errorData = await response.json();
            if (errorData.errorCode === "FufSizeTooLarge") {
                throw new FufSizeTooLargeError_1.FufSizeTooLargeError(errorData.message);
            }
            throw new HttpClient_1.HttpError(errorData.message, response.status);
        }
        return await response.json();
    }
    async setPresentation(presentationType, content) {
        let request = this.httpClient.put(this.getDraftPresentationUrl(presentationType));
        if (typeof content === "string") {
            request = request.withRawBody(content);
        }
        else if ((0, IsBuffer_1.isBuffer)(content)) {
            request = request.withRawBody(content);
        }
        else {
            request = request.withJsonBody(content);
        }
        await request;
    }
    async patchPresentation(presentationType, presentationContent) {
        await this.httpClient.patch(this.getDraftPresentationUrl(presentationType)).withJsonBody(presentationContent);
    }
    async getFileProperties(driveUri, ignoreSizeCheck) {
        const response = await this.httpClient
            .get(`v1/files/get-properties`)
            .withQueryParams({ driveUri: driveUri, ignoreSizeCheck: ignoreSizeCheck ?? false })
            .acceptCode(400)
            .acceptCode(413);
        if (response.status === 400 || response.status === 413 || response.status === 200) {
            return response.json();
        }
        return undefined;
    }
    async getFileNdsProperties({ driveUri, declarationGfv, requestedGfv, ignoreSizeCheck, }) {
        const response = await this.httpClient
            .get(`v1/files/get-properties-nds`)
            .withQueryParams({
            driveUri: driveUri,
            ignoreSizeCheck: ignoreSizeCheck ?? false,
            declarationGfv: declarationGfv,
            requiredGfv: requestedGfv,
        })
            .acceptCode(400)
            .acceptCode(413);
        if (response.status === 400 || response.status === 413 || response.status === 200) {
            return response.json();
        }
        return undefined;
    }
    async prepareAndGetFileProperties(driveUri, gfv, ignoreSizeCheck) {
        const response = await this.httpClient
            .get(`v1/files/prepare-and-get-properties`)
            .withQueryParams({ gfv: gfv, driveUri: driveUri, ignoreSizeCheck: ignoreSizeCheck ?? false })
            .acceptCode(413);
        if (response.status === 400) {
            return undefined;
        }
        if (response.status === 413) {
            const errorData = await response.json();
            if (errorData.errorCode === "FufSizeTooLarge") {
                throw new FufSizeTooLargeError_1.FufSizeTooLargeError(errorData.message);
            }
            throw new HttpClient_1.HttpError(errorData.message, response.status);
        }
        return await response.json();
    }
    async uploadAndGetFileProperties(driveUris, gfv, abortSignal) {
        try {
            const response = await this.httpClient
                .post(`v1/files/upload-and-get-properties`)
                .withQueryParams({ gfv: gfv })
                .withJsonBody(driveUris)
                .withAbortSignal(abortSignal)
                .acceptCode(400)
                .acceptCode(413);
            if (response.status !== 200) {
                const data = await response.json();
                return { serverError: data.errorCode, invalidSourceDriveContentNames: "", sourcesTotalCount: 0 };
            }
            const data = await response.json();
            return data;
        }
        catch (e) {
            return {
                serverError: ServerErrors.UnknownError,
                invalidSourceDriveContentNames: "",
                sourcesTotalCount: 0,
            };
        }
    }
    async acceptMultipleTransferType(files, abortSignal) {
        try {
            const response = await this.httpClient
                .post(`v1/ns/${this.draftFullId.ns}/drafts/${this.draftFullId.draftId}/transfer/combine-multiple`)
                .withJsonBody(files)
                .withAbortSignal(abortSignal)
                .acceptCode(400)
                .acceptCode(413);
            if (response.status !== 200) {
                const data = await response.json();
                return { serverError: data.errorCode };
            }
            return undefined;
        }
        catch (e) {
            return { serverError: ServerErrors.UnknownError };
        }
    }
    async setFufByDriveUri(driveUri) {
        await this.httpClient.put(this.getDraftUrl("set-fuf")).withQueryParams({ driveUri: driveUri });
    }
    async excelsToPartialFufs(gfv, pathToTable, driveUris) {
        try {
            const response = await this.httpClient
                .post("v1/files/excels-to-partial-fufs")
                .withQueryParams({ gfv: gfv, pathToTable: pathToTable })
                .withJsonBody(driveUris);
            if (response.status === 200) {
                return await response.json();
            }
            return undefined;
        }
        catch (e) {
            return undefined;
        }
    }
    async importXmls(driveUris, combineMethod, combinatorType, isImportByFuf, args) {
        const response = await this.httpClient
            .post(this.getDraftUrl("transfer/import-xmls"))
            .withQueryParams({
            combineMethod: combineMethod,
            combinatorType: combinatorType,
            importByFuf: isImportByFuf,
        })
            .withJsonBody({ FufDriveUris: driveUris, arguments: args });
        if (response.status === 400) {
            return undefined;
        }
        return await response.json();
    }
    async setDraftSettings(settings) {
        await this.httpClient.patch(this.getDraftUrl("external/set-settings")).withJsonBody(settings);
    }
    async getDraftSettings() {
        const response = await this.httpClient.get(this.getDraftUrl("external/get-settings"));
        return response.json();
    }
    async normalizeByRules(rules) {
        await this.httpClient.post(this.getDraftUrl("test-normalize")).withJsonBody(rules);
    }
    getHrefDownloadDraftPresentation(presentationType) {
        return this.httpClient.url(this.getDraftPresentationUrl(presentationType));
    }
    getDraftUrl(draftRelatedUrl) {
        return `v1/ns/${this.draftFullId.ns}/drafts/${this.draftFullId.draftId}/${draftRelatedUrl}`;
    }
    getDraftPresentationUrl(presentationType) {
        return this.getDraftUrl(`presentations/${presentationType.toString()}`);
    }
    async setMultipleWithNormalization(rules) {
        const uri = this.getDraftUrl("inner/set-multiple-with-normalization");
        const finalRules = rules.map(x => ({ ...x, path: (0, ModelPath_1.convertToLegacyPath)(x.path) }));
        const result = await this.httpClient.patch(uri).withJsonBody(finalRules);
        return result.status == 200 ? await result.json() : result.status;
    }
    async normalizeDirtyMasks() {
        const uri = this.getDraftUrl("inner/normalize-dirty-masks");
        const result = await this.httpClient.post(uri);
        return result.status == 200 ? await result.json() : result.status;
    }
    async searchInstancesByText(text, pathsPrefix, pathsToSearch, maxInstanceCount) {
        const maxInstanceCountParams = maxInstanceCount != undefined ? { maxInstanceCount: maxInstanceCount } : {};
        const params = { searchString: text, pathsPrefix: pathsPrefix, ...maxInstanceCountParams };
        return this.httpClient
            .post(this.getDraftUrl("inner/search-by-string"))
            .withQueryParams(params)
            .withJsonBody(pathsToSearch)
            .json();
    }
    async queryCollection(request, abortSignal) {
        return this.httpClient
            .post(this.getDraftUrl("inner/query-collection"))
            .withJsonBody(request)
            .withAbortSignal(abortSignal)
            .json();
    }
    async resolveInnerPath(request, abortSignal) {
        return this.httpClient
            .post(this.getDraftUrl("inner/resolve-inner-path"))
            .withJsonBody(request)
            .withAbortSignal(abortSignal)
            .json();
    }
    async customNormalize(normalizerId, pathPrefixes, abortSignal) {
        const params = { normalizerId: normalizerId };
        const result = await this.httpClient
            .post(this.getDraftUrl("inner/custom-normalize"))
            .withQueryParams(params)
            .withJsonBody(pathPrefixes.map(x => x.toString()))
            .withAbortSignal(abortSignal);
        return result.status == 200 ? await result.json() : result.status;
    }
    async importEmployees(request) {
        await this.httpClient.patch(this.getDraftUrl("staff")).withJsonBody(request);
    }
    async deleteInstances(deletingRules) {
        await this.httpClient.post(this.getDraftUrl("inner/delete-instances")).withJsonBody(deletingRules);
    }
    async runMassAction(actionId, parameters) {
        const queryParams = { actionId: actionId };
        await this.httpClient
            .post(this.getDraftUrl("inner/mass-action"))
            .withQueryParams(queryParams)
            .withJsonBody(parameters);
    }
    print(type, body) {
        return this.httpClient
            .post(this.getDraftUrl(`presentations/inner/print/${type}`))
            .withJsonBody(body)
            .json();
    }
    async printExcel(pathToTable) {
        try {
            const response = await this.httpClient
                .post(this.getDraftUrl(`presentations/table/print`))
                .withQueryParams({ pathToTable: pathToTable });
            if (response.status === 200) {
                const tableExcel = response.json();
                return tableExcel;
            }
            return undefined;
        }
        catch (e) {
            return undefined;
        }
    }
    async checkByPopd() {
        const response = await this.httpClient.get(`v1/ns/${this.draftFullId.ns}/drafts/${this.draftFullId.draftId}/checkByPopd`);
        return response.json();
    }
    async getHistory() {
        const url = this.getDraftUrl("innerhistory");
        const body = await this.httpClient
            .get(url)
            .acceptCode(400)
            .acceptCode(404)
            .acceptCode(500)
            .json();
        return body.map(item => ({
            ...item,
            DateUtc: (0, TypeConversionHelper_1.convertDateFromStringIfNeeded)(item.DateUtc),
            ChangesExpiresUtc: (0, TypeConversionHelper_1.convertDateFromStringIfNeeded)(item.ChangesExpiresUtc),
        }));
    }
    async undoToChange(changeGUID) {
        const url = this.getDraftUrl(`innerhistory/${changeGUID}`);
        const body = await this.httpClient.post(url).acceptCode(400).acceptCode(500);
        return body.status;
    }
    async getGFV() {
        const url = `v1/ns/${this.draftFullId.ns}/drafts/${this.draftFullId.draftId}`;
        const body = await this.httpClient.get(url).json();
        const gfv = body.GFV;
        return `${gfv.Group}${gfv.Format}${gfv.Variant}`;
    }
    async getUser() {
        const url = `v1/user`;
        const body = await this.httpClient.get(url).json();
        return body;
    }
}
exports.CandyApiDraftClient = CandyApiDraftClient;
