"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFileName = exports.getContentType = exports.HttpError = exports.HttpClient = exports.HttpClientRequestWrapper = exports.HttpClientResponse = void 0;
const Errors_1 = require("../Errors");
const TypingUtils_1 = require("../TypingUtils");
function trimTrailingSlash(url) {
    return url.slice(-1) === "/" ? url.slice(0, -1) : url;
}
function trimLeadingSlash(url) {
    return url[0] === "/" ? url.slice(1) : url;
}
class HttpClientResponse {
    fetchResponse;
    constructor(fetchResponse) {
        this.fetchResponse = fetchResponse;
    }
    get status() {
        return this.fetchResponse.status;
    }
    get headers() {
        return this.fetchResponse.headers;
    }
    async json() {
        const responseData = await this.fetchResponse.json();
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return responseData;
    }
    async blob() {
        const responseData = await this.fetchResponse.blob();
        return responseData;
    }
    async text() {
        const responseData = await this.fetchResponse.text();
        return responseData;
    }
}
exports.HttpClientResponse = HttpClientResponse;
class HttpClientRequestWrapper {
    executeRequest;
    acceptCodes = [];
    httpStatusHandlers = {};
    queryParams = {};
    signal;
    bodyFactory;
    contentType;
    constructor(execute) {
        this.executeRequest = execute;
    }
    withAbortSignal(signal) {
        this.signal = signal;
        return this;
    }
    withContentType(contentType) {
        this.contentType = contentType;
        return this;
    }
    withBodyFromBuffer(body) {
        if (body === false || body == undefined) {
            return this;
        }
        this.bodyFactory = () => body;
        return this;
    }
    withJsonBody(serializableBody) {
        if (serializableBody !== false && serializableBody != undefined) {
            this.bodyFactory = () => JSON.stringify(serializableBody);
        }
        return this;
    }
    withQueryParams(queryParams) {
        if (queryParams) {
            this.queryParams = {
                ...this.queryParams,
                ...queryParams,
            };
        }
        return this;
    }
    withRawBody(body) {
        if (body !== false && body != undefined) {
            this.bodyFactory = () => body;
        }
        return this;
    }
    withHttpStatusHandler(statusCode, handler) {
        this.httpStatusHandlers = {
            ...this.httpStatusHandlers,
            [statusCode]: handler,
        };
        return this;
    }
    withHttpStatusHandlers(httpStatusHandlers) {
        this.httpStatusHandlers = { ...httpStatusHandlers };
        return this;
    }
    async json() {
        const result = await this;
        return result.json();
    }
    async blob() {
        const result = await this;
        return result.blob();
    }
    acceptCode(code) {
        this.acceptCodes.push(code);
        return this;
    }
    then = (...args) => {
        const resultPromise = (async () => {
            const result = await this.executeRequest(this.createConfig());
            if (result instanceof HttpClientResponse) {
                return result;
            }
            else {
                return new HttpClientResponse(result);
            }
        })();
        return resultPromise.then(...args);
    };
    createConfig() {
        return {
            signal: this.signal,
            bodyFactory: this.bodyFactory,
            acceptCodes: this.acceptCodes,
            httpStatusHandlers: this.httpStatusHandlers,
            queryParams: this.queryParams,
            contentType: this.contentType,
        };
    }
    static create(fn) {
        return new HttpClientRequestWrapper(fn);
    }
}
exports.HttpClientRequestWrapper = HttpClientRequestWrapper;
class HttpClient {
    fetchImpl;
    baseUrl;
    authenticationProvider;
    defaultParams = {};
    exceptionLogger;
    httpStatusHandlers;
    constructor(fetchImpl, baseUrl, authenticationProvider, defaultQueryParams, exceptionLogger, httpStatusHandlers) {
        this.fetchImpl = fetchImpl;
        this.baseUrl = baseUrl == undefined ? baseUrl : trimTrailingSlash(baseUrl);
        this.defaultParams = defaultQueryParams;
        this.authenticationProvider = authenticationProvider;
        this.exceptionLogger = exceptionLogger;
        this.httpStatusHandlers = { ...httpStatusHandlers };
    }
    get(url) {
        return this.method("GET")(url);
    }
    post(url) {
        return this.method("POST")(url);
    }
    put(url) {
        return this.method("PUT")(url);
    }
    patch(url) {
        return this.method("PATCH")(url);
    }
    delete(url) {
        return this.method("DELETE")(url);
    }
    method(method) {
        return (url) => HttpClientRequestWrapper.create(async (config) => {
            const fetchUrl = this.url(url, config.queryParams);
            let request = this.appendDefaultRequestHeaders({ method: method }, config.contentType);
            request = this.authenticationProvider(request);
            if (config.signal) {
                request.signal = config.signal;
            }
            if (config.bodyFactory) {
                if (method === "GET") {
                    throw new Errors_1.InvalidArgumentError("method", "Body is not allowed for GET requests");
                }
                request.body = config.bodyFactory();
            }
            try {
                const response = await this.fetchImpl(fetchUrl, request);
                await this.checkStatus(response, config);
                return response;
            }
            catch (e) {
                this.exceptionLogger?.(e, { fetchUrl: fetchUrl });
                return (0, TypingUtils_1.reject)(e);
            }
        });
    }
    appendDefaultRequestHeaders(options, contentType) {
        options.headers = options.headers ?? {};
        // @ts-ignore
        options.headers["Content-Type"] = contentType ?? "application/json";
        // @ts-ignore
        options.headers["Cache-Control"] = "no-cache, no-store";
        // @ts-ignore
        options.headers["Pragma"] = "no-cache";
        // @ts-ignore
        options.headers["Expires"] = "0";
        return options;
    }
    url(path, params) {
        let result = path;
        if (this.baseUrl != undefined) {
            result = this.baseUrl + "/" + trimLeadingSlash(result);
        }
        const resultParams = { ...this.defaultParams, ...params };
        if (path.includes("?") && params != undefined && Object.keys(resultParams).length > 0) {
            throw new Errors_1.NotSupportedError(`Passing search params in url and via params object is not supported`);
        }
        result = result + this.createQueryString(resultParams);
        return result;
    }
    async checkStatus(response, config) {
        const isSuccessResponseStatus = response.status >= 200 && response.status < 300;
        const isAcceptCodesIncludesResponseStatus = (config.acceptCodes ?? []).includes(response.status);
        if (!!config.httpStatusHandlers[response.status]) {
            await config.httpStatusHandlers[response.status]?.(response);
        }
        else if (!!this.httpStatusHandlers[response.status]) {
            await this.httpStatusHandlers[response.status]?.(response);
        }
        if (!isSuccessResponseStatus && !isAcceptCodesIncludesResponseStatus) {
            const errorText = await response.text();
            let serverResponse;
            try {
                serverResponse = JSON.parse(errorText);
            }
            catch (e) {
                serverResponse = undefined;
            }
            if (serverResponse != undefined) {
                if ("error" in serverResponse && typeof serverResponse.error === "object") {
                    throw new HttpError(JSON.stringify(serverResponse.error), response.status);
                }
                throw new HttpError(
                // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
                serverResponse.Message ?? serverResponse.message ?? serverResponse.Error, response.status);
            }
            throw new HttpError(errorText, response.status);
        }
    }
    postSync(url, params, data) {
        const request = new XMLHttpRequest();
        request.open("POST", this.url(url, params), false);
        request.withCredentials = true;
        request.setRequestHeader("Content-Type", "application/json; charset=UTF-8");
        request.send(JSON.stringify(data));
        return {
            status: request.status,
            json: () => JSON.parse(request.responseText),
        };
    }
    makeParamValueString(param, value) {
        return `${encodeURIComponent(param)}=${encodeURIComponent(value)}`;
    }
    createQueryString(params) {
        if (params == undefined) {
            return "";
        }
        const params2 = params;
        let result = Object.keys(params)
            .map(key => {
            const value = params2[key];
            if (typeof value === "string") {
                return this.makeParamValueString(key, value);
            }
            if (typeof value === "number" || typeof value === "boolean") {
                return this.makeParamValueString(key, value.toString());
            }
            if (Array.isArray(value)) {
                return value
                    .map(item => {
                    if (typeof item === "string") {
                        return this.makeParamValueString(key, item);
                    }
                    if (typeof item === "number") {
                        return this.makeParamValueString(key, item.toString());
                    }
                    return undefined;
                })
                    .filter(x => x != undefined)
                    .join("&");
            }
            return undefined;
        })
            .filter(x => x != undefined)
            .join("&");
        result = result ? "?" + result : "";
        return result;
    }
    createFetchImpl() {
        const fetchWithOptions = (input, init) => {
            if (typeof input === "string" || input instanceof URL) {
                const fetchUrl = this.url(input.toString(), undefined);
                init = init ?? {};
                init.headers = init.headers ?? {};
                // @ts-ignore
                init.headers["Cache-Control"] = "no-cache, no-store";
                // @ts-ignore
                init.headers["Pragma"] = "no-cache";
                // @ts-ignore
                init.headers["Expires"] = "0";
                init = this.authenticationProvider(init);
                return this.fetchImpl(fetchUrl, init);
            }
            else {
                const fetchUrl = this.url(input.url, undefined);
                let requestInit = {
                    headers: {
                        ["Cache-Control"]: "no-cache, no-store",
                        ["Pragma"]: "no-cache",
                        ["Expires"]: "0",
                        ...input.headers,
                    },
                    signal: input.signal,
                    cache: input.cache,
                    credentials: input.credentials,
                    integrity: input.integrity,
                    keepalive: input.keepalive,
                    method: input.method,
                    mode: input.mode,
                    redirect: input.redirect,
                    referrer: input.referrer,
                    referrerPolicy: input.referrerPolicy,
                };
                requestInit = this.authenticationProvider(requestInit);
                return this.fetchImpl(fetchUrl, requestInit);
            }
        };
        return fetchWithOptions;
    }
}
exports.HttpClient = HttpClient;
class HttpError extends Error {
    statusCode;
    constructor(message, statusCode) {
        super(message);
        this.name = "HttpError";
        this.statusCode = statusCode;
    }
    get message() {
        return super.message + `. StatusCode: ${this.statusCode}`;
    }
}
exports.HttpError = HttpError;
const getContentType = (response) => response.headers.get("Content-Type");
exports.getContentType = getContentType;
const getFileName = (response) => response.headers.get("Content-Disposition")?.split(";")[1]?.split("=")[1]?.replace(/['"]/g, "");
exports.getFileName = getFileName;
