"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IterUtils = void 0;
exports.reverseArray = reverseArray;
exports.toAsyncIterator = toAsyncIterator;
exports.findLast = findLast;
exports.concat = concat;
exports.iterateValues = iterateValues;
exports.iterateKeys = iterateKeys;
exports.iterateEntries = iterateEntries;
exports.count = count;
const TypingUtils_1 = require("./TypingUtils");
const Errors_1 = require("./Errors");
function* reverseArray(array) {
    for (let i = array.length - 1; i >= 0; i--) {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        yield array[i];
    }
}
function toAsyncIterator(generator) {
    return AsyncIterator.from(generator());
}
function findLast(items, condition) {
    let result;
    for (const item of items) {
        if (condition(item)) {
            result = item;
        }
    }
    return result;
}
function* concat(...iterables) {
    for (const iterableItem of iterables) {
        yield* iterableItem;
    }
}
function* iterateValues(obj) {
    for (const key in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, key)) {
            yield obj[key];
        }
    }
}
function* iterateKeys(obj) {
    for (const key in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, key)) {
            yield key;
        }
    }
}
function* iterateEntries(obj) {
    for (const key in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, key)) {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            yield [key, obj[key]];
        }
    }
}
function count(items) {
    let result = 0;
    for (const _item of items) {
        result++;
    }
    return result;
}
const EmptySingleElement = Symbol.for("EmptySingleElement");
const NoValueMarker = Symbol.for("NoValueMarker");
class IterUtils {
    static count = count;
    static concat = concat;
    static isEmpty(iterable) {
        for (const _item of iterable) {
            return false;
        }
        return true;
    }
    static groupBy(iterable, selectKey) {
        const result = new Map();
        for (const item of iterable) {
            const key = selectKey(item);
            const keyList = result.get(key);
            if (keyList == undefined) {
                result.set(key, [item]);
            }
            else {
                keyList.push(item);
            }
        }
        return result;
    }
    static sortBy(items, selector) {
        const result = [...items];
        result.sort((aItem, bItem) => {
            const leftValue = selector(aItem);
            const rightValue = selector(bItem);
            if (leftValue === rightValue) {
                return 0;
            }
            else if (leftValue < rightValue) {
                return -1;
            }
            else {
                return 1;
            }
        });
        return result;
    }
    static last(items) {
        if (Array.isArray(items)) {
            return items[items.length - 1];
        }
        let lastItem;
        for (const item of items) {
            lastItem = item;
        }
        return lastItem;
    }
    static first(items) {
        if (Array.isArray(items)) {
            return items[0];
        }
        let firstItem;
        for (const item of items) {
            firstItem = item;
            break;
        }
        return firstItem;
    }
    static single(items, singleAssertMessage) {
        if (Array.isArray(items)) {
            if (items.length !== 1) {
                throw new Error(singleAssertMessage ?? `Sequence must contain exactly single element, but got ${items.length}`);
            }
            return items[0] ?? (0, TypingUtils_1.reject)();
        }
        else {
            let singleElement = EmptySingleElement;
            for (const item of items) {
                if (singleElement !== EmptySingleElement) {
                    throw new Error(singleAssertMessage ?? `Sequence must contain exactly single element, but got at least 2`);
                }
                singleElement = item;
            }
            if (singleElement === EmptySingleElement) {
                throw new Error(singleAssertMessage ?? "Sequence must contain exactly single element, but got 0");
            }
            return singleElement;
        }
    }
    static *iterateEntries(items) {
        let index = 0;
        for (const item of items) {
            yield [index, item];
            index++;
        }
    }
    static *skipLast(items, skipCount) {
        if (skipCount === 0) {
            yield* items;
            return;
        }
        const prevItems = new Array(skipCount).fill(EmptySingleElement);
        let resultItem = EmptySingleElement;
        let currentIndex = 0;
        for (const item of items) {
            // @ts-expect-error В общем, строка выше гарантирует длину массива
            resultItem = prevItems[currentIndex];
            prevItems[currentIndex] = item;
            currentIndex = (currentIndex + 1) % skipCount;
            if (resultItem != EmptySingleElement) {
                yield resultItem;
            }
        }
    }
    static minBy(items, compareFunc) {
        let minValue = NoValueMarker;
        for (const item of items) {
            if (minValue === NoValueMarker) {
                minValue = item;
            }
            else if (compareFunc(item, minValue) === -1) {
                minValue = item;
            }
        }
        if (minValue === NoValueMarker) {
            throw new Errors_1.InvalidArgumentError("items", "Items should be not empty list");
        }
        return minValue;
    }
    static minByOrUndefined(items, compareFunc) {
        let minValue = NoValueMarker;
        for (const item of items) {
            if (minValue === NoValueMarker) {
                minValue = item;
            }
            else if (compareFunc(item, minValue) === -1) {
                minValue = item;
            }
        }
        if (minValue === NoValueMarker) {
            return undefined;
        }
        return minValue;
    }
    static *range(inStartOrCount, inItemCount) {
        const start = inItemCount === undefined ? 0 : inStartOrCount;
        const itemCount = inItemCount === undefined ? inStartOrCount : inItemCount;
        if (itemCount < 0) {
            throw new Errors_1.InvalidArgumentError("itemCount", "itemCount must be a non negative number");
        }
        for (let i = start; i < start + itemCount; i++) {
            yield i;
        }
    }
    /**
     * @summary Вставляет между элементами массива заданный элемент и возвращает новый массив
     */
    static intersperse(items, item) {
        return this.intersperseFn(items, () => item);
    }
    /**
     * @summary Вставляет между элементами массива элемент, создаваемый функцией, и возвращает новый массив
     */
    static intersperseFn(items, item) {
        return items.flatMap((x, index) => (index === 0 ? [x] : [item(x, index), x]));
    }
    static *zip(...iterables) {
        const iterators = iterables.map(it => Iterator.from(it));
        let hasUndoneIterators = true;
        while (hasUndoneIterators) {
            const resultEntry = [];
            hasUndoneIterators = false;
            for (const [i, iterator] of this.iterateEntries(iterators)) {
                if (iterator === NoValueMarker) {
                    resultEntry.push(undefined);
                }
                else {
                    const resultValue = iterator.next();
                    if (resultValue.done) {
                        resultEntry.push(undefined);
                        iterators[i] = NoValueMarker;
                    }
                    else {
                        resultEntry.push(resultValue.value);
                        hasUndoneIterators = true;
                    }
                }
            }
            if (hasUndoneIterators) {
                yield resultEntry;
            }
        }
    }
    static *distinctBy(items, idSelector) {
        const set = new Set();
        for (const item of items) {
            const id = idSelector(item);
            if (!set.has(id)) {
                set.add(id);
                yield item;
            }
        }
    }
    static reduce(items, reducer, initialValue) {
        let index = 0;
        return Iterator.from(items).reduce((acc, item) => reducer(acc, item, index++), initialValue);
    }
    static *batch(items, batchSize) {
        const iterator = Iterator.from(items);
        const safeBatchSize = batchSize > 0 ? batchSize : 1;
        let done = false;
        while (!done) {
            const batch = [];
            while (batch.length < safeBatchSize) {
                const current = iterator.next();
                if (current.done) {
                    done = true;
                    break;
                }
                else {
                    batch.push(current.value);
                }
            }
            if (batch.length > 0) {
                yield batch;
            }
        }
    }
}
exports.IterUtils = IterUtils;
