"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelPathImpl = exports.PathTokens = void 0;
exports.isModelPath = isModelPath;
exports.convertToLegacyPath = convertToLegacyPath;
exports.convertToAbsoluteModelPath = convertToAbsoluteModelPath;
exports.convertToRelativeModelPath = convertToRelativeModelPath;
exports.emptyModelPath = emptyModelPath;
exports.emptyAbsoluteModelPath = emptyAbsoluteModelPath;
exports.createModelPath = createModelPath;
exports.createFromMask = createFromMask;
exports.createAbsoluteFromMask = createAbsoluteFromMask;
exports.createRelativeFromMask = createRelativeFromMask;
exports.createAbsolute = createAbsolute;
exports.createRelativeResolvedModelPath = createRelativeResolvedModelPath;
exports.createAbsoluteFromTokens = createAbsoluteFromTokens;
exports.createAbsoluteResolvedFromTokens = createAbsoluteResolvedFromTokens;
exports.createRelativeFromTokens = createRelativeFromTokens;
exports.joinModelPaths = joinModelPaths;
const IterableUtils_1 = require("../IterableUtils");
const TypingUtils_1 = require("../TypingUtils");
const EachCurrentCollision_1 = require("./EachCurrentCollision");
const ModelPathBuilder_1 = require("./ModelPathBuilder");
const PathMatcher_1 = require("./PathMatcher");
const EachSymbol = Symbol.for("__EACH__");
const CurrentSymbol = Symbol.for("__CURRENT__");
class PathTokens {
    static each = EachSymbol;
    static current = CurrentSymbol;
    static isMultiToken(part) {
        return part === PathTokens.each || part === PathTokens.current;
    }
    static isSimpleToken(part) {
        return typeof part === "string";
    }
    static isInstanceToken(part) {
        return typeof part === "string" && /^\d+$/.test(part);
    }
    static isInstanceTokenOrInstancesSpec(part) {
        return typeof part === "string" && (/^\d+$/.test(part) || /^\[(\d+\,?)+\]$/i.test(part));
    }
}
exports.PathTokens = PathTokens;
function isModelPath(path) {
    return typeof path !== "string" && path instanceof ModelPathImpl;
}
function convertToLegacyPath(path) {
    return isModelPath(path) ? path.toLegacyPath() : path;
}
function convertToAbsoluteModelPath(path) {
    return isModelPath(path) ? path : createAbsoluteFromMask(path, PathTokens.each);
}
function convertToRelativeModelPath(path) {
    return isModelPath(path) ? path : createRelativeFromMask(path, PathTokens.each);
}
function emptyModelPath() {
    return new ModelPathImpl([], false);
}
function emptyAbsoluteModelPath() {
    return new ModelPathImpl([], true);
}
function createModelPath(path, absolute) {
    const partPartStrings = path.split("/");
    ensurePlainPathPartsDoesNotContainMultiplicity(partPartStrings);
    if (absolute === "auto") {
        if (path.startsWith("/")) {
            return new ModelPathImpl(partPartStrings.slice(1), true);
        }
        else {
            return new ModelPathImpl(partPartStrings, false);
        }
    }
    if (absolute && path.startsWith("/")) {
        return new ModelPathImpl(partPartStrings.slice(1), true);
    }
    return new ModelPathImpl(partPartStrings, absolute);
}
function createFromMask(path, absolute, multiplicitySymbol) {
    const partPartStrings = path.split("/").map(x => (x === "*" ? multiplicitySymbol : x));
    if (absolute === "auto") {
        if (path.startsWith("/")) {
            return new ModelPathImpl(partPartStrings.slice(1), true);
        }
        else {
            return new ModelPathImpl(partPartStrings, false);
        }
    }
    if (absolute && path.startsWith("/")) {
        return new ModelPathImpl(partPartStrings.slice(1), true);
    }
    return new ModelPathImpl(partPartStrings, absolute);
}
function createAbsoluteFromMask(path, multiplicitySymbol) {
    const partPartStrings = path.split("/").map(x => (x === "*" ? multiplicitySymbol : x));
    if (path.startsWith("/")) {
        return new ModelPathImpl(partPartStrings.slice(1), true);
    }
    return new ModelPathImpl(partPartStrings, true);
}
function createRelativeFromMask(path, multiplicitySymbol) {
    const partPartStrings = path.split("/").map(x => (x === "*" ? multiplicitySymbol : x));
    return new ModelPathImpl(partPartStrings, false);
}
function ensurePlainPathPartsDoesNotContainMultiplicity(parts) {
    if (parts.some(x => x === "*")) {
        throw new Error("Path must not contain * symbol");
    }
}
function createAbsolute(path) {
    const partPartStrings = path.split("/");
    ensurePlainPathPartsDoesNotContainMultiplicity(partPartStrings);
    if (path.startsWith("/")) {
        return new ModelPathImpl(partPartStrings.slice(1), true);
    }
    return new ModelPathImpl(partPartStrings, true);
}
function createRelativeResolvedModelPath(path) {
    const partPartStrings = path.split("/");
    ensurePlainPathPartsDoesNotContainMultiplicity(partPartStrings);
    return new ModelPathImpl(partPartStrings, false);
}
function createAbsoluteFromTokens(tokens) {
    return new ModelPathImpl(tokens, true);
}
function createAbsoluteResolvedFromTokens(tokens) {
    ensurePlainPathPartsDoesNotContainMultiplicity(tokens);
    return new ModelPathImpl(tokens, true);
}
function createRelativeFromTokens(tokens) {
    return new ModelPathImpl(tokens, false);
}
function joinModelPaths(...parts) {
    let absolute = false;
    let resultParts = [];
    for (const joinItem of parts) {
        if (typeof joinItem === "string") {
            resultParts.push(joinItem);
        }
        else if (isModelPath(joinItem)) {
            if (joinItem.isAbsolute()) {
                absolute = true;
                resultParts = [...joinItem.getPathPartsAsArray()];
            }
            else {
                resultParts.push(...joinItem.getPathPartsAsArray());
            }
        }
        else if (Array.isArray(joinItem)) {
            resultParts.push(...joinItem);
        }
        else if (PathTokens.isMultiToken(joinItem)) {
            resultParts.push(joinItem);
        }
    }
    return new ModelPathImpl(resultParts, absolute);
}
function isAllTokensResolved(tokens) {
    // Я что-то немогу сообразить как тут всё строго написать...
    // @ts-expect-error
    return tokens.every(PathTokens.isSimpleToken);
}
class ModelPathImpl {
    tokens;
    absolute;
    legacyPath;
    cachedHash;
    constructor(pathParts, absolute) {
        this.tokens = pathParts.filter(x => x !== "");
        this.absolute = absolute;
    }
    get resolved() {
        return isAllTokensResolved(this.tokens);
    }
    getPathParts() {
        return this.tokens;
    }
    getPathPartsAsArray() {
        return this.tokens;
    }
    *getSimpleTokensStrict() {
        for (const token of this.tokens) {
            if (!PathTokens.isSimpleToken(token)) {
                throw new Error("Path must not contain spec tokens");
            }
            yield token;
        }
    }
    getSimpleTokensAsArrayStrict() {
        for (const token of this.tokens) {
            if (!PathTokens.isSimpleToken(token)) {
                throw new Error("Path must not contain spec tokens");
            }
        }
        // Проверка выше гараентирует что будут только SimplePathToken.
        // Все map и filter убраны для производительности
        // @ts-ignore
        return this.tokens;
    }
    joinWith(...path) {
        return joinModelPaths(this, ...path);
    }
    getFirstMultiplicityInstanceNumber() {
        const firstIndex = this.tokens.find(PathTokens.isInstanceToken);
        return firstIndex ? Number(firstIndex) : -1;
    }
    isResolved() {
        return isAllTokensResolved(this.tokens);
    }
    resolveAll(instance) {
        const tokens = this.tokens.map(token => {
            if (PathTokens.isSimpleToken(token)) {
                return token;
            }
            else {
                return instance;
            }
        });
        return new ModelPathImpl(tokens, this.absolute);
    }
    isContainIteration() {
        return this.tokens.some(x => x === PathTokens.each);
    }
    isContainCurrentIteration() {
        return this.tokens.some(x => x === PathTokens.current);
    }
    isContainCurrentOrEachIteration() {
        return this.tokens.some(x => x === PathTokens.current || x === PathTokens.each);
    }
    getLastListPath() {
        if (!this.isContainIteration()) {
            throw new Error("Path must contain iteration");
        }
        // Проверка выше гарантирует, что будет PathTokens.each
        // @ts-ignore
        const lastIterationIndex = this.tokens.lastIndexOf(PathTokens.each);
        return this.tokens.slice(0, lastIterationIndex);
    }
    getPathWithoutLastIteration() {
        if (!this.isContainCurrentOrEachIteration()) {
            return this;
        }
        const lastIterationIndex = this.getLastIndexOfCurrentOrEachIteration();
        if (lastIterationIndex === -1) {
            throw new Error("Path must contain current or each iteration");
        }
        const tokensWithoutLastIteration = this.tokens.slice(0, lastIterationIndex);
        return new ModelPathImpl(tokensWithoutLastIteration, this.absolute);
    }
    getLastIndexOfCurrentOrEachIteration() {
        if (!this.isContainCurrentOrEachIteration()) {
            return -1;
        }
        // Проверка выше гарантирует, что будет PathTokens.each или PathTokens.current
        // @ts-ignore
        const lastIterationIndex = this.tokens.lastIndexOf(PathTokens.each);
        // @ts-ignore
        const lastCurrentIterationIndex = this.tokens.lastIndexOf(PathTokens.current);
        return Math.max(lastIterationIndex, lastCurrentIterationIndex);
    }
    isEndsWithIteration() {
        const lastItem = IterableUtils_1.IterUtils.last(this.tokens);
        return lastItem == PathTokens.each;
    }
    isEndsWithCurrentIteration() {
        const lastItem = IterableUtils_1.IterUtils.last(this.tokens);
        return lastItem == PathTokens.current;
    }
    isAbsolute() {
        return this.absolute;
    }
    toString() {
        if (this.tokens.length === 0) {
            return "";
        }
        return (this.absolute ? "/" : "") + this.tokens.map(x => this.pathPartToString(x)).join("/");
    }
    toLegacyPath() {
        return this.legacyPath ?? (this.legacyPath = this.tokens.map(x => this.pathPartToString(x)).join("/"));
    }
    isEquals(modelPath, eachCurrentCollision = EachCurrentCollision_1.EachCurrentCollision.AreSame) {
        if (eachCurrentCollision === EachCurrentCollision_1.EachCurrentCollision.AreSame) {
            return this.toLegacyPath() === modelPath.toLegacyPath();
        }
        else {
            return (this.isAbsolute() === modelPath.isAbsolute() &&
                Iterator.from(IterableUtils_1.IterUtils.zip(this.getPathParts(), modelPath.getPathParts())).every(([t1, t2]) => t1 === t2));
        }
    }
    /**
     * Всегда целое положительное беззнаковое 32-битное число
     */
    getHash(eachCurrentCollision = EachCurrentCollision_1.EachCurrentCollision.AreSame) {
        if (this.cachedHash == undefined) {
            let hash = 5381;
            for (const token of (0, IterableUtils_1.reverseArray)(this.tokens)) {
                let currentStr;
                if (PathTokens.isSimpleToken(token)) {
                    currentStr = token;
                }
                else {
                    if (eachCurrentCollision === EachCurrentCollision_1.EachCurrentCollision.AreDifferent) {
                        currentStr = token === PathTokens.each ? "*" : "!";
                    }
                    else {
                        currentStr = "*";
                    }
                }
                let stringIndex = currentStr.length;
                while (stringIndex) {
                    hash = (hash * 33) ^ currentStr.charCodeAt(--stringIndex);
                }
            }
            /*
                JavaScript does bitwise operations (like XOR, above) on 32-bit signed
                integers. Since we want the results to be always positive, convert the
                signed int to an unsigned by doing an unsigned bitshift.
          */
            this.cachedHash = hash >>> 0;
        }
        return this.cachedHash;
    }
    isIncludes(modelPath) {
        const targetTokens = modelPath.getPathPartsAsArray();
        if (this.tokens.length !== targetTokens.length) {
            return false;
        }
        for (let i = 0; i < this.tokens.length; i++) {
            const leftPart = this.tokens[i];
            const rightPart = targetTokens[i];
            if (typeof leftPart === "string") {
                if (typeof rightPart === "string") {
                    if (leftPart !== rightPart) {
                        return false;
                    }
                    continue;
                }
                else {
                    return false;
                }
            }
            else if (leftPart === PathTokens.current || leftPart === PathTokens.each) {
                if (typeof rightPart === "string") {
                    if (!isFinite(Number(rightPart))) {
                        return false;
                    }
                    continue;
                }
                else {
                    if (rightPart !== PathTokens.current && rightPart !== PathTokens.each) {
                        return false;
                    }
                    continue;
                }
            }
        }
        return true;
    }
    getIterationRank(eachCurrentCollision = EachCurrentCollision_1.EachCurrentCollision.AreDifferent) {
        return (0, IterableUtils_1.count)(Iterator.from(this.tokens).filter(x => eachCurrentCollision == EachCurrentCollision_1.EachCurrentCollision.AreDifferent
            ? x === PathTokens.each
            : PathTokens.isMultiToken(x)));
    }
    isEmpty() {
        return this.tokens.length === 0;
    }
    matchFor(modelPath) {
        if (!this.isAbsolute()) {
            throw new Error(`Cannot match non absolute path '${this.toString()}'`);
        }
        if (!modelPath.isAbsolute()) {
            throw new Error(`Cannot match non absolute path '${modelPath.toString()}'`);
        }
        return PathMatcher_1.PathMatcher.match(this.tokens, modelPath.getPathPartsAsArray());
    }
    startsWith(modelPath) {
        const thisParts = this.getPathPartsAsArray();
        const otherParts = modelPath.getPathPartsAsArray();
        return otherParts.every((value, index) => value === thisParts[index]);
    }
    endsWith(modelPath) {
        const thisParts = this.getPathPartsAsArray().slice().reverse();
        const otherParts = modelPath.getPathPartsAsArray().slice().reverse();
        return otherParts.every((value, index) => value === thisParts[index]);
    }
    *splitByMultiplicity() {
        let lastSimpleParts = new ModelPathBuilder_1.ModelPathBuilder(this.isAbsolute());
        for (const token of this.tokens) {
            if (PathTokens.isSimpleToken(token)) {
                lastSimpleParts.append(token);
            }
            else {
                if (!lastSimpleParts.empty) {
                    yield lastSimpleParts.toResolvedPath();
                    lastSimpleParts = new ModelPathBuilder_1.ModelPathBuilder(false);
                }
                yield token;
            }
        }
        if (!lastSimpleParts.empty) {
            yield lastSimpleParts.toResolvedPath();
        }
    }
    normalize() {
        const resultTokens = [];
        for (const pathPart of this.tokens) {
            if (pathPart === "..") {
                if (!resultTokens.pop()) {
                    if (this.isAbsolute()) {
                        throw new Error(`Path is invalid '${this.toString()}'`);
                    }
                    else {
                        resultTokens.push("..");
                    }
                }
                continue;
            }
            if (pathPart === ".") {
                continue;
            }
            resultTokens.push(pathPart);
        }
        return new ModelPathImpl(resultTokens, this.absolute);
    }
    /**
     * @summary Спецификаторы множественности Each заменяет на Current
     */
    toCurrentIteration() {
        if (this.isResolved()) {
            return this;
        }
        else {
            // NOTE Тут не получается наложить дискриминатор
            // @ts-expect-error
            const pathParts = this.tokens.map(x => x === PathTokens.each ? PathTokens.current : x);
            return new ModelPathImpl(pathParts, this.absolute);
        }
    }
    toEachIteration() {
        if (this.isResolved()) {
            return this;
        }
        else {
            // NOTE Тут не получается наложить дискриминатор
            // @ts-expect-error
            const pathParts = this.tokens.map(x => x === PathTokens.current ? PathTokens.each : x);
            return new ModelPathImpl(pathParts, this.absolute);
        }
    }
    toAbsolute() {
        if (!this.absolute) {
            throw new Error(`Path '${this.toLegacyPath()} is not absolute'`);
        }
        return new ModelPathImpl(this.tokens, true);
    }
    *getAllParentPaths() {
        const result = new ModelPathBuilder_1.ModelPathBuilder(this.isAbsolute());
        for (const token of IterableUtils_1.IterUtils.skipLast(this.getPathParts(), 1)) {
            result.append(token);
            yield result.toPath();
        }
    }
    trimLastStarIfLastToken() {
        const lastItem = IterableUtils_1.IterUtils.last(this.tokens);
        if (lastItem != undefined && PathTokens.isMultiToken(lastItem)) {
            return new ModelPathImpl(this.tokens.slice(0, -1), this.absolute);
        }
        return this;
    }
    trimFirstStarIfLastToken() {
        const firstItem = IterableUtils_1.IterUtils.first(this.tokens);
        if (firstItem != undefined && PathTokens.isMultiToken(firstItem)) {
            return new ModelPathImpl(this.tokens.slice(1), this.absolute);
        }
        return this;
    }
    getSuffixPathFrom(modelFieldPath) {
        return createRelativeFromTokens(modelFieldPath
            .matchFor(this)
            .map(x => (x.kind === PathMatcher_1.MatchKind.DifferentSuffix ? x.source : undefined))
            .filter(TypingUtils_1.isNotNullOrUndefined)
            .flatMap(x => x.getPathPartsAsArray()));
    }
    getParentPath() {
        const result = new ModelPathBuilder_1.ModelPathBuilder(this.absolute);
        for (const token of IterableUtils_1.IterUtils.skipLast(this.getPathParts(), 1)) {
            result.append(token);
        }
        // Тут получается так, что по другому никак
        // @ts-expect-error
        return result.toPath();
    }
    applyInstancesDirty(instances, fillWithMultiplicityIfHasNoInstance) {
        if (this.isResolved() || instances.length === 0) {
            return this;
        }
        let instanceCounter = 0;
        const result = new ModelPathBuilder_1.ModelPathBuilder(true);
        for (const token of this.getPathParts()) {
            if (PathTokens.isMultiToken(token) && instanceCounter < instances.length) {
                const instanceOrEach = instances[instanceCounter] ??
                    (fillWithMultiplicityIfHasNoInstance == true ? PathTokens.each : (0, TypingUtils_1.reject)());
                result.append(instanceOrEach);
                instanceCounter++;
            }
            else {
                result.append(token);
            }
        }
        // Тут получается так, что по другому никак
        // @ts-expect-error
        return result.toPath();
    }
    pathPartToString(pathPart) {
        if (typeof pathPart === "string") {
            return pathPart;
        }
        if (pathPart === PathTokens.each) {
            return "*";
        }
        if (pathPart === PathTokens.current) {
            return "*";
        }
        throw new Error(`Unknown path part specified: ${(0, TypingUtils_1.ensureAndGetString)(pathPart)}`);
    }
}
exports.ModelPathImpl = ModelPathImpl;
