"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.unitReducer = unitReducer;
const UnitStateSelect_1 = require("../../../../Helpers/StateAccessors/UnitStateSelect");
const ViewModelSelect_1 = require("../../../../Helpers/StateAccessors/ViewModelSelect");
const UnitId_1 = require("../UnitId/UnitId");
const UnitIdImmutableSet_1 = require("../UnitId/UnitIdImmutableSet");
const StringArrayKey_1 = require("../../../../../../Common/StringArrayKey");
const Map2_1 = require("../../../../../../Common/ModelPath/Map/Map2");
const unitActions_1 = require("./unitActions");
function unitReducer(state, action) {
    switch (action.type) {
        case unitActions_1.UnitActionType.UnitHighlight: {
            if ((0, UnitId_1.isUnitIdEquals)(action.payload.unitId, state.highlightUnit)) {
                return state;
            }
            const highlightUnitsInfo = state.highlightUnitsInfo ?? new Map2_1.Map2();
            if (action.payload.unitId.instance != undefined) {
                const key = new StringArrayKey_1.StringArrayKey([
                    action.payload.unitId.pageId,
                    ...(action.payload.unitId.parentInstances ?? []),
                ]);
                highlightUnitsInfo.set(key, action.payload.unitId.instance);
            }
            return { ...state, highlightUnit: action.payload.unitId, highlightUnitsInfo: highlightUnitsInfo };
        }
        case unitActions_1.UnitActionType.ShowModalUnit: {
            const { targetUnitId, modalUnitId } = action.payload;
            return {
                ...state,
                modalUnits: [
                    ...state.modalUnits.filter(x => x.targetUnitId.pageId !== targetUnitId.pageId),
                    {
                        targetUnitId: targetUnitId,
                        modalUnitId: modalUnitId,
                    },
                ],
            };
        }
        case unitActions_1.UnitActionType.CloseModalUnit: {
            const { targetUnitId } = action.payload;
            const index = state.modalUnits.findIndex(x => (0, UnitId_1.isUnitIdEquals)(x.targetUnitId, targetUnitId));
            return {
                ...state,
                modalUnits: [...state.modalUnits.slice(0, index), ...state.modalUnits.slice(index + 1)],
            };
        }
        case unitActions_1.UnitActionType.ReplaceModalUnit: {
            const { currentModalUnitId, nextModalUnitId } = action.payload;
            const index = state.modalUnits.findIndex(x => (0, UnitId_1.isUnitIdEquals)(x.modalUnitId, currentModalUnitId));
            const currentModalUnitInfo = state.modalUnits[index];
            if (currentModalUnitInfo == undefined) {
                return state;
            }
            return {
                ...state,
                modalUnits: [
                    ...state.modalUnits.slice(0, index),
                    {
                        ...currentModalUnitInfo,
                        modalUnitId: nextModalUnitId,
                    },
                    ...state.modalUnits.slice(index + 1),
                ],
            };
        }
        case unitActions_1.UnitActionType.InitUnitInfo: {
            const { unitInfos } = action.payload;
            return { ...state, unitInfos: unitInfos };
        }
        case unitActions_1.UnitActionType.SetUnitTableWidth: {
            return updateUnitWidthByTableWidth(state, action.payload);
        }
        case unitActions_1.UnitActionType.RemoveUnitTableWidth: {
            return updateUnitWidthByTableDeletion(state, action.payload);
        }
        case unitActions_1.UnitActionType.UpdateEmptyUnits: {
            const { unitId } = action.payload;
            const emptyUnits = action.payload.empty
                ? UnitIdImmutableSet_1.UnitIdSet.add(state.emptyUnits, unitId)
                : UnitIdImmutableSet_1.UnitIdSet.remove(state.emptyUnits, unitId);
            return { ...state, emptyUnits: emptyUnits };
        }
        case unitActions_1.UnitActionType.AddUnit: {
            const emptyUnits = UnitIdImmutableSet_1.UnitIdSet.remove(state.emptyUnits, action.payload.unitId);
            return { ...state, emptyUnits: emptyUnits };
        }
        case unitActions_1.UnitActionType.RemoveUnit: {
            const { unitId, index, bindingPath } = action.payload;
            let emptyUnits = state.emptyUnits;
            const children = ViewModelSelect_1.currentViewModelSelect.children(state, bindingPath);
            if (children.length === 0) {
                emptyUnits = UnitIdImmutableSet_1.UnitIdSet.add(emptyUnits, unitId);
            }
            let nextModalUnits = state.modalUnits;
            const modalUnitIndex = state.modalUnits.findIndex(x => x.modalUnitId.pageId === unitId.pageId && x.modalUnitId.instance === index);
            if (modalUnitIndex >= 0) {
                nextModalUnits = [
                    ...nextModalUnits.slice(0, modalUnitIndex),
                    ...nextModalUnits.slice(modalUnitIndex + 1),
                ];
            }
            return { ...state, emptyUnits: emptyUnits, modalUnits: nextModalUnits };
        }
        default:
            return state;
    }
}
function updateUnitWidthByTableWidth(state, { tableId, unitId, width }) {
    if (UnitStateSelect_1.UnitStateSelect.getUnitTableWidths(state, unitId)[tableId] === width) {
        return state;
    }
    const updatedState = UnitStateSelect_1.UnitStateSelect.setUnitTableWidth(state, unitId, tableId, width);
    return recalculateUnitWidthStateByNewState(state, unitId, updatedState);
}
function updateUnitWidthByTableDeletion(state, { tableId, unitId }) {
    const updatedState = UnitStateSelect_1.UnitStateSelect.deleteUnitTableWidth(state, unitId, tableId);
    return recalculateUnitWidthStateByNewState(state, unitId, updatedState);
}
function recalculateUnitWidthStateByNewState(oldState, unitId, newState) {
    const oldUnitWidth = UnitStateSelect_1.UnitStateSelect.getUnitWidth(oldState, unitId) || 0;
    const tablesWidthsCollection = UnitStateSelect_1.UnitStateSelect.getUnitTableWidths(newState, unitId);
    const tablesWidths = Object.values(tablesWidthsCollection);
    const isNoTables = tablesWidths.length === 0;
    if (isNoTables) {
        return UnitStateSelect_1.UnitStateSelect.deleteUnitWidth(newState, unitId);
    }
    const maxTableWidth = Math.max(...tablesWidths);
    if (maxTableWidth === oldUnitWidth) {
        return newState;
    }
    return UnitStateSelect_1.UnitStateSelect.setUnitWidth(newState, unitId, maxTableWidth);
}
