"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FiasApi = exports.maxSteads = exports.maxItems = void 0;
const utils_1 = require("../../../../Engine/utils");
const TypingUtils_1 = require("../../../../../../Common/TypingUtils");
const acceptConvertationHelper_1 = require("./Utils/Convertations/acceptConvertationHelper");
exports.maxItems = 6;
exports.maxSteads = 3;
class FiasApi {
    baseUrl;
    maxItems;
    maxSteadsCount;
    division;
    addressLevel;
    searchSteads;
    constructor(formUrl, maxItemsCount, division, addressLevel, searchSteads = false, maxSteadsCount = exports.maxSteads) {
        this.baseUrl = this.setApiUrl(formUrl);
        this.maxItems = maxItemsCount;
        this.maxSteadsCount = maxSteadsCount;
        this.division = division;
        this.addressLevel = addressLevel;
        this.searchSteads = searchSteads;
    }
    setApiUrl(url) {
        if (url.includes("testkontur") || url.includes("localhost")) {
            return "https://api.testkontur.ru/fias/v1.2";
        }
        else {
            return "https://api.kontur.ru/fias/v1.2";
        }
    }
    async fetchSteadsByTextAndParentFiasId(text, parentFiasId) {
        const steadNumber = this.extractSteadNumberFromText(text);
        const URLRequest = this.createSteadUrlBySteadNumber(steadNumber, parentFiasId);
        const response = await fetch(URLRequest);
        const steads = await response.json();
        return steads;
    }
    async fetchSuggestAddressesByText(text) {
        const URLRequest = this.createAddressUrlByStringAddress(text);
        const response = await fetch(URLRequest);
        const suggestAddresses = await response.json();
        const uniqAddresses = this.extractUniqAddresses(suggestAddresses);
        return uniqAddresses;
    }
    async fetchFullAddressesByFiasId(fiasId) {
        const URLRequest = this.createFullAddressUrlByFiasId(fiasId);
        const response = await fetch(URLRequest);
        if (response.status !== 200) {
            return {};
        }
        const address = await response.json();
        if ("error" in address) {
            return {};
        }
        return address;
    }
    async fetchFlatByParentFiasId(parentFiasId, number) {
        const URLRequest = this.createFullAddressUrlToFetchFlat(parentFiasId, number);
        const response = await fetch(URLRequest);
        const flat = await response.json();
        return flat;
    }
    async fetchCountriesByName(countryName) {
        const URLRequest = this.createFullAddressUrlToFetchCountry(countryName);
        const response = await fetch(URLRequest);
        const countries = await response.json();
        return countries;
    }
    async fetchRegions() {
        const URLRequest = this.createRegionsRequestUrl();
        const response = await fetch(URLRequest);
        const regions = await response.json();
        return regions;
    }
    convertAbbreviation(value, helper) {
        return (0, acceptConvertationHelper_1.acceptHelperConvertationToValue)(value, helper);
    }
    extractSteadNumberFromText(text) {
        const spitedAddress = text?.split(" ");
        const steadNumber = spitedAddress && spitedAddress[spitedAddress.length - 1];
        const prevSymbol = spitedAddress && Number(spitedAddress[spitedAddress.length - 2]);
        const isPrevSymbolNumber = !prevSymbol && isNaN(prevSymbol || NaN);
        if (steadNumber && steadNumber.trim() && isPrevSymbolNumber) {
            return steadNumber;
        }
        return "";
    }
    createAddressUrlByStringAddress(address) {
        const preparedAddress = this.prepareAddressBeforeRequest(address);
        return (this.baseUrl +
            `/addresses/search?address=${preparedAddress}&limit=${this.maxItems}&level=${this.addressLevel}&division=${this.division}&schema=gar&searchSteads=${this.searchSteads}`);
    }
    createSteadUrlBySteadNumber(steadNumber, parentFiasId) {
        return (this.baseUrl +
            `/steads?prefix=${steadNumber}&parentFiasId=${parentFiasId}&limit=${this.maxSteadsCount}&division=${this.division}`);
    }
    createRegionsRequestUrl() {
        return this.baseUrl + `/addresses/regions`;
    }
    createFullAddressUrlByFiasId(fiasId) {
        return this.baseUrl + `/addresses/structural/${fiasId}?division=${this.division}&schema=gar`;
    }
    createFullAddressUrlToFetchFlat(houseId, flatNum) {
        return this.baseUrl + `/rooms?prefix=${flatNum}&parentFiasId=${houseId}&limit=1&schema=gar`;
    }
    createFullAddressUrlToFetchCountry(country) {
        return this.baseUrl + `/countries?prefix=${country}&limit=${this.maxItems}`;
    }
    extractUniqAddresses(addresses) {
        return addresses.filter((value, index, self) => self.findIndex(address => address.suggest?.fiasId === value.suggest?.fiasId) === index);
    }
    prepareAddressBeforeRequest(address) {
        if ((0, TypingUtils_1.isNullOrEmpty)(address)) {
            return "";
        }
        if ((0, utils_1.detectIE)()) {
            return encodeURI(address);
        }
        return address.trim();
    }
}
exports.FiasApi = FiasApi;
