"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FilterBuilder = exports.resolveToAllFilter = exports.resolveToNoneFilter = void 0;
exports.isResolveToAllFilter = isResolveToAllFilter;
exports.isResolveToNoneFilter = isResolveToNoneFilter;
const PicklistTypes_1 = require("../../../../Engine/Core/Picklists/PicklistTypes");
const DigestQueryFilter_1 = require("../../../../../../Common/CandyApiClient/DigestApiClient/Types/DigestQueryFilter");
const TypingUtils_1 = require("../../../../../../Common/TypingUtils");
exports.resolveToNoneFilter = { type: "ResolveToNoneFilter" };
exports.resolveToAllFilter = { type: "ResolveToAllFilter" };
function isResolveToAllFilter(value) {
    return "type" in value && value.type === "ResolveToAllFilter";
}
function isResolveToNoneFilter(value) {
    return "type" in value && value.type === "ResolveToNoneFilter";
}
class FilterBuilder {
    output;
    stack;
    tempColumnName;
    tempValueOperand;
    constructor() {
        this.output = [];
        this.stack = [];
        this.tempColumnName = "";
        this.tempValueOperand = "";
    }
    column = (name) => {
        this.tempColumnName = name;
        return this;
    };
    eql = (value) => {
        this.tempValueOperand = value;
        this.addOperandToStack(DigestQueryFilter_1.DigestFilterOperator.Equal);
        this.clearTempDataOperand();
        return this;
    };
    eqlMultiple = (value) => {
        if ((0, TypingUtils_1.isNullOrEmpty)(value)) {
            return this.eql(value);
        }
        try {
            const array = JSON.parse(value);
            if (array.length === 0) {
                return this.eql(value);
            }
            array.forEach((item, index) => {
                this.tempValueOperand = item;
                this.addOperandToStack(DigestQueryFilter_1.DigestFilterOperator.Equal);
                if (index !== array.length - 1) {
                    this.stack.push(DigestQueryFilter_1.DigestLogicalOperator.Or);
                }
            });
        }
        catch (e) {
            return this.eql(value);
        }
        this.clearTempDataOperand();
        return this;
    };
    contains = (value) => {
        this.tempValueOperand = value;
        this.addOperandToStack(DigestQueryFilter_1.DigestFilterOperator.Substring);
        this.clearTempDataOperand();
        return this;
    };
    includes = (value) => {
        this.tempValueOperand = value;
        this.addOperandToStack(DigestQueryFilter_1.DigestFilterOperator.ContainsInList);
        this.clearTempDataOperand();
        return this;
    };
    startWith = (value) => {
        this.tempValueOperand = value;
        this.addOperandToStack(DigestQueryFilter_1.DigestFilterOperator.Prefix);
        this.clearTempDataOperand();
        return this;
    };
    in = (value) => {
        if (!Array.isArray(value)) {
            throw new Error(`Expected array, got ${typeof value}`);
        }
        const filter = this.formFilterFromArray(value);
        this.output.push(filter);
        this.clearTempDataOperand();
        return this;
    };
    get and() {
        const stack = this.stack;
        const output = this.output;
        const lastOperator = stack[stack.length - 1];
        if (lastOperator === DigestQueryFilter_1.DigestLogicalOperator.Or) {
            stack.push(DigestQueryFilter_1.DigestLogicalOperator.And);
        }
        else if (lastOperator === DigestQueryFilter_1.DigestLogicalOperator.And) {
            output.push(DigestQueryFilter_1.DigestLogicalOperator.And);
        }
        else {
            stack.push(DigestQueryFilter_1.DigestLogicalOperator.And);
        }
        return this;
    }
    get or() {
        const stack = this.stack;
        const output = this.output;
        const lastOperator = stack[stack.length - 1];
        if (lastOperator === DigestQueryFilter_1.DigestLogicalOperator.And) {
            const lastElement = stack.pop();
            if (lastElement !== undefined) {
                output.push(lastElement);
            }
            stack.push(DigestQueryFilter_1.DigestLogicalOperator.Or);
        }
        else if (lastOperator === DigestQueryFilter_1.DigestLogicalOperator.Or) {
            output.push(DigestQueryFilter_1.DigestLogicalOperator.Or);
        }
        else {
            stack.push(DigestQueryFilter_1.DigestLogicalOperator.Or);
        }
        return this;
    }
    build() {
        const output = [...this.output, ...this.stack.reverse()];
        const buildResult = this.buildFilter(output)[0];
        return buildResult != undefined && (0, PicklistTypes_1.isPicklistFilter)(buildResult) ? buildResult : undefined;
    }
    buildFilter(output, index = 0) {
        const operator = output[index];
        if (operator != undefined && (0, PicklistTypes_1.isPicklistLogicalOperator)(operator)) {
            const operands = output.slice(index - 2, index).filter(PicklistTypes_1.isPicklistFilter);
            const items = {
                operator: operator,
                operands: operands,
            };
            output.splice(index - 2, 3, items);
            return this.buildFilter(output, index - 1);
        }
        return output.length === 1 ? output : this.buildFilter(output, index + 1);
    }
    addOperandToStack(operator) {
        this.output.push({
            operator: operator,
            column: this.tempColumnName,
            value: this.tempValueOperand,
        });
    }
    clearTempDataOperand() {
        this.tempColumnName = "";
        this.tempValueOperand = "";
    }
    formFilterFromArray(value) {
        return {
            operator: DigestQueryFilter_1.DigestLogicalOperator.Or,
            operands: value.map(item => ({
                operator: DigestQueryFilter_1.DigestFilterOperator.Equal,
                column: this.tempColumnName,
                value: item,
            })),
        };
    }
}
exports.FilterBuilder = FilterBuilder;
