"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Calculator = void 0;
const areEqual_1 = __importDefault(require("fbjs/lib/areEqual"));
const lodash_uniqwith_1 = __importDefault(require("lodash.uniqwith"));
const IterableUtils_1 = require("../../../../../Common/IterableUtils");
const CoreTypes_1 = require("../CoreTypes");
const ModelEvent_1 = require("../EventManagement/ModelEvents/ModelEvent");
const MaskRegistrator_1 = require("../PathUtils/MaskRegistrator");
const MaskResolver_1 = require("../PathUtils/MaskResolver");
const PathUtils_1 = require("../PathUtils/PathUtils");
const OperandsGetter_1 = require("./OperandsGetter");
class Calculator {
    autocalcs;
    autocalcHooks;
    hooksMasks;
    argsMasksCache;
    dataDeclarationHelper;
    calculationFunctionCallContextProvider;
    sections;
    errorManager;
    constructor(calculationFunctionCallContextProvider, dataDeclarationHelper, sections, errorManager, calculationFunctions, calculationHooks) {
        this.calculationFunctionCallContextProvider = calculationFunctionCallContextProvider;
        this.sections = sections;
        this.errorManager = errorManager;
        this.dataDeclarationHelper = dataDeclarationHelper;
        this.autocalcs = calculationFunctions ?? {};
        this.autocalcHooks = calculationHooks ?? {};
        this.hooksMasks = new MaskRegistrator_1.MaskRegistrator();
        this.argsMasksCache = {};
        for (const hookPathMask of (0, IterableUtils_1.iterateKeys)(this.autocalcHooks)) {
            const [nodePath, valueName] = PathUtils_1.PathUtils.splitToPathAndValueName(hookPathMask);
            this.hooksMasks.registerMask(nodePath, valueName);
        }
    }
    loadFunctions(calculationHooks, calculationFunctions) {
        this.autocalcs = calculationFunctions ?? {};
        this.autocalcHooks = calculationHooks ?? {};
        this.hooksMasks = new MaskRegistrator_1.MaskRegistrator();
        this.argsMasksCache = {};
        for (const hookPathMask of (0, IterableUtils_1.iterateKeys)(this.autocalcHooks)) {
            const [nodePath, valueName] = PathUtils_1.PathUtils.splitToPathAndValueName(hookPathMask);
            this.hooksMasks.registerMask(nodePath, valueName);
        }
    }
    getArgumentsPaths(model, calcPath) {
        const calcMask = this.dataDeclarationHelper.getMask(calcPath);
        const dependsOnMasks = calcMask != undefined ? this.getDependsOnMasks(calcMask) : [];
        return dependsOnMasks
            .map(mask => {
            const maskWithResolvedInstances = PathUtils_1.PathUtils.resolveMaskToDeepestCommonInstance(calcPath, mask);
            const dependsOnPaths = MaskResolver_1.MaskResolver.resolveMask(model, maskWithResolvedInstances ?? mask);
            return dependsOnPaths;
        })
            .flat();
    }
    calculateAll(model, initEvents) {
        let nextEventsCalculate = initEvents.filter(ModelEvent_1.isChangeModelEventInfo);
        while (nextEventsCalculate.length > 0) {
            const calculatedEvents = this.calculateEvents(model, nextEventsCalculate);
            nextEventsCalculate = calculatedEvents.filter(ModelEvent_1.isChangeModelEventInfo);
        }
    }
    calculate(model, eventArgs) {
        let eventInfosToProcess = [eventArgs].filter(x => x.name != undefined);
        while (eventInfosToProcess.length > 0) {
            const hookContexts = this.createHooksContextSeparated(eventInfosToProcess);
            const eventsContext = model.beginGetEvents();
            for (const hook of (0, lodash_uniqwith_1.default)(hookContexts, areEqual_1.default)) {
                this.calculateHook(model, hook);
            }
            eventInfosToProcess = eventsContext.releaseAndGetEvent().filter(x => x.name != undefined);
        }
    }
    calculateEvents(model, changeEvents) {
        const fullMasks = this.findAllMasksByEvents(changeEvents);
        const hooksToExecute = this.createMaskToHookMap(fullMasks);
        const hooksContext = this.createHooksContext(hooksToExecute);
        const eventsContext = model.beginGetEvents();
        for (const hook of (0, lodash_uniqwith_1.default)(hooksContext, areEqual_1.default)) {
            this.calculateHook(model, hook);
        }
        return eventsContext.releaseAndGetEvent();
    }
    createHooksContextSeparated(eventsToProcess) {
        const result = [];
        for (const eventInfo of eventsToProcess) {
            const mask = this.findMaskByEvent(eventInfo);
            if (mask == undefined) {
                continue;
            }
            const hooks = this.autocalcHooks[mask];
            if (hooks == undefined) {
                continue;
            }
            for (const hook of hooks) {
                result.push(this.createHookExecutionContext(hook, mask, eventInfo));
            }
        }
        return result;
    }
    createHooksContext(hooksMap) {
        const result = [];
        for (const [mask, hookMap] of (0, IterableUtils_1.iterateEntries)(hooksMap)) {
            result.push(...hookMap.map(hook => this.createHookExecutionContext(hook, mask, undefined)));
        }
        return result;
    }
    createHookExecutionContext(hook, hooksMask, eventArgs) {
        return {
            eventArgs: eventArgs,
            fn: hook.fn,
            basedMask: PathUtils_1.PathUtils.getAbsolutePath(hooksMask, hook.base),
        };
    }
    createMaskToHookMap(fullMasks) {
        const filteredHooks = {};
        for (const mask of fullMasks) {
            const autocalcHook = this.autocalcHooks[mask];
            if (autocalcHook != undefined) {
                filteredHooks[mask] = autocalcHook;
            }
        }
        return filteredHooks;
    }
    findMaskByEvent(changeEvent) {
        if (changeEvent.fullPath == undefined) {
            throw new Error("InvalidProgramState");
        }
        return this.hooksMasks.getMasks(changeEvent.path, changeEvent.name)[0];
    }
    findAllMasksByEvents(changeEvents) {
        const result = [];
        for (const changeEvent of changeEvents) {
            const mask = this.findMaskByEvent(changeEvent);
            if (mask != undefined && !result.includes(mask)) {
                result.push(mask);
            }
        }
        return result;
    }
    calculateHook(model, hookContext) {
        const fullPath = hookContext.eventArgs?.fullPath;
        const pathByMask = this.getPathPartForMask(hookContext.basedMask, fullPath);
        if (pathByMask == undefined) {
            const basePaths = MaskResolver_1.MaskResolver.resolveMask(model, hookContext.basedMask);
            for (const path of basePaths) {
                this.executeHookFunctions(model, path, hookContext);
            }
        }
        else {
            this.executeHookFunctions(model, pathByMask, hookContext);
        }
    }
    getPathPartForMask(mask, path) {
        if (typeof mask !== "string" || typeof path !== "string") {
            return undefined;
        }
        if (mask === "") {
            return "";
        }
        const maskValues = PathUtils_1.PathUtils.split(mask);
        const pathValues = PathUtils_1.PathUtils.split(path).slice(0, maskValues.length);
        const isSame = (maskValue, i) => maskValue === "*" ? !isNaN(Number(pathValues[i])) : maskValue === pathValues[i];
        if (maskValues.every(isSame)) {
            return PathUtils_1.PathUtils.join(pathValues);
        }
        return undefined;
    }
    executeHookFunctions(model, path, hook) {
        const operandsGetter = new OperandsGetter_1.OperandsGetter(model, this.dataDeclarationHelper, path, hook.eventArgs);
        const autocalcFunction = this.autocalcs[hook.fn];
        if (autocalcFunction == undefined) {
            throw new Error(`Calculation hook function ${hook.fn} not found.`);
        }
        const calculationResults = autocalcFunction.call(this.calculationFunctionCallContextProvider.getCalculationFunctionCallContext(model), operandsGetter);
        for (const calculationResult of calculationResults) {
            this.processCalculationResult(model, calculationResult, path);
        }
    }
    getDependsOnMasks(calcMask) {
        if (this.argsMasksCache[calcMask] == undefined) {
            this.buildArgumentsByMaskMap();
        }
        return this.argsMasksCache[calcMask] || [];
    }
    buildArgumentsByMaskMap() {
        for (const [argumentPath, autocalcHooks] of (0, IterableUtils_1.iterateEntries)(this.autocalcHooks)) {
            if (autocalcHooks == undefined) {
                continue;
            }
            const [argNodePath] = PathUtils_1.PathUtils.splitToPathAndValueName(argumentPath);
            for (const hook of autocalcHooks) {
                if (hook.res === undefined) {
                    continue;
                }
                const basePath = PathUtils_1.PathUtils.getAbsolutePath(argumentPath, hook.base);
                const resPath = PathUtils_1.PathUtils.getAbsolutePath(basePath, hook.res);
                if (!this.argsMasksCache[resPath]) {
                    this.argsMasksCache[resPath] = [];
                }
                if (!this.argsMasksCache[resPath]?.includes(argNodePath)) {
                    this.argsMasksCache[resPath]?.push(argNodePath);
                }
            }
        }
    }
    processCalculationResult(model, [path, fieldName, value], basePath) {
        const absolutePath = PathUtils_1.PathUtils.getAbsolutePath(basePath, path);
        if (this.sections.isIgnoredPathByMasks(absolutePath)) {
            return;
        }
        if (fieldName === "error") {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-argument
            this.errorManager.addErrorCached(model, absolutePath, value);
        }
        model.setViewModelField(absolutePath, (0, CoreTypes_1.nonRegularFieldName)(fieldName), value);
    }
}
exports.Calculator = Calculator;
