"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Calculator2 = void 0;
const IterableUtils_1 = require("../../../../../Common/IterableUtils");
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const ModelEvent_1 = require("../EventManagement/ModelEvents/ModelEvent");
const Funcs_1 = require("../../../../../Common/Funcs");
const MaskResolver_1 = require("../PathUtils/MaskResolver");
const PathUtils_1 = require("../PathUtils/PathUtils");
const PathMatcher_1 = require("../../../../../Common/ModelPath/PathMatcher");
const Errors_1 = require("../../../../../Common/Errors");
const Map2_1 = require("../../../../../Common/ModelPath/Map/Map2");
const Set2_1 = require("../../../../../Common/ModelPath/Set/Set2");
const ModelPathHelper_1 = require("../../../../../Common/ModelPath/ModelPathHelper");
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const EachCurrentCollision_1 = require("../../../../../Common/ModelPath/EachCurrentCollision");
const AggregatedValidationFunction_1 = require("./AggregatedValidationFunction");
const CalculationTarget_1 = require("./CalculationTarget");
const EvaluationContext_1 = require("./EvaluationContext");
const LegacyValidationFunctionAdapter_1 = require("./Validations/LegacyValidationFunctionAdapter");
const ValidationFunction_1 = require("./Validations/ValidationFunction");
const CalculationFunctionDependency_1 = require("./CalculationFunctionDependency");
class Calculator2 {
    calculationFunctions = [];
    picklistsProvider;
    ignoredPathChecker;
    errorManager;
    dataDeclarationHelper;
    cachedNextFunctions;
    externalPropertiesProvider;
    constructor(errorManager, dataDeclarationHelper, ignoredPathChecker, picklistsProvider, externalPropertiesProvider) {
        this.externalPropertiesProvider = externalPropertiesProvider;
        this.picklistsProvider = picklistsProvider;
        this.ignoredPathChecker = ignoredPathChecker;
        this.dataDeclarationHelper = dataDeclarationHelper;
        this.errorManager = errorManager;
    }
    clearFunctions() {
        this.calculationFunctions = [];
        this.dropCache();
    }
    getFunctionsByTarget(path) {
        return this.calculationFunctions.filter(calculationFunction => calculationFunction
            .getTarget()
            .modelPath.matchFor(path)
            .every(match => match.kind !== PathMatcher_1.MatchKind.DifferentSuffix));
    }
    addCalculationFunction(calculationFunction) {
        if (calculationFunction instanceof ValidationFunction_1.ValidationFunction ||
            calculationFunction instanceof LegacyValidationFunctionAdapter_1.LegacyValidationFunctionAdapter) {
            let aggregatedByTargetFunction = this.calculationFunctions.find((x) => x instanceof AggregatedValidationFunction_1.AggregatedValidationFunction && x.getTarget().isEquals(calculationFunction.getTarget()));
            if (aggregatedByTargetFunction == undefined) {
                aggregatedByTargetFunction = new AggregatedValidationFunction_1.AggregatedValidationFunction(calculationFunction.getTarget());
                this.calculationFunctions.push(aggregatedByTargetFunction);
            }
            aggregatedByTargetFunction.addValidationFunction(calculationFunction);
        }
        else {
            this.calculationFunctions.push(calculationFunction);
        }
        this.dropCache();
    }
    async processEvents(model, modelChangeEventInfos) {
        this.ensurePrepared();
        for (const x of modelChangeEventInfos) {
            await this.processEvent(model, x, fn => true);
        }
    }
    async calculateFunctions(model, acceptFunction) {
        const orderedCalculationsFunctions = this.getAllOrderedFunctions();
        for (const calculationFunction of orderedCalculationsFunctions) {
            if (acceptFunction(calculationFunction)) {
                const executionPlan = calculationFunction.getFullPlan();
                const context = this.createEvaluationContext(model);
                await executionPlan.execute(context);
            }
        }
    }
    async calculateAll(model, acceptTarget, acceptFunction) {
        const orderedCalculationsFunctions = this.getAllOrderedFunctions()
            .filter(acceptFunction ?? Funcs_1.truePredicate)
            .filter(calculationFunction => acceptTarget(calculationFunction.getTarget().modelPath.toAbsolute()));
        for (const calculationFunction of orderedCalculationsFunctions) {
            const executionPlan = calculationFunction.getFullPlan();
            const context = this.createEvaluationContext(model);
            await executionPlan.execute(context);
        }
    }
    async revalidateTargets(model, targetPaths) {
        const executionPlans = [];
        for (const targetPath of targetPaths) {
            for (const calculationFunction of this.calculationFunctions) {
                if (calculationFunction.getTarget().targetAttribute !== CalculationTarget_1.CalculationTargetAttribute.Validation) {
                    continue;
                }
                if (calculationFunction.getTarget().modelPath.isIncludes(targetPath)) {
                    executionPlans.push(calculationFunction.getPlanForTarget(targetPath));
                    continue;
                }
            }
        }
        for (const executionPlan of executionPlans) {
            const context = this.createEvaluationContext(model);
            await executionPlan.execute(context);
        }
    }
    async recalculateTargets(model, targetLegacyPaths) {
        const targetPaths = Iterator.from(targetLegacyPaths).map(x => (0, ModelPath_1.createFromMask)(x, true, ModelPath_1.PathTokens.each));
        const executionPlans = [];
        for (const targetPath of targetPaths) {
            for (const calculationFunction of this.calculationFunctions) {
                if (calculationFunction.getTarget().modelPath.isIncludes(targetPath)) {
                    executionPlans.push(calculationFunction.getPlanForTarget(targetPath));
                    continue;
                }
            }
        }
        for (const executionPlan of executionPlans) {
            const context = this.createEvaluationContext(model);
            await executionPlan.execute(context);
        }
    }
    async processEvent(model, modelChangeEventInfo, acceptFunction) {
        const changeModelPath = modelChangeEventInfo.path;
        const orderedCalculationsFunctions = Iterator.from(this.getOrderedFunctions(new CalculationTarget_1.CalculationTarget(changeModelPath, CalculationTarget_1.CalculationTargetAttribute.Value))).filter(acceptFunction);
        const allEventBatches = [];
        const addedChildren = this.getAddedChildren(modelChangeEventInfo);
        const addedChildrenInfo = addedChildren != undefined
            ? { path: modelChangeEventInfo.path, instancesToIterate: addedChildren }
            : undefined;
        for (const calculationFunction of orderedCalculationsFunctions) {
            const allModelValueChangeEvents = (0, IterableUtils_1.concat)([modelChangeEventInfo], Iterator.from(allEventBatches)
                .flatMap(x => x)
                .filter(ModelEvent_1.isChangeViewModelEvent));
            const executionPlans = this.getExecutionPlan(calculationFunction, allModelValueChangeEvents, calculationFunction.pathIsOutsideDependenciesList(changeModelPath), addedChildrenInfo);
            if (executionPlans == undefined) {
                continue;
            }
            const eventCollectionContext = model.beginGetEvents();
            const eventsForContext = (0, IterableUtils_1.concat)([modelChangeEventInfo], Iterator.from(allEventBatches)
                .flatMap(x => x)
                .filter(ModelEvent_1.isChangeViewModelEvent));
            const context = this.createEvaluationContext(model, eventsForContext);
            for (const executionPlan of executionPlans) {
                const result = executionPlan.execute(context);
                if (result != undefined) {
                    await result;
                }
            }
            const nextEvents = eventCollectionContext.releaseAndGetEvent();
            allEventBatches.push(nextEvents);
        }
    }
    processEventSync(model, modelChangeEventInfo, acceptFunction) {
        const changeModelPath = modelChangeEventInfo.path;
        const orderedCalculationsFunctions = Iterator.from(this.getOrderedFunctions(new CalculationTarget_1.CalculationTarget(changeModelPath, CalculationTarget_1.CalculationTargetAttribute.Value))).filter(acceptFunction);
        const allEventBatches = [];
        const addedChildren = this.getAddedChildren(modelChangeEventInfo);
        const addedChildrenInfo = addedChildren != undefined
            ? { path: modelChangeEventInfo.path, instancesToIterate: addedChildren }
            : undefined;
        for (const calculationFunction of orderedCalculationsFunctions) {
            const allModelValueChangeEvents = (0, IterableUtils_1.concat)([modelChangeEventInfo], Iterator.from(allEventBatches)
                .flatMap(x => x)
                .filter(ModelEvent_1.isChangeViewModelEvent));
            const executionPlans = this.getExecutionPlan(calculationFunction, allModelValueChangeEvents, calculationFunction.pathIsOutsideDependenciesList(changeModelPath), addedChildrenInfo, true);
            if (executionPlans == undefined) {
                continue;
            }
            const eventCollectionContext = model.beginGetEvents();
            const eventsForContext = (0, IterableUtils_1.concat)([modelChangeEventInfo], Iterator.from(allEventBatches)
                .flatMap(x => x)
                .filter(ModelEvent_1.isChangeViewModelEvent));
            const context = this.createEvaluationContext(model, eventsForContext);
            for (const executionPlan of executionPlans) {
                const result = executionPlan.execute(context);
                if (result != undefined) {
                    throw new Errors_1.InvalidProgramStateError();
                }
            }
            const nextEvents = eventCollectionContext.releaseAndGetEvent();
            allEventBatches.push(nextEvents);
        }
    }
    process(model, eventArg, acceptFunction) {
        return this.processEvent(model, eventArg, acceptFunction ?? (fn => true));
    }
    processSync(model, eventArg, acceptFunction) {
        this.processEventSync(model, eventArg, acceptFunction ?? (fn => true));
    }
    getAddedChildren(modelChangeEventInfo) {
        if (modelChangeEventInfo.name === "children") {
            const value = Array.isArray(modelChangeEventInfo.value) ? modelChangeEventInfo.value : [];
            const oldValue = Array.isArray(modelChangeEventInfo.oldValue) ? modelChangeEventInfo.oldValue : [];
            const oldValueSet = new Set(oldValue);
            const addedChildren = value.filter((x) => !oldValueSet.has(x));
            return addedChildren;
        }
        return undefined;
    }
    getExecutionPlan(calculationFunction, allModelValueChangeEvents, changedPathIsOutsideFunctionDependencies, iterationData, sync) {
        const targetPaths = new Set2_1.Set2();
        let targetPathIterationRank = -1;
        for (const modelEvent of allModelValueChangeEvents) {
            const dependencies = calculationFunction.getDependencies();
            for (const dependency of dependencies) {
                let currentTargetPath;
                if (modelEvent.name === "value") {
                    currentTargetPath = dependency.getTargetByChangedPath(modelEvent.path);
                }
                if (modelEvent.name === "children") {
                    currentTargetPath = dependency.getTargetByChangedChildrenPath(modelEvent.path);
                }
                if (currentTargetPath != undefined) {
                    const currentTargetPathRank = currentTargetPath.getIterationRank();
                    if (currentTargetPathRank === targetPathIterationRank && changedPathIsOutsideFunctionDependencies) {
                        targetPaths.add(currentTargetPath);
                    }
                    if (currentTargetPathRank > targetPathIterationRank) {
                        targetPaths.add(currentTargetPath);
                        targetPathIterationRank = currentTargetPathRank;
                    }
                }
            }
            if (dependencies.length === 0 && modelEvent.name === "children") {
                const tempDependency = new CalculationFunctionDependency_1.CalculationFunctionDependency(calculationFunction.getTarget().modelPath, (0, ModelPath_1.createAbsoluteFromTokens)(calculationFunction.getTarget().modelPath.getLastListPath()));
                const currentTargetPath = tempDependency.getTargetByChangedChildrenPath(modelEvent.path);
                if (currentTargetPath != undefined) {
                    targetPaths.add(currentTargetPath);
                }
            }
        }
        return targetPaths.isEmpty()
            ? undefined
            : Iterator.from(targetPaths.values())
                .map(x => calculationFunction.getPlanForTarget(x, iterationData, sync))
                .toArray();
    }
    dfs(calculationTarget, visited, fns, fn) {
        if (visited.find(x => x.isEquals(calculationTarget))) {
            return;
        }
        const deps = this.calculationFunctions.filter(x => x.isDependsOn(calculationTarget));
        if (deps.length > 0) {
            for (const dep of deps) {
                this.dfs(dep.getTarget(), visited, fns, dep);
            }
        }
        if (fn != undefined) {
            fns.unshift(fn);
        }
        visited.push(calculationTarget);
    }
    linkTargetNodeByDependencyToNode(targetMap, dependencyTarget, currentNode) {
        const targetNode = targetMap.get(dependencyTarget);
        if (targetNode != undefined) {
            targetNode.nextNodes = [
                ...IterableUtils_1.IterUtils.distinctBy([...targetNode.nextNodes, currentNode], x => x.calculationFunction.getTarget().toString()),
            ];
        }
    }
    addToGraph(dependencyMap, targetMap, calculationFunction, functionId) {
        const dependentNodes = dependencyMap.get(calculationFunction.getTarget()) ?? [];
        const currentNode = {
            calculationFunction: calculationFunction,
            nextNodes: dependentNodes,
            id: functionId,
        };
        const dependencyTargets = IterableUtils_1.IterUtils.distinctBy(calculationFunction
            .getDependencies()
            .map(dep => new CalculationTarget_1.CalculationTarget(dep.getDependencyPath(), CalculationTarget_1.CalculationTargetAttribute.Value)), x => x.toString());
        for (const dependencyTarget of dependencyTargets) {
            dependencyMap.setOrUpdate(dependencyTarget, __ => [currentNode], (item, value) => [...value, currentNode]);
            this.linkTargetNodeByDependencyToNode(targetMap, dependencyTarget, currentNode);
        }
        targetMap.set(calculationFunction.getTarget(), currentNode);
        const modelPath = calculationFunction.getTarget().modelPath;
        const modelPathIterationRank = modelPath.getIterationRank(EachCurrentCollision_1.EachCurrentCollision.AreSame);
        const isRequiredToAddDependencyForMultiplicity = modelPathIterationRank > 0 &&
            ((0, TypingUtils_1.arrayHasLength)(calculationFunction.getDependencies(), 0) ||
                calculationFunction
                    .getDependencies()
                    .some(dep => dep.getDependencyPath().getIterationRank(EachCurrentCollision_1.EachCurrentCollision.AreSame) !=
                    modelPathIterationRank));
        if (isRequiredToAddDependencyForMultiplicity) {
            dependencyMap.setOrUpdate(new CalculationTarget_1.CalculationTarget((0, ModelPath_1.createAbsoluteFromTokens)(modelPath.getLastListPath()), CalculationTarget_1.CalculationTargetAttribute.Value), __ => [currentNode], (item, value) => [...value, currentNode]);
        }
    }
    buildFunctionsGraphAndGetDependencyMap(calculationFunctions) {
        const dependencyMap = new Map2_1.Map2();
        const targetMap = new Map2_1.Map2();
        for (const [index, calculationFunction] of calculationFunctions.entries()) {
            this.addToGraph(dependencyMap, targetMap, calculationFunction, index);
        }
        return dependencyMap;
    }
    buildCachedOrderedFunctions(dependencyMap) {
        const cachedNextFunctions = new Map2_1.Map2();
        for (const dependencyKey of dependencyMap.keys()) {
            cachedNextFunctions?.set(dependencyKey, Iterator.from(this.getOrderedFunctionsByGraph(dependencyKey, dependencyMap)).toArray());
        }
        return cachedNextFunctions;
    }
    ensurePrepared() {
        if (this.cachedNextFunctions == undefined) {
            const dependencyMap = this.buildFunctionsGraphAndGetDependencyMap(this.calculationFunctions);
            this.cachedNextFunctions = this.buildCachedOrderedFunctions(dependencyMap);
        }
    }
    getOrderedFunctions(updatedTarget) {
        this.ensurePrepared();
        const legacyMask = this.dataDeclarationHelper.getMask(updatedTarget.modelPath);
        const targetPathMask = legacyMask != undefined
            ? (0, ModelPath_1.createFromMask)(legacyMask, true, ModelPath_1.PathTokens.each)
            : this.getMaskDirty(updatedTarget.modelPath);
        const maskTarget = new CalculationTarget_1.CalculationTarget(targetPathMask, updatedTarget.targetAttribute);
        const allFinalFunction = this.cachedNextFunctions?.getOrSet(maskTarget, () => []) ?? [];
        return allFinalFunction;
    }
    getMaskDirty(exactPath) {
        return (0, ModelPath_1.createAbsoluteFromTokens)(Iterator.from(exactPath.getPathParts())
            .map(x => (typeof x === "string" && !isNaN(parseInt(x, 10)) ? ModelPath_1.PathTokens.each : x))
            .toArray());
    }
    getOrderedFunctionsByGraph(maskTarget, depMap) {
        const firstFunctions = depMap.get(maskTarget) ?? [];
        const visitedNodes = new Set();
        const filteredNodes = [];
        for (const node of this.getNextFunctions(...firstFunctions)) {
            if (!visitedNodes.has(node.id)) {
                filteredNodes.push(node);
                visitedNodes.add(node.id);
            }
        }
        const allNodes = (0, IterableUtils_1.reverseArray)(filteredNodes);
        return allNodes.map(x => x.calculationFunction);
    }
    *getNextFunctions(...nodes) {
        for (const node of nodes) {
            for (const nextNode of node.nextNodes) {
                if (!nextNode.calculationFunction.getTarget().isEquals(node.calculationFunction.getTarget())) {
                    yield* this.getNextFunctions(nextNode);
                }
            }
            yield node;
        }
    }
    getAllOrderedFunctions() {
        const orderedFns = [];
        const visited = [];
        for (const calculationFunction of this.calculationFunctions) {
            this.dfs(calculationFunction.getTarget(), visited, orderedFns, calculationFunction);
        }
        return orderedFns;
    }
    dropCache() {
        this.cachedNextFunctions = undefined;
    }
    getArgumentsPaths(model, calcPath) {
        const calcPathMask = (0, ModelPath_1.createFromMask)(this.dataDeclarationHelper.getMask(calcPath) ?? calcPath, true, ModelPath_1.PathTokens.each);
        const result = this.calculationFunctions
            .find(fn => {
            const calcFuncTarget = fn.getTarget();
            return (calcFuncTarget.targetAttribute === CalculationTarget_1.CalculationTargetAttribute.Value &&
                calcFuncTarget.modelPath.isEquals(calcPathMask));
        })
            ?.getDependencies()
            ?.map(dep => dep.getDependencyPath())
            ?.map(dep => this.getUnwrappedAggregationTargets(dep))
            ?.flat()
            ?.map(mask => {
            const legacyMask = mask.toLegacyPath();
            const maskWithResolvedInstances = PathUtils_1.PathUtils.resolveMaskToDeepestCommonInstance(calcPath, legacyMask);
            const dependsOnPaths = MaskResolver_1.MaskResolver.resolveMask(model, maskWithResolvedInstances ?? legacyMask);
            return dependsOnPaths;
        })
            ?.flat() ?? [];
        return [...IterableUtils_1.IterUtils.distinctBy(result, x => x)];
    }
    getUnwrappedAggregationTargets(aggregationModelPath) {
        if ((0, ModelPathHelper_1.isAggregationPath)(aggregationModelPath)) {
            return (this.calculationFunctions
                .find(fn => {
                const calcFuncTarget = fn.getTarget();
                return (calcFuncTarget.targetAttribute === CalculationTarget_1.CalculationTargetAttribute.Value &&
                    calcFuncTarget.modelPath.isEquals(aggregationModelPath));
            })
                ?.getDependencies()
                ?.map(dep => dep.getDependencyPath())
                ?.map(dep => this.getUnwrappedAggregationTargets(dep))
                ?.flat() ?? []);
        }
        return [aggregationModelPath];
    }
    createEvaluationContext(model, relatedEvents) {
        return new EvaluationContext_1.EvaluationContext(model, this.errorManager, relatedEvents ?? [], this.ignoredPathChecker, this.picklistsProvider, this.externalPropertiesProvider);
    }
}
exports.Calculator2 = Calculator2;
