"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EvaluationContext = void 0;
const MaskResolver_1 = require("../PathUtils/MaskResolver");
const Errors_1 = require("../../../../../Common/Errors");
class EvaluationContext {
    model;
    relatedEvents;
    ignoredPathChecker;
    iterationContexts = [];
    errorManager;
    externalPropertiesProvider;
    picklistsProvider;
    constructor(model, errorManager, relatedEvents, ignoredPathChecker, picklistsProvider, externalPropertiesProvider, iterationContexts = []) {
        this.externalPropertiesProvider = externalPropertiesProvider;
        this.picklistsProvider = picklistsProvider;
        this.ignoredPathChecker = ignoredPathChecker;
        this.model = model;
        this.iterationContexts = iterationContexts;
        this.errorManager = errorManager;
        this.relatedEvents = [...relatedEvents];
    }
    getPicklistProvider() {
        return this.picklistsProvider;
    }
    getExternalPropertiesProvider() {
        return this.externalPropertiesProvider;
    }
    set(pathParts, value) {
        return this.model.setValue(this.resolveIterationContextsStrict(pathParts), value);
    }
    resolvePath(targetPath) {
        return this.resolveIterationContexts(targetPath);
    }
    setError(targetPath, error) {
        const targetLegacyPath = this.resolveIterationContextsStrict(targetPath);
        if (error === undefined) {
            this.errorManager.clearCached(this.model, targetLegacyPath);
        }
        else {
            this.errorManager.clearCached(this.model, targetLegacyPath);
            this.errorManager.addErrorCached(this.model, targetLegacyPath, error);
        }
    }
    setWarning(targetPath, error) {
        const targetLegacyPath = this.resolveIterationContexts(targetPath).toLegacyPath();
        if (error === undefined) {
            this.errorManager.clearCached(this.model, targetLegacyPath);
        }
        else {
            this.errorManager.clearCached(this.model, targetLegacyPath);
            this.errorManager.addWarningCached(this.model, targetLegacyPath, error);
        }
    }
    setAutoValue(targetPath, value) {
        this.model.setViewModelField(this.resolveIterationContextsStrict(targetPath), "autoValue", value);
    }
    get(modelPath) {
        return this.model.getValue(this.resolveIterationContextsStrict(modelPath));
    }
    getInstances(modelPath) {
        return this.model.getChildren(this.resolveIterationContextsStrict(modelPath)) ?? [];
    }
    getInstanceCount(modelPath) {
        return (this.model.getChildren(this.resolveIterationContextsStrict(modelPath)) ?? []).length;
    }
    getAutoValue(modelPath) {
        return this.model.getViewModelField(this.resolveIterationContextsStrict(modelPath), "autoValue");
    }
    getIgnoreAutoFlag(modelPath) {
        return (this.model.getViewModelField(this.resolveIterationContextsStrict(modelPath), "ignoreAutoFlag") ?? false);
    }
    *getAllResolvedValues(modelPath) {
        const resolvedPath = this.resolveIterationContexts(modelPath).toAbsolute();
        const allResolvedPaths = MaskResolver_1.MaskResolver.resolvePath(this.model, resolvedPath);
        for (const path of allResolvedPaths) {
            yield this.model.getValue(path);
        }
    }
    getAutoFlag(modelPath) {
        const sourceValue = this.model.getViewModelField(this.resolveIterationContextsStrict(modelPath), "autoFlag");
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return sourceValue === "true" ? true : sourceValue;
    }
    pushIterationContext(iterationContext) {
        return new EvaluationContext(this.model, this.errorManager, this.relatedEvents, this.ignoredPathChecker, this.picklistsProvider, this.externalPropertiesProvider, [...this.iterationContexts, iterationContext]);
    }
    popIterationContext() {
        return new EvaluationContext(this.model, this.errorManager, this.relatedEvents, this.ignoredPathChecker, this.picklistsProvider, this.externalPropertiesProvider, this.iterationContexts.slice(0, this.iterationContexts.length - 1));
    }
    isIgnoredPath(targetPath) {
        return this.ignoredPathChecker.isIgnoredPath(this.resolveIterationContextsStrict(targetPath));
    }
    resolveIterationContexts(targetPath) {
        let result = targetPath;
        for (const iterationContext of this.iterationContexts) {
            result = iterationContext.applyTo(result);
        }
        return result;
    }
    resolveIterationContextsStrict(targetPath) {
        let result = targetPath;
        for (const iterationContext of this.iterationContexts) {
            result = iterationContext.applyTo(result);
        }
        if (!result.isResolved()) {
            throw new Errors_1.InvalidArgumentError("targetPath", `Target path must be fully resolved`);
        }
        return result;
    }
}
exports.EvaluationContext = EvaluationContext;
