"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LegacyValidationFunctionAdapter = void 0;
const ModelPath_1 = require("../../../../../../Common/ModelPath/ModelPath");
const CalculationFunctionBase_1 = require("../AutoCalculations/CalculationFunctionBase");
const CalculationFunctionDependency_1 = require("../CalculationFunctionDependency");
const CalculationTarget_1 = require("../CalculationTarget");
class LegacyValidationFunctionAdapter extends CalculationFunctionBase_1.CalculationFunctionBase {
    calculationTarget;
    dependencies;
    validatorPathMask;
    enableLegacyValidators;
    customValidationFunctionCallContextProvider;
    getIsValidationsDisabled;
    localization;
    store;
    errorManager;
    picklistsProvider;
    validator;
    condition;
    constructor(validatorPathMask, validator, localization, store, errorManager, picklistsProvider, customValidationFunctionCallContextProvider, enableLegacyValidators, getIsValidationsDisabled, deps, condition) {
        const targetPath = (0, ModelPath_1.createFromMask)(validatorPathMask, true, ModelPath_1.PathTokens.each);
        super(targetPath);
        this.condition = condition;
        this.validatorPathMask = validatorPathMask;
        this.enableLegacyValidators = enableLegacyValidators;
        this.customValidationFunctionCallContextProvider = customValidationFunctionCallContextProvider;
        this.getIsValidationsDisabled = getIsValidationsDisabled;
        this.localization = localization;
        this.store = store;
        this.errorManager = errorManager;
        this.picklistsProvider = picklistsProvider;
        this.validator = validator;
        this.calculationTarget = new CalculationTarget_1.CalculationTarget(targetPath, CalculationTarget_1.CalculationTargetAttribute.Validation);
        const dependenciesList = [(0, ModelPath_1.createFromMask)(validatorPathMask, true, ModelPath_1.PathTokens.current)];
        if (deps != undefined) {
            dependenciesList.push(...deps);
        }
        this.dependencies = dependenciesList.map(x => new CalculationFunctionDependency_1.CalculationFunctionDependency(targetPath, x));
    }
    asDebugString() {
        if ("validate" in this.validator) {
            return this.validator.asDebugString();
        }
        else {
            return "[ValidatorOnlyForEngineApprovalTests]";
        }
    }
    getDependencies() {
        return this.dependencies;
    }
    isDependsOn(calculationTarget) {
        return (calculationTarget.targetAttribute === CalculationTarget_1.CalculationTargetAttribute.Value &&
            this.dependencies.some(x => x.isMatchToPath(calculationTarget.modelPath)));
    }
    getTarget() {
        return this.calculationTarget;
    }
    async execute(context) {
        if (this.getIsValidationsDisabled()) {
            context.setError(this.targetPath, undefined);
            return;
        }
        if (this.condition != undefined) {
            const condResult = this.condition(context);
            if (!condResult) {
                context.setError(this.targetPath, undefined);
                return;
            }
        }
        const resolvedTargetPath = context.resolvePath(this.targetPath);
        if (context.isIgnoredPath(resolvedTargetPath)) {
            return;
        }
        if ("validate" in this.validator) {
            const result = await this.validator.validate({
                localizer: this.localization,
                store: this.store,
                path: resolvedTargetPath.toLegacyPath(),
                customValidationFunctionCallContextProvider: this.customValidationFunctionCallContextProvider,
                errorManager: this.errorManager,
                model: context.model,
                picklistsProvider: this.picklistsProvider,
                evaluationContext: context,
            }, {
                value: context.get((0, ModelPath_1.createFromMask)(this.validatorPathMask, true, ModelPath_1.PathTokens.current)),
                autoValue: context.getAutoValue((0, ModelPath_1.createFromMask)(this.validatorPathMask, true, ModelPath_1.PathTokens.current)),
            });
            if (result.valid) {
                context.setError(this.targetPath, undefined);
            }
            else {
                if (result.type === "error") {
                    context.setError(this.targetPath, result.message ?? "");
                }
                else {
                    context.setWarning(this.targetPath, result.message ?? "");
                }
            }
        }
        else {
            const evaluationResult = this.executeValidatorOnlyForEngineApprovalTests(context.model, this.validator, resolvedTargetPath);
            context.setError(this.targetPath, evaluationResult);
        }
    }
    executeValidatorOnlyForEngineApprovalTests(model, validator, targetPath) {
        if (!this.enableLegacyValidators) {
            throw new Error("Legacy validation functions is disabled");
        }
        let errorMessage;
        validator.call({
            ...this.customValidationFunctionCallContextProvider.getCustomValidationFunctionCallContext(model),
            errorManager: {
                addError: (nodePath, message) => {
                    errorMessage = message;
                },
                addErrorCached: (nodePath, message) => {
                    errorMessage = message;
                },
                addWarning: (nodePath, message) => {
                    // none
                },
                clearErrors: (nodePath) => {
                    // none
                },
                clearWarnings: (nodePath) => {
                    // none
                },
                removeWarning: (nodePath, message) => {
                    // none
                },
            },
        }, targetPath.toLegacyPath());
        return errorMessage;
    }
}
exports.LegacyValidationFunctionAdapter = LegacyValidationFunctionAdapter;
