"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ChangeSetManager = void 0;
const IterableUtils_1 = require("../../../../../Common/IterableUtils");
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const Deferred_1 = require("../../../Helpers/Deferred");
const TypingUtils_2 = require("../../../Helpers/TypingUtils");
const FormActionCreators_1 = require("../../ActionCreators/FormActionCreators");
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const UUID_1 = require("../../../../../Common/UUID");
const AbsoluteModelFieldPath_1 = require("../../../../../Common/ModelPath/AbsoluteModelFieldPath");
const ExternalEvents_1 = require("../EventManagement/ExternalEvents/ExternalEvents");
const ChangeSet_1 = require("./ChangeSet");
const IChangeSetManager_1 = require("./IChangeSetManager");
class ChangeSetManager {
    getStore;
    useServerSideCalculations;
    onConflictDetected;
    candyApiDraftClient;
    attachmentsPublicForms;
    externalEventManager;
    changeSetSendQueue = [];
    redefinedChangeSets = [];
    serverChangesCallbacks = {};
    changeSet;
    autosendIntervalHandler = undefined;
    processingRequest = false;
    currentDataVersion = 0;
    isInConflictState = false;
    constructor(candyApiDraftClient, getStore, onConflictDetected, useServerSideCalculations, attachmentsPublicForms, externalEventManager) {
        this.attachmentsPublicForms = attachmentsPublicForms;
        this.useServerSideCalculations = useServerSideCalculations;
        this.changeSet = new ChangeSet_1.ChangeSet(this.useServerSideCalculations);
        this.onConflictDetected = onConflictDetected;
        this.candyApiDraftClient = candyApiDraftClient;
        this.externalEventManager = externalEventManager;
        this.getStore = getStore;
    }
    applyEvents(events) {
        const changeSet = this.getCurrentChangeSet();
        const isRealChangeSet = changeSet === this.changeSet;
        if (isRealChangeSet) {
            const canBeMerged = isRealChangeSet && events.every(evt => changeSet.canBeMerged(evt));
            if (!canBeMerged) {
                (0, TypingUtils_2.runFloatingAsyncAction)(() => this.enqueueChangeSetToSend(this.changeSet));
                this.changeSet = new ChangeSet_1.ChangeSet(this.useServerSideCalculations);
            }
        }
        for (const event of events) {
            this.getCurrentChangeSet().appendEventInfo(event);
        }
        if (events.length > 0) {
            if (this.redefinedChangeSets.length === 0 &&
                this.getStore().getState().formVisualState.savingStatus === "success") {
                this.setSavingStatus("unsavedChanges");
            }
        }
    }
    scheduleInnerChangeAction(action) {
        const promise = this.enqueueChangeSetToSend(action);
        if (!this.processingRequest) {
            (0, TypingUtils_2.runFloatingAsyncAction)(() => this.processEnqueuedChangeSets(false));
        }
        return promise;
    }
    send = (options = {}) => {
        const promise = this.enqueueChangeSetToSend(this.changeSet);
        this.changeSet = new ChangeSet_1.ChangeSet(this.useServerSideCalculations);
        if (!this.processingRequest) {
            (0, TypingUtils_2.runFloatingAsyncAction)(() => this.processEnqueuedChangeSets(options.force ?? false, options.operationName));
        }
        const attachmentsPromises = Object.values(this.attachmentsPublicForms).map(attachmentForm => attachmentForm.save());
        return promise.then(async (_) => await Promise.all(attachmentsPromises)).then();
    };
    sendSync() {
        if (this.changeSet.isEmpty()) {
            return;
        }
        if (this.processingRequest) {
            throw new Error("Cannot send ChangeSet: there are sending in progress");
        }
        if (this.isInConflictState) {
            throw new IChangeSetManager_1.ConflictChangeSetError(`Conflict detected`);
        }
        this.stopAutosend();
        try {
            const status = this.candyApiDraftClient.sendChangeSetSync(this.changeSet.toServerChangeSet(), this.currentDataVersion);
            if (typeof status === "object") {
                this.setCurrentDataVersion(status.dataVersion);
            }
            else if (status === 409) {
                this.setConflictState();
                throw new IChangeSetManager_1.ConflictChangeSetError(`Conflict detected`);
            }
            else if (status !== 200) {
                throw new Error(`Failed to save ChangeSet with status: ${status}`);
            }
            this.changeSet = new ChangeSet_1.ChangeSet(this.useServerSideCalculations);
        }
        finally {
            this.startAutosend(this.useServerSideCalculations ? 500 : 10000);
        }
    }
    startAutosend(sendInterval) {
        if (this.autosendIntervalHandler != undefined) {
            this.stopAutosend();
        }
        const sendAction = () => {
            if (!this.processingRequest) {
                (0, TypingUtils_2.runFloatingAsyncAction)(() => this.send());
            }
        };
        // @ts-ignore
        this.autosendIntervalHandler = setInterval(sendAction, sendInterval);
    }
    stopAutosend() {
        if (this.autosendIntervalHandler == undefined) {
            return;
        }
        clearInterval(this.autosendIntervalHandler);
    }
    redefine() {
        this.redefinedChangeSets.push(new ChangeSet_1.ChangeSet(this.useServerSideCalculations));
    }
    isRedefined() {
        return this.redefinedChangeSets.length > 0;
    }
    dropRedefined() {
        this.redefinedChangeSets.pop();
    }
    mergeRedefined() {
        const lastChangeSet = this.redefinedChangeSets.pop();
        if (lastChangeSet != undefined) {
            // NOTE Тут трудноуловимый баг: prependWithChangeSet добавляет в целевой данные, как если бы lastChangeSet,
            // шел перед целевым. Тут явно хотели другого
            this.getCurrentChangeSet().prependWithChangeSet(lastChangeSet);
        }
    }
    onServerChangesReceived(callback) {
        const key = (0, UUID_1.uuid)();
        this.serverChangesCallbacks[key] = callback;
        return () => delete this.serverChangesCallbacks[key];
    }
    __testOnly__getChangeSetToSend() {
        return this.changeSet;
    }
    setCurrentDataVersion(version) {
        this.currentDataVersion = version;
        this.isInConflictState = false;
    }
    enqueueChangeSetToSend(changeSetOrAction) {
        const currentChangeSendDeferred = new Deferred_1.Deferred();
        this.changeSetSendQueue.push([changeSetOrAction, currentChangeSendDeferred]);
        return currentChangeSendDeferred.promise;
    }
    dequeueChangeSetToSend() {
        return this.changeSetSendQueue.shift();
    }
    getCurrentChangeSet() {
        return IterableUtils_1.IterUtils.last(this.redefinedChangeSets) ?? this.changeSet;
    }
    setSavingStatus(value) {
        this.externalEventManager.triggerExternalEvent((0, ExternalEvents_1.formSavingStatusEvent)(value));
        this.getStore().dispatch((0, FormActionCreators_1.setFormSaveStatus)(value));
    }
    setConflictState() {
        this.isInConflictState = true;
        this.onConflictDetected();
    }
    async processEnqueuedChangeSets(force = false, operationName) {
        const queueItem = this.dequeueChangeSetToSend();
        if (queueItem == undefined) {
            return;
        }
        this.processingRequest = true;
        try {
            const [changeSetOrAction, deferred] = queueItem;
            const changeSet = changeSetOrAction instanceof ChangeSet_1.ChangeSet ? changeSetOrAction : undefined;
            if (changeSet != undefined && changeSet.isEmpty()) {
                deferred.resolve();
                return;
            }
            this.setSavingStatus("saving");
            try {
                const serverChangeSet = (changeSet ?? new ChangeSet_1.ChangeSet(this.useServerSideCalculations)).toServerChangeSet();
                const action = changeSetOrAction instanceof ChangeSet_1.ChangeSet
                    ? async () => {
                        const result = this.useServerSideCalculations
                            ? await this.candyApiDraftClient.sendChangeWithNormalizationSet(serverChangeSet, this.currentDataVersion, undefined, operationName)
                            : await this.candyApiDraftClient.sendChangeSet(serverChangeSet, this.currentDataVersion, operationName);
                        return result;
                    }
                    : changeSetOrAction;
                const result = await action();
                if (typeof result === "object" && "dataVersion" in result) {
                    this.setCurrentDataVersion(result.dataVersion);
                    if (result.changedPaths != undefined && result.changedPaths.length > 0) {
                        for (const changesCallback of Object.values(this.serverChangesCallbacks)) {
                            await changesCallback(serverChangeSet, this.changeSetSendQueue
                                .map(([x, _]) => (x instanceof ChangeSet_1.ChangeSet ? x : undefined))
                                .filter(TypingUtils_1.isNotNullOrUndefined)
                                .concat(this.changeSet), result.changedPaths.map(x => AbsoluteModelFieldPath_1.AbsoluteModelFieldPath.createFromMask(x, ModelPath_1.PathTokens.each)));
                        }
                    }
                }
                else if (typeof result === "number" && result === 409) {
                    this.setConflictState();
                    throw new IChangeSetManager_1.ConflictChangeSetError("Conflict detected!");
                }
                else {
                    throw new Error("Unknown response type!");
                }
                deferred.resolve();
                this.setSavingStatus("success");
            }
            catch (error) {
                if (changeSet != undefined) {
                    const nextChangeSetInQueue = this.changeSetSendQueue
                        .map(([x, _]) => (x instanceof ChangeSet_1.ChangeSet ? x : undefined))
                        .filter(TypingUtils_1.isNotNullOrUndefined)[0];
                    const nextChangeSet = nextChangeSetInQueue != undefined ? nextChangeSetInQueue : this.changeSet;
                    nextChangeSet.prependWithChangeSet(changeSet);
                }
                deferred.reject(error);
                this.setSavingStatus(force ? "error" : "unsavedChanges");
            }
        }
        finally {
            this.processingRequest = false;
            (0, TypingUtils_2.runFloatingAsyncAction)(() => this.processEnqueuedChangeSets());
        }
    }
}
exports.ChangeSetManager = ChangeSetManager;
