"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FilteringManager = void 0;
const lodash_1 = require("lodash");
const AbsoluteModelFieldPath_1 = require("../../../../../Common/ModelPath/AbsoluteModelFieldPath");
const MultiLineFilter_1 = require("../../../Controls/MultiControls/MultiLine/MultiLineFilter");
const MultilineStateSelect_1 = require("../../../Helpers/StateAccessors/MultilineStateSelect");
const FiltersSessionHelpers_1 = require("../../../Controls/MultiControls/MultiLine/Store/FiltersSessionHelpers");
const FilteringHelpers_1 = require("../../../Helpers/FilteringHelpers");
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const AllActionCreators_1 = require("../../ActionCreators/AllActionCreators");
const TypingUtils_1 = require("../../../Helpers/TypingUtils");
const Set2_1 = require("../../../../../Common/ModelPath/Set/Set2");
const TypingUtils_2 = require("../../../../../Common/TypingUtils");
const PathMatcher_1 = require("../../../../../Common/ModelPath/PathMatcher");
const Calculator2Compatibility_1 = require("../Calculator2/Calculator2Compatibility");
const LazyLoadingManager_1 = require("../LazyLoading/LazyLoadingManager");
const multiLineActions_1 = require("../../../Controls/MultiControls/MultiLine/Store/multiLineActions");
const MultilineHelpers_1 = require("../../../Helpers/StateAccessors/MultilineHelpers");
const ServerSideFilteringStrategy_1 = require("./ServerSideFilteringStrategy");
const ClientSideFilteringStrategy_1 = require("./ClientSideFilteringStrategy");
const DefaultServerSideFilterDebounceTimeout = 500;
const DefaultClientSideFilterDebounceTimeout = 400;
const ShortFilterDebounceTimeout = 20;
const getSubscriptionKey = (bindingPath, filteredUniqKey) => `${bindingPath}_${filteredUniqKey}`;
class FilteringManager {
    subscriptions = new Map();
    getState;
    lazyLoadingManager;
    sendToServerAction;
    dispatch;
    serverSideFilteringStrategy;
    clientSideFilteringStrategy;
    constructor(lazyLoadingManager, getState, dispatch, candyApiDraftClient, sendToServerAction) {
        this.lazyLoadingManager = lazyLoadingManager;
        this.dispatch = dispatch;
        this.getState = getState;
        this.serverSideFilteringStrategy = new ServerSideFilteringStrategy_1.ServerSideFilteringStrategy(getState, candyApiDraftClient);
        this.clientSideFilteringStrategy = new ClientSideFilteringStrategy_1.ClientSideFilteringStrategy(getState);
        this.sendToServerAction = sendToServerAction;
    }
    subscribe(bindingPath, filteredUniqKey, useServerFilters, enableFilterTracking, useShortDebounce, defaultFilter) {
        const path = (0, ModelPath_1.convertToLegacyPath)(bindingPath);
        if (defaultFilter != undefined) {
            this.dispatch((0, AllActionCreators_1.filter)(path, path, {
                expression: defaultFilter,
                filterSourceId: "defaultFilterSelect",
                filterSourceInfo: {
                    type: MultiLineFilter_1.FilterSourceInfoType.DefaultFilter,
                },
            }, filteredUniqKey));
        }
        const key = getSubscriptionKey((0, ModelPath_1.convertToLegacyPath)(bindingPath), filteredUniqKey);
        const queue = (0, Calculator2Compatibility_1.createActionQueue)("FilterQueue");
        const subscription = {
            multilinePath: (0, ModelPath_1.convertToAbsoluteModelPath)(bindingPath),
            filteredUniqKey: filteredUniqKey,
            enableFilterTracking: enableFilterTracking,
            trackingPaths: new Set2_1.Set2(),
            dispose: TypingUtils_2.noop,
            handleFilterChangeDebounced: (0, lodash_1.debounce)(() => queue.enqueueWithPromise(() => this.handleFilterChange((0, ModelPath_1.convertToLegacyPath)(bindingPath), filteredUniqKey)), useShortDebounce
                ? ShortFilterDebounceTimeout
                : useServerFilters
                    ? DefaultServerSideFilterDebounceTimeout
                    : DefaultClientSideFilterDebounceTimeout),
        };
        this.subscriptions.set(key, subscription);
        (0, TypingUtils_1.runFloatingAsyncAction)(() => this.handleFilterChange((0, ModelPath_1.convertToLegacyPath)(bindingPath), filteredUniqKey, false));
    }
    unsubscribe(bindingPath, filteredUniqKey) {
        const key = getSubscriptionKey((0, ModelPath_1.convertToLegacyPath)(bindingPath), filteredUniqKey);
        const subscription = this.getSubscription((0, ModelPath_1.convertToLegacyPath)(bindingPath), filteredUniqKey);
        subscription?.dispose();
        this.subscriptions.delete(key);
    }
    notifyWithChanges(changedPathFields) {
        const subscriptionsToRefresh = new Set();
        for (const changedPathField of changedPathFields) {
            for (const subscription of this.subscriptions.values()) {
                for (const filterPath of subscription.trackingPaths.values()) {
                    if (filterPath.field === changedPathField.field &&
                        filterPath.path
                            .matchFor(changedPathField.path)
                            .every(x => x.kind === PathMatcher_1.MatchKind.MatchedPath || x.kind === PathMatcher_1.MatchKind.MatchedMultiplicity)) {
                        subscriptionsToRefresh.add(subscription);
                    }
                }
            }
        }
        for (const subscriptionToRefresh of subscriptionsToRefresh) {
            this.requestFiltering(subscriptionToRefresh.multilinePath, subscriptionToRefresh.filteredUniqKey);
        }
    }
    async activePageChanged(bindingPath, filteredUniqKey, page) {
        const modelPath = (0, ModelPath_1.convertToAbsoluteModelPath)(bindingPath);
        const multilineState = (0, MultilineStateSelect_1.getMultilineState)(this.getState().multiLineInfos, bindingPath, filteredUniqKey);
        if (multilineState == undefined) {
            return;
        }
        const displayedInstances = (0, MultilineHelpers_1.getDisplayedInstances)(this.getState(), bindingPath, filteredUniqKey);
        const updatedPageInstances = this.getPreloadedInstances(page, multilineState.instancesPerPage, displayedInstances, 0);
        const isLoadingRequired = this.getNotLoadedInstances((0, ModelPath_1.convertToAbsoluteModelPath)(bindingPath), updatedPageInstances).length > 0;
        try {
            if (isLoadingRequired) {
                this.dispatch((0, AllActionCreators_1.setMultilineLoading)((0, ModelPath_1.convertToLegacyPath)(bindingPath), filteredUniqKey, true));
            }
            this.dispatch({
                type: multiLineActions_1.MultiLineActionType.GoToPage,
                payload: {
                    bindingPath: (0, ModelPath_1.convertToLegacyPath)(bindingPath),
                    page: page,
                    filteredUniqKey: filteredUniqKey,
                },
            });
            const updatedMultilineState = (0, MultilineStateSelect_1.getMultilineState)(this.getState().multiLineInfos, bindingPath, filteredUniqKey);
            await this.loadInstancesIfNeeded(modelPath, filteredUniqKey, this.getPreloadedInstances(updatedMultilineState.currentPage, updatedMultilineState.instancesPerPage, displayedInstances));
        }
        finally {
            if (isLoadingRequired) {
                this.dispatch((0, AllActionCreators_1.setMultilineLoading)((0, ModelPath_1.convertToLegacyPath)(bindingPath), filteredUniqKey, false));
            }
        }
    }
    requestFiltering(bindingPath, filteredUniqKey) {
        const subscription = this.getSubscription(bindingPath, filteredUniqKey);
        if (subscription != undefined) {
            (0, TypingUtils_1.runFloatingAsyncAction)(() => subscription.handleFilterChangeDebounced() ?? Promise.resolve());
        }
    }
    requestFilteringAsync(bindingPath, filteredUniqKey) {
        const subscription = this.getSubscription(bindingPath, filteredUniqKey);
        if (subscription != undefined) {
            return subscription.handleFilterChangeDebounced() ?? Promise.resolve();
        }
        return Promise.resolve();
    }
    async requestFilteringAllMultilines() {
        const state = this.getState();
        const allMultilinesToFilter = Object.keys(state.multiLineInfos).flatMap(path => Object.keys(state.multiLineInfos[path] ?? {}).map(filterUniqueId => ({
            multilinePath: path,
            filterUniqueId: filterUniqueId,
        })));
        const resultPromises = [];
        for (const item of allMultilinesToFilter) {
            const result = this.requestFilteringAsync(item.multilinePath, item.filterUniqueId);
            resultPromises.push(result);
        }
        await Promise.all(resultPromises);
    }
    async requestInstancesWithExtraFilter(multilinePath, filterUniqueKey, filterExpression) {
        const multilineState = (0, MultilineStateSelect_1.getMultilineState)(this.getState().multiLineInfos, multilinePath, filterUniqueKey);
        const filterStrategy = this.getFilteringStrategy(multilineState);
        const filterSource = {
            expression: filterExpression,
            filterSourceId: "NoSource",
            filterSourceInfo: { type: MultiLineFilter_1.FilterSourceInfoType.DefaultFilter },
        };
        const sourceFilters = multilineState.filters ?? [];
        const resultFilter = [...sourceFilters, filterSource];
        const filteredInstances = await filterStrategy.performFilteringAndSorting((0, ModelPath_1.convertToLegacyPath)(multilinePath), filterUniqueKey, resultFilter, multilineState.sort);
        return filteredInstances;
    }
    getSubscription(bindingPath, filteredUniqKey) {
        const key = getSubscriptionKey((0, ModelPath_1.convertToLegacyPath)(bindingPath), filteredUniqKey);
        const subscription = this.subscriptions.get(key);
        return subscription;
    }
    updateSubscriptionDependencies(multilinePath, filterUniqueKey, filters) {
        const subscription = this.getSubscription(multilinePath, filterUniqueKey);
        if (subscription != undefined) {
            subscription.trackingPaths.clear();
            if (subscription.enableFilterTracking) {
                const multilineAbsolutePath = (0, ModelPath_1.convertToAbsoluteModelPath)(multilinePath);
                const paths = filters
                    .flatMap(x => (0, MultiLineFilter_1.getAllPathsFromExpression)(x.expression))
                    .map(x => (0, FilteringHelpers_1.resolveFilterDataPath)(x, multilineAbsolutePath));
                for (const path of paths) {
                    subscription.trackingPaths.add(path);
                }
                if (!subscription.trackingPaths.isEmpty()) {
                    subscription.trackingPaths.add(new AbsoluteModelFieldPath_1.AbsoluteModelFieldPath(multilineAbsolutePath, "children"));
                }
            }
        }
    }
    getFilteringStrategy(multilineState) {
        const isServerFilter = multilineState.useServerFilter ?? false;
        return isServerFilter ? this.serverSideFilteringStrategy : this.clientSideFilteringStrategy;
    }
    async sendDataToServerIfRequired(multilineState) {
        const isServerFilter = multilineState.useServerFilter ?? false;
        if (isServerFilter) {
            await this.sendToServerAction();
        }
    }
    getNotLoadedInstances(bindingPath, instances) {
        const lazyLoadingManager = this.lazyLoadingManager;
        const isLazyLoadingEnabled = lazyLoadingManager.isLazyLoadingEnabled();
        const requiredInstances = isLazyLoadingEnabled
            ? instances.filter(x => lazyLoadingManager.getInstanceLazyLoadingState(bindingPath, x) ===
                LazyLoadingManager_1.LazyLoadingInstanceState.OnlyInitialData ||
                lazyLoadingManager.getInstanceLazyLoadingState(bindingPath, x) ===
                    LazyLoadingManager_1.LazyLoadingInstanceState.Pending)
            : [];
        return requiredInstances;
    }
    async loadInstancesIfNeeded(bindingPath, filterUniqueKey, instances) {
        const lazyLoadingManager = this.lazyLoadingManager;
        const requiredInstances = this.getNotLoadedInstances(bindingPath, instances);
        if (requiredInstances.length > 0) {
            await lazyLoadingManager.requestInstancesLoad(bindingPath, ...requiredInstances);
        }
    }
    getPreloadedInstances(currentPage, instancesPerPage, filteredInstances, preloadedPagesCount = 4) {
        const result = filteredInstances.slice((currentPage - 1) * instancesPerPage, (currentPage + preloadedPagesCount) * instancesPerPage);
        return result;
    }
    async handleFilterChange(multilinePath, filterUniqueKey, resetToFirstPage = true) {
        const multilineState = (0, MultilineStateSelect_1.getMultilineState)(this.getState().multiLineInfos, multilinePath, filterUniqueKey);
        if (multilineState == undefined) {
            return;
        }
        try {
            this.dispatch((0, AllActionCreators_1.setMultilineLoading)(multilinePath, filterUniqueKey, true));
            const filters = multilineState.filters ?? [];
            this.storeFilters(this.getState(), multilinePath, filterUniqueKey, filters);
            this.updateSubscriptionDependencies(multilinePath, filterUniqueKey, filters.filter(x => x.filterSourceInfo.type === MultiLineFilter_1.FilterSourceInfoType.DefaultFilter));
            await this.sendDataToServerIfRequired(multilineState);
            const filterStrategy = this.getFilteringStrategy(multilineState);
            const displayedInstances = await filterStrategy.performFilteringAndSorting(multilinePath, filterUniqueKey, multilineState.filters ?? [], multilineState.sort);
            const currentPage = resetToFirstPage
                ? 1
                : this.calculateCurrentPage(multilineState.currentPage, displayedInstances.length, multilineState.instancesPerPage);
            await this.loadInstancesIfNeeded((0, ModelPath_1.convertToAbsoluteModelPath)(multilinePath), filterUniqueKey, this.getPreloadedInstances(currentPage, multilineState.instancesPerPage, displayedInstances));
            this.dispatch((0, AllActionCreators_1.setFilteredData)(multilinePath, filterUniqueKey, displayedInstances, currentPage));
        }
        finally {
            this.dispatch((0, AllActionCreators_1.setMultilineLoading)(multilinePath, filterUniqueKey, false));
        }
    }
    calculateCurrentPage(pageNumber, displayedInstancesCount, instancesPerPage) {
        const pageCount = Math.ceil(displayedInstancesCount / instancesPerPage);
        const currentPage = Math.max(1, Math.min(pageNumber, pageCount));
        return currentPage;
    }
    storeFilters(state, multilinePath, filteredUniqKey, filters) {
        const storeFiltersWithinSession = (0, MultilineStateSelect_1.getMultilineState)(state.multiLineInfos, multilinePath, filteredUniqKey)?.storeFiltersWithinSession;
        if (storeFiltersWithinSession) {
            (0, FiltersSessionHelpers_1.setFilter)(filteredUniqKey, multilinePath, JSON.stringify(filters.filter(x => x.filterSourceInfo.type !== MultiLineFilter_1.FilterSourceInfoType.DefaultFilter)));
        }
    }
}
exports.FilteringManager = FilteringManager;
