"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Model = void 0;
const IterableUtils_1 = require("../../../../../Common/IterableUtils");
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const ModelPathSet_1 = require("../../../../../Common/ModelPath/Set/ModelPathSet");
const AllActionCreators_1 = require("../../ActionCreators/AllActionCreators");
const ModelEvent_1 = require("../EventManagement/ModelEvents/ModelEvent");
const PathUtils_1 = require("../PathUtils/PathUtils");
const MultilineHelpers_1 = require("../../../Helpers/StateAccessors/MultilineHelpers");
const Errors_1 = require("../../../../../Common/Errors");
const AbsoluteResolvedModelPathSet_1 = require("../../../../../Common/ModelPath/Set/AbsoluteResolvedModelPathSet");
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const Set2_1 = require("../../../../../Common/ModelPath/Set/Set2");
const ModelHunk_1 = require("./ModelHunk");
class Model {
    parentModel;
    modelNormalizer;
    pathWithTouchedErrors = new ModelPathSet_1.ModelPathSet();
    eventCollectionContexts = new Set();
    pathWithTouchedWarnings = new ModelPathSet_1.ModelPathSet();
    useShallowChildrenRemove;
    updates;
    deletedPaths;
    constructor(parentModel, modelNormalizer, dataDeclarationHelper, useShallowChildrenRemove) {
        this.useShallowChildrenRemove = useShallowChildrenRemove;
        this.parentModel = parentModel;
        this.modelNormalizer = modelNormalizer;
        this.updates = new ModelHunk_1.ModelHunk();
        this.deletedPaths = new AbsoluteResolvedModelPathSet_1.AbsoluteResolvedModelPathSet();
    }
    addEvent(event) {
        for (const eventCollectionContext of this.eventCollectionContexts.values()) {
            eventCollectionContext.addEvent(event);
        }
    }
    beginGetEvents() {
        const releaseAction = (x) => {
            if (x instanceof EventCollectionContext) {
                this.eventCollectionContexts.delete(x);
            }
        };
        const eventCollectionContext = new EventCollectionContext(releaseAction);
        this.eventCollectionContexts.add(eventCollectionContext);
        return eventCollectionContext;
    }
    getViewModel(bindingPath) {
        const path = (0, ModelPath_1.convertToAbsoluteModelPath)(bindingPath);
        if (this.deletedPaths.has(path)) {
            return this.updates.get(path);
        }
        const parentValue = this.parentModel.getViewModel(path);
        const dataValue = this.updates.get(path);
        if (parentValue != undefined && dataValue != undefined) {
            return { ...parentValue, ...dataValue };
        }
        return parentValue ?? dataValue;
    }
    setChildren(inPath, children) {
        const path = (0, ModelPath_1.convertToAbsoluteModelPath)(inPath);
        const prevInstancesSet = new Set(this.getChildren(path) ?? []);
        this.setViewModelFieldInternal(path, "children", children);
        const nextInstancesSet = new Set(children);
        const addedChildren = nextInstancesSet
            .values()
            .filter(child => !prevInstancesSet.has(child))
            .toArray();
        const removedChildren = prevInstancesSet
            .values()
            .filter(child => !nextInstancesSet.has(child))
            .toArray();
        for (const newChildInstance of addedChildren) {
            const childPath = path.joinWith(newChildInstance);
            if (!this.useShallowChildrenRemove) {
                this.removeByPrefix(childPath);
            }
            else {
                this.removeInternalSilently(childPath);
            }
            this.modelNormalizer.normalizeByPrefix(this, childPath);
        }
        for (const child of removedChildren) {
            const childPath = path.joinWith(child);
            if (!this.useShallowChildrenRemove) {
                this.removeByPrefix(childPath);
            }
            else {
                this.removeInternalSilently(childPath);
            }
        }
    }
    setValue(bindingPath, value) {
        this.setViewModelField(bindingPath, "value", value);
    }
    setViewModelField(inPath, name, value) {
        const path = (0, ModelPath_1.convertToAbsoluteModelPath)(inPath);
        if (name === "children") {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            const childrenValue = value;
            this.setChildren(path, childrenValue);
        }
        else {
            this.setViewModelFieldInternal(path, name, value);
        }
    }
    setBatch(batch) {
        for (const [path, viewModel] of ModelHunk_1.ModelHunk.iterateEntriesNew(batch)) {
            for (const [fieldName, value] of (0, IterableUtils_1.iterateEntries)(viewModel)) {
                this.setViewModelFieldInternal(path, fieldName, value);
            }
        }
    }
    setBatchByPrefix(prefix, batch) {
        const newModelChunk = {};
        for (const [path, value] of (0, IterableUtils_1.iterateEntries)(batch)) {
            newModelChunk[`${(0, ModelPath_1.convertToLegacyPath)(prefix)}/${path}`] = value;
        }
        this.setBatch(newModelChunk);
    }
    addInstance(path) {
        const allExistInstance = this.getChildren(path) ?? [];
        const instance = (0, MultilineHelpers_1.findFreeInstanceNumber)(allExistInstance);
        this.setChildren(path, (0, MultilineHelpers_1.getInstancesAfterAdd)(allExistInstance, instance, allExistInstance.length));
        return instance;
    }
    removeInstance(path, instance) {
        this.removeInstances(path, [instance]);
    }
    removeInstances(path, instances) {
        const allExistInstance = this.getChildren(path);
        if (allExistInstance) {
            this.setChildren(path, (0, MultilineHelpers_1.getInstancesAfterRemove)(allExistInstance, instances));
        }
    }
    copySubtree(sourceBindingPath, targetBindingPath, doNotCopyPaths) {
        const instancePath = (0, ModelPath_1.convertToAbsoluteModelPath)(sourceBindingPath);
        const dataWithRelativePaths = ModelHunk_1.ModelHunk.create();
        const instanceData = this.getBatch(instancePath);
        const deleteIgnorePath = ([_, relativePath]) => !doNotCopyPaths?.some(notCopy => relativePath.includes(notCopy));
        const pathsToCopy = ModelHunk_1.ModelHunk.paths(instanceData)
            .map(absolutePath => [
            absolutePath,
            absolutePath.replace(`${instancePath.toLegacyPath()}/`, ""),
        ])
            .filter(deleteIgnorePath);
        for (const [absolutePath, relativePath] of pathsToCopy) {
            ModelHunk_1.ModelHunk.set(dataWithRelativePaths, relativePath, ModelHunk_1.ModelHunk.get(instanceData, absolutePath));
        }
        this.setBatchByPrefix(targetBindingPath, dataWithRelativePaths);
    }
    getViewModelField(bindingPath, name) {
        const path = (0, ModelPath_1.convertToAbsoluteModelPath)(bindingPath);
        if (PathUtils_1.PathUtils.isPicklistFieldValueName(name)) {
            const viewModel = this.getViewModel(path);
            return viewModel?.picklist?.[PathUtils_1.PathUtils.getPicklistFieldName(name)];
        }
        return this.getViewModelFieldInternal(path, name);
    }
    getValue(bindingPath) {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return this.getViewModelField(bindingPath, "value");
    }
    getChildren(bindingPath) {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return this.getViewModelField(bindingPath, "children");
    }
    getBatch(nodePathPrefix) {
        const values = this.parentModel.getBatch(nodePathPrefix);
        if (ModelHunk_1.ModelHunk.isEmpty(this.updates) && this.deletedPaths.isEmpty()) {
            return values;
        }
        for (const path of ModelHunk_1.ModelHunk.iteratePaths(values)) {
            if (this.deletedPaths.has((0, ModelPath_1.createAbsolute)(path))) {
                ModelHunk_1.ModelHunk.set(values, path, ModelHunk_1.ModelHunk.get(this.updates, path));
            }
            else if (ModelHunk_1.ModelHunk.get(this.updates, path) != undefined) {
                ModelHunk_1.ModelHunk.set(values, path, { ...ModelHunk_1.ModelHunk.get(values, path), ...ModelHunk_1.ModelHunk.get(this.updates, path) });
            }
        }
        return values;
    }
    setChildrenWithoutNormalization(path, instances) {
        this.setViewModelFieldInternal((0, ModelPath_1.convertToAbsoluteModelPath)(path), "children", instances);
    }
    remove(path) {
        this.removeInternal(path, true);
    }
    flushIntoStore(store) {
        if (!this.updates.isEmpty() || !this.deletedPaths.isEmpty()) {
            store.dispatch((0, AllActionCreators_1.setValues)(this.updates, [...this.deletedPaths.values()]));
            this.updates = new ModelHunk_1.ModelHunk();
            this.deletedPaths = new AbsoluteResolvedModelPathSet_1.AbsoluteResolvedModelPathSet();
            this.pathWithTouchedErrors.clear();
            this.pathWithTouchedWarnings.clear();
        }
    }
    removeInternal(path, processChildren) {
        this.addEvent((0, ModelEvent_1.createRemoveModelEventInfo)(path));
        const modelValue = this.getViewModel(path);
        if (modelValue == undefined) {
            return;
        }
        for (const [name, value] of (0, IterableUtils_1.iterateEntries)(modelValue)) {
            this.addEvent((0, ModelEvent_1.createRemoveModelEventInfo)(path, name, value));
            if (processChildren) {
                if (name === "children" && modelValue.children != undefined) {
                    for (const child of modelValue.children) {
                        this.removeByPrefix(path.joinWith(child));
                    }
                }
            }
        }
        this.deletedPaths.add(path);
        this.updates.remove(path);
    }
    removeInternalSilently(path) {
        const parentBatch = this.parentModel.getBatch(path);
        const pathsToDelete = IterableUtils_1.IterUtils.concat(Iterator.from(ModelHunk_1.ModelHunk.iterateEntriesNew(parentBatch)).map(x => x[0] ?? (0, TypingUtils_1.reject)()), [path]);
        for (const pathToDelete of pathsToDelete) {
            this.deletedPaths.add(pathToDelete);
        }
    }
    setViewModelFieldInternal(path, name, nextValue) {
        const prevValue = this.getViewModelFieldInternal(path, name);
        if (prevValue === nextValue) {
            return;
        }
        if (name === "error") {
            if (!this.pathWithTouchedErrors.has(path)) {
                this.pathWithTouchedErrors.add(path);
            }
            else {
                if (nextValue == undefined || (Array.isArray(nextValue) && nextValue.length === 0)) {
                    return;
                }
            }
        }
        if (name === "warning") {
            if (!this.pathWithTouchedWarnings.has(path)) {
                this.pathWithTouchedWarnings.add(path);
            }
            else {
                if (nextValue == undefined || (Array.isArray(nextValue) && nextValue.length === 0)) {
                    return;
                }
            }
        }
        const viewModelUpdate = this.updates.getOrCreate(path, {});
        viewModelUpdate[name] = nextValue;
        this.addEvent((0, ModelEvent_1.createChangeModelEventInfo)(path, name, prevValue, nextValue));
    }
    getViewModelFieldInternal(path, fieldName) {
        if (this.deletedPaths.has(path)) {
            // @ts-ignore
            return this.updates.get(path)?.[fieldName];
        }
        // @ts-ignore
        return this.updates.get(path)?.[fieldName] ?? this.parentModel.getViewModelField(path, fieldName);
    }
    removeByPrefix(nodePathPrefix) {
        const batch = this.parentModel.getBatch(nodePathPrefix);
        const pathFromUpdates = Iterator.from(this.updates.iteratePaths()).filter(x => x.startsWith(nodePathPrefix));
        const pathsToDelete = new Set2_1.Set2(IterableUtils_1.IterUtils.concat(Iterator.from(ModelHunk_1.ModelHunk.iterateEntriesNew(batch)).map(x => x[0] ?? (0, TypingUtils_1.reject)()), [nodePathPrefix], pathFromUpdates));
        for (const path of pathsToDelete.values()) {
            this.removeInternal(path, false);
        }
    }
}
exports.Model = Model;
class EventCollectionContext {
    events = [];
    releaseAction;
    constructor(releaseAction) {
        this.releaseAction = releaseAction;
    }
    addEvent(event) {
        this.events.push(event);
    }
    releaseAndGetEvent() {
        this.releaseAction(this);
        const resultEvents = this.events;
        this.events = [];
        this.releaseAction = () => {
            throw new Errors_1.InvalidProgramStateError();
        };
        return resultEvents;
    }
}
