"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelHunk = void 0;
exports.isModelHunk = isModelHunk;
const StringHelpers_1 = require("../../../Helpers/StringHelpers");
const IterableUtils_1 = require("../../../../../Common/IterableUtils");
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const ServerRawResponse_1 = require("../../../../../Common/CandyApiClient/Types/GetInner/ServerRawResponse");
const ViewModelHelpers_1 = require("../../../Helpers/ViewModelHelpers");
function isModelHunk(modelChunkOrHunk) {
    return modelChunkOrHunk instanceof ModelHunk;
}
class ModelHunk {
    tokenNode = new Map();
    static create() {
        return {};
    }
    static paths(chunk) {
        return Iterator.from(this.iteratePaths(chunk)).toArray();
    }
    static iteratePaths(chunk) {
        if (isModelHunk(chunk)) {
            return Iterator.from(chunk.iteratePaths()).map(x => x.toLegacyPath());
        }
        else {
            return (0, IterableUtils_1.iterateKeys)(chunk);
        }
    }
    static get(chunk, path) {
        if (isModelHunk(chunk)) {
            return chunk.get((0, ModelPath_1.createModelPath)(path, true));
        }
        else {
            return chunk[path];
        }
    }
    static set(chunk, path, viewModel) {
        if (isModelHunk(chunk)) {
            chunk.set((0, ModelPath_1.convertToAbsoluteModelPath)(path), viewModel);
        }
        else {
            chunk[(0, ModelPath_1.convertToLegacyPath)(path)] = viewModel;
        }
    }
    static updateIfExists(chunk, path, updateFunc) {
        if (isModelHunk(chunk)) {
            chunk.updateIfExists((0, ModelPath_1.createModelPath)(path, true), updateFunc);
        }
        else {
            const viewModel = chunk[path];
            if (viewModel) {
                updateFunc(viewModel);
            }
        }
    }
    static *iterateEntries(chunk) {
        if (isModelHunk(chunk)) {
            yield* Iterator.from(chunk.iterateEntries()).map(([path, viewModel]) => [
                path.toLegacyPath(),
                viewModel,
            ]);
        }
        else {
            for (const [path, value] of (0, IterableUtils_1.iterateEntries)(chunk)) {
                if (value != undefined) {
                    yield [path, value];
                }
            }
        }
    }
    static *iterateEntriesNew(chunk) {
        if (isModelHunk(chunk)) {
            yield* chunk.iterateEntries();
        }
        else {
            for (const [path, value] of (0, IterableUtils_1.iterateEntries)(chunk)) {
                if (value != undefined) {
                    yield [(0, ModelPath_1.createModelPath)(path, true), value];
                }
            }
        }
    }
    static isEmpty(chunk) {
        if (isModelHunk(chunk)) {
            return chunk.isEmpty();
        }
        else {
            return IterableUtils_1.IterUtils.isEmpty((0, IterableUtils_1.iterateKeys)(chunk));
        }
    }
    static hasPath(chunk, path) {
        if (isModelHunk(chunk)) {
            return chunk.hasPath((0, ModelPath_1.createModelPath)(path, true));
        }
        else {
            return Boolean(chunk[path]);
        }
    }
    static remove(chunk, path) {
        if (isModelHunk(chunk)) {
            chunk.remove((0, ModelPath_1.createModelPath)(path, true));
        }
        else {
            delete chunk[path];
        }
    }
    static createFromValuesBatch(valuesBatch) {
        return valuesBatch;
    }
    static createHunkFromValuesBatch(valuesBatch) {
        const result = new ModelHunk();
        for (const [path, value] of (0, IterableUtils_1.iterateEntries)(valuesBatch)) {
            result.set((0, ModelPath_1.createModelPath)(path, true), value);
        }
        return result;
    }
    static toValuesBatch(chunk) {
        return chunk;
    }
    getTokenNodes() {
        return this.tokenNode;
    }
    isEmpty() {
        return this.tokenNode.size === 0;
    }
    *iteratePaths() {
        yield* this.iterateChildTokenPaths(undefined, this.tokenNode);
    }
    *iterateEntries() {
        yield* this.iterateChildTokenEntries(undefined, this.tokenNode);
    }
    updateIfExists(path, updateFunc) {
        const currentNode = this.getTokenNode(path);
        if (currentNode?.viewModel != undefined) {
            updateFunc(currentNode.viewModel);
        }
    }
    set(path, viewModel) {
        const currentNode = this.getOrCreateTokenNode(path);
        currentNode.viewModel = viewModel;
    }
    getOrCreate(path, viewModel) {
        const currentNode = this.getOrCreateTokenNode(path);
        return currentNode.viewModel ?? (currentNode.viewModel = viewModel);
    }
    remove(path) {
        let currentNodeParentMap;
        let currentNode;
        let currentNodeToken;
        for (const pathToken of path.getPathParts()) {
            if (!ModelPath_1.PathTokens.isSimpleToken(pathToken)) {
                throw new Error("Path spec is not supported in model hunk");
            }
            if (currentNode == undefined) {
                const nextNode = this.tokenNode.get(pathToken);
                if (nextNode == undefined) {
                    return;
                }
                currentNodeToken = pathToken;
                currentNodeParentMap = this.tokenNode;
                currentNode = nextNode;
            }
            else {
                if (currentNode.childNodes == undefined) {
                    currentNode.childNodes = new Map();
                }
                const nextNode = currentNode.childNodes.get(pathToken);
                if (nextNode == undefined) {
                    return;
                }
                currentNodeToken = pathToken;
                currentNodeParentMap = currentNode.childNodes;
                currentNode = nextNode;
            }
        }
        if (currentNodeParentMap != undefined && currentNodeToken != undefined) {
            currentNodeParentMap.delete(currentNodeToken);
        }
    }
    toModelChunk() {
        const result = ModelHunk.create();
        for (const [path, value] of this.iterateEntries()) {
            ModelHunk.set(result, path.toLegacyPath(), value);
        }
        return result;
    }
    get(path) {
        const currentNode = this.getTokenNode(path);
        return currentNode?.viewModel;
    }
    hasPath(path) {
        return this.getTokenNode(path)?.viewModel != undefined;
    }
    *iterateChildTokenPaths(prefix, tokenNodesMap) {
        for (const [token, tokenNode] of tokenNodesMap) {
            const currentPath = prefix == undefined ? (0, ModelPath_1.createModelPath)(token, true) : prefix.joinWith(token);
            if (tokenNode.viewModel != undefined) {
                yield currentPath;
            }
            if (tokenNode.childNodes != undefined) {
                yield* this.iterateChildTokenPaths(currentPath, tokenNode.childNodes);
            }
        }
    }
    *iterateChildTokenEntries(prefix, tokenNodesMap) {
        for (const [token, tokenNode] of tokenNodesMap) {
            const currentPath = prefix == undefined ? (0, ModelPath_1.createModelPath)(token, true) : prefix.joinWith(token);
            if (tokenNode.viewModel != undefined) {
                yield [currentPath, tokenNode.viewModel];
            }
            if (tokenNode.childNodes != undefined) {
                yield* this.iterateChildTokenEntries(currentPath, tokenNode.childNodes);
            }
        }
    }
    getOrCreateTokenNode(path) {
        let currentNode;
        for (const pathToken of path.getPathParts()) {
            if (!ModelPath_1.PathTokens.isSimpleToken(pathToken)) {
                throw new Error("Path spec is not supported in model hunk");
            }
            if (currentNode == undefined) {
                let nextNode = this.tokenNode.get(pathToken);
                if (nextNode == undefined) {
                    nextNode = {};
                    this.tokenNode.set(pathToken, nextNode);
                }
                currentNode = nextNode;
            }
            else {
                if (currentNode.childNodes == undefined) {
                    currentNode.childNodes = new Map();
                }
                let nextNode = currentNode.childNodes.get(pathToken);
                if (nextNode == undefined) {
                    nextNode = {};
                    currentNode.childNodes.set(pathToken, nextNode);
                }
                currentNode = nextNode;
            }
        }
        if (currentNode == undefined) {
            throw new Error("Empty path is invalid");
        }
        return currentNode;
    }
    getTokenNode(path) {
        let currentNode;
        for (const pathToken of path.getPathParts()) {
            if (!ModelPath_1.PathTokens.isSimpleToken(pathToken)) {
                throw new Error("Path spec is not supported in model hunk");
            }
            if (currentNode == undefined) {
                const nextNode = this.tokenNode.get(pathToken);
                if (nextNode == undefined) {
                    return undefined;
                }
                currentNode = nextNode;
            }
            else {
                if (currentNode.childNodes == undefined) {
                    currentNode.childNodes = new Map();
                }
                const nextNode = currentNode.childNodes.get(pathToken);
                if (nextNode == undefined) {
                    return undefined;
                }
                currentNode = nextNode;
            }
        }
        return currentNode;
    }
    mergeIntoViewModel(nextViewModelState) {
        if (this.isEmpty()) {
            return nextViewModelState;
        }
        const result = { ...nextViewModelState };
        for (const [childToken, childNode] of this.tokenNode.entries()) {
            // @ts-ignore child token не содержит ., поэтому тут всё норм
            result[childToken] = this.mergeIntoViewModelInternal(result[childToken], childNode);
        }
        return result;
    }
    fillFromServerJsonResponse(serverResponse) {
        if (!this.isEmpty()) {
            throw new Error("Hunk must be empty before fill with server data");
        }
        function buildNodeFromServerResponse(value) {
            const result = {};
            for (const [fieldName, fieldValue] of (0, IterableUtils_1.iterateEntries)(value)) {
                if (fieldName === ".") {
                    // @ts-expect-error
                    result.viewModel = fieldValue;
                }
                else {
                    if (result.childNodes == undefined) {
                        result.childNodes = new Map();
                    }
                    // @ts-expect-error
                    result.childNodes.set(fieldName, buildNodeFromServerResponse(fieldValue));
                }
            }
            return result;
        }
        for (const [fieldName, value] of (0, IterableUtils_1.iterateEntries)(serverResponse)) {
            this.tokenNode.set(fieldName, buildNodeFromServerResponse(value));
        }
    }
    fillFromServerRawResponse(serverResponse) {
        if (!this.isEmpty()) {
            throw new Error("Hunk must be empty before fill with server data");
        }
        const fillTokenNodesMap = (pathTokens, viewModel, tokenNodes) => {
            if (pathTokens.length === 0 || pathTokens[0] == undefined) {
                return;
            }
            const pathToken = pathTokens[0];
            if (!tokenNodes.has(pathToken)) {
                tokenNodes.set(pathToken, {});
            }
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            const tokenNode = tokenNodes.get(pathToken);
            if (pathTokens.length === 1) {
                tokenNode.viewModel = viewModel;
            }
            else {
                if (tokenNode.childNodes == undefined) {
                    tokenNode.childNodes = new Map();
                }
                const remainedPath = pathTokens.slice(1);
                fillTokenNodesMap(remainedPath, viewModel, tokenNode.childNodes);
            }
        };
        const buildRootNodeFromRawServerResponse = (item) => {
            for (const [path, stringViewModel] of (0, IterableUtils_1.iterateEntries)((0, ViewModelHelpers_1.getPathsViewModels)(item["@"]))) {
                const viewModel = (0, ViewModelHelpers_1.parseStringViewModel)(stringViewModel);
                fillTokenNodesMap(path.split("/"), viewModel, this.tokenNode);
            }
        };
        const buildNodeFromRawServerResponse = (item) => {
            for (const [pathWithInstance, model] of (0, IterableUtils_1.iterateEntries)(item)) {
                const [multiplicityPath, instance] = (0, StringHelpers_1.splitLastOccurrence)(pathWithInstance, "@");
                const pathTokensBase = multiplicityPath
                    .split("/")
                    .concat(Number(instance).toString());
                for (const [viewModelPath, stringViewModel] of (0, IterableUtils_1.iterateEntries)((0, ViewModelHelpers_1.getPathsViewModels)(model))) {
                    const instancePathTokens = pathTokensBase
                        .concat(viewModelPath.split("/"))
                        .filter(Boolean);
                    const viewModel = (0, ViewModelHelpers_1.parseStringViewModel)(stringViewModel);
                    fillTokenNodesMap(instancePathTokens, viewModel, this.tokenNode);
                }
            }
        };
        for (const item of serverResponse) {
            if ((0, ServerRawResponse_1.isServerRawResponseRootItem)(item)) {
                buildRootNodeFromRawServerResponse(item);
            }
            else {
                buildNodeFromRawServerResponse(item);
            }
        }
    }
    mergeIntoViewModelInternal(nodeViewModel, tokenNode) {
        const result = { ...nodeViewModel };
        if (tokenNode.viewModel != undefined) {
            result["."] = { ...(result["."] ?? {}), ...tokenNode.viewModel };
        }
        if (tokenNode.childNodes != undefined) {
            for (const [childToken, childNode] of tokenNode.childNodes.entries()) {
                // @ts-ignore child token не содержит ., поэтому тут всё норм
                result[childToken] = this.mergeIntoViewModelInternal(result[childToken], childNode);
            }
        }
        return result;
    }
}
exports.ModelHunk = ModelHunk;
