"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelNormalizer = void 0;
const IterableUtils_1 = require("../../../../../Common/IterableUtils");
const DataDeclaration_1 = require("../DataDeclaration/DataDeclaration");
const MaskResolver_1 = require("../PathUtils/MaskResolver");
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const Errors_1 = require("../../../../../Common/Errors");
class ModelNormalizer {
    dataDeclarationHelper;
    isReadOnly;
    fetchFunctionCallContextProvider;
    sections;
    constructor(fetchFunctionCallContextProvider, dataDeclarationHelper, sections, isReadOnly) {
        this.isReadOnly = isReadOnly;
        this.fetchFunctionCallContextProvider = fetchFunctionCallContextProvider;
        this.dataDeclarationHelper = dataDeclarationHelper;
        this.sections = sections;
    }
    normalizeSection(model, sectionName) {
        this.sections.removeSectionFromIgnore(sectionName);
        const sectionPrefix = this.sections.getSectionPrefix(sectionName);
        if (sectionPrefix != undefined) {
            this.normalizeByPrefix(model, sectionPrefix);
        }
        else {
            const masks = this.sections.masks(sectionName);
            this.normalizeTrie(model, this.buildPathsTrie(masks));
        }
    }
    normalizeByPrefix(model, nodePathPrefix) {
        const paths = MaskResolver_1.MaskResolver.getPathsByPrefix(this.dataDeclarationHelper, model, nodePathPrefix);
        this.normalizeTrie(model, this.buildPathsTrie([...paths]));
    }
    normalizeTrie(model, trie) {
        this.normalizeTrieNode(model, trie, (0, ModelPath_1.emptyAbsoluteModelPath)());
    }
    normalizeTrieNode(model, trieNode, nodePath) {
        if (trieNode.leafNode) {
            if (!this.sections.isIgnoredPathByMasks(nodePath)) {
                this.normalizeViewModelAtPath(model, nodePath);
            }
            if (this.dataDeclarationHelper.isNodeHasChildrenEntry(nodePath)) {
                const instances = model.getChildren(nodePath) ?? [];
                for (const instance of instances) {
                    this.normalizeByPrefix(model, nodePath.joinWith(instance));
                }
            }
        }
        this.normalizeDescendants(model, nodePath, trieNode);
    }
    normalizeDescendants(model, nodePath, trieNode) {
        for (const [pathToken, trieChild] of trieNode.children.entries()) {
            if (ModelPath_1.PathTokens.isInstanceToken(pathToken) && trieNode.children.has(ModelPath_1.PathTokens.each)) {
                continue;
            }
            if (ModelPath_1.PathTokens.isMultiToken(pathToken)) {
                if (!nodePath.isResolved()) {
                    throw new Errors_1.InvalidProgramStateError();
                }
                const instances = model.getChildren(nodePath) ?? [];
                for (const instance of instances) {
                    this.normalizeTrieNode(model, trieChild, nodePath.joinWith(instance));
                }
            }
            else {
                this.normalizeTrieNode(model, trieChild, nodePath.joinWith(pathToken));
            }
        }
    }
    buildPathsTrie(paths) {
        const trieRootNode = { children: new Map() };
        for (const path of paths) {
            let node = trieRootNode;
            for (const token of path.getPathParts()) {
                let nextNode = node.children.get(token);
                if (nextNode == undefined) {
                    nextNode = { children: new Map() };
                    node.children.set(token, nextNode);
                }
                node = nextNode;
            }
            node.leafNode = true;
        }
        return trieRootNode;
    }
    normalizeViewModelAtPath(model, nodePath) {
        const nodeDataDeclaration = this.dataDeclarationHelper.getEntry(nodePath);
        const viewModelUpdates = {};
        for (const [fieldName, dataSources] of (0, IterableUtils_1.iterateEntries)(nodeDataDeclaration)) {
            if (dataSources != undefined &&
                this.isAttributeRequireNormalization(fieldName, nodeDataDeclaration, model, nodePath)) {
                this.normalizeField(model, nodePath, fieldName, dataSources, viewModelUpdates);
            }
        }
        for (const [field, value] of (0, IterableUtils_1.iterateEntries)(viewModelUpdates)) {
            if (field === "children") {
                // NOTE Просто поверим, что всё получится
                const instances = value;
                model.setChildrenWithoutNormalization(nodePath, instances);
            }
            else {
                model.setViewModelField(nodePath, field, value);
            }
        }
    }
    isAttributeRequireNormalization(fieldName, nodeDataDeclaration, model, nodePath) {
        return ((0, DataDeclaration_1.isAttributeRequireNormalization)(fieldName) &&
            !(fieldName === "value" &&
                nodeDataDeclaration.disableValueAutoInit &&
                model.getViewModelField(nodePath, "disableValueAutoInit") === "true"));
    }
    normalizeField(model, nodePath, fieldName, dataSources, viewModelUpdates) {
        for (const dataSource of dataSources) {
            if ((0, DataDeclaration_1.isConstValueSource)(dataSource)) {
                // @ts-ignore
                viewModelUpdates[fieldName] = dataSource;
                break;
            }
            else if ((0, DataDeclaration_1.isTakeFromModelValueSource)(dataSource)) {
                const modelValue = model.getViewModelField(nodePath, fieldName);
                if (modelValue != undefined) {
                    break;
                }
            }
            else if ((0, DataDeclaration_1.isFetchFunctionValueSource)(dataSource)) {
                if (this.isReadOnly) {
                    break;
                }
                const fnResult = dataSource.fn.call(this.fetchFunctionCallContextProvider.getFetchFunctionCallContext(model), nodePath.toLegacyPath());
                if (fnResult !== undefined) {
                    // @ts-ignore
                    viewModelUpdates[fieldName] = fnResult;
                    break;
                }
            }
            else {
                throw new Error(`Unknown data source descriptors in data declaration. FieldName: ${fieldName}. Value source: ${JSON.stringify(dataSource)}`);
            }
        }
    }
}
exports.ModelNormalizer = ModelNormalizer;
