"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NavigationPathMapper = void 0;
const IterableUtils_1 = require("../../../../../Common/IterableUtils");
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const PathUtils_1 = require("./PathUtils");
class NavigationPathMapper {
    dataDeclarationHelper;
    modelAccessor;
    lazyLoadingManager;
    candyApiDraftClient;
    constructor(modelAccessor, dataDeclarationHelper, lazyLoadingManager, candyApiDraftClient) {
        this.candyApiDraftClient = candyApiDraftClient;
        this.modelAccessor = modelAccessor;
        this.dataDeclarationHelper = dataDeclarationHelper;
        this.lazyLoadingManager = lazyLoadingManager;
    }
    isCandyPath(path) {
        const pathStr = (0, ModelPath_1.isModelPath)(path) ? path.toLegacyPath() : path;
        return this.dataDeclarationHelper.hasMask(pathStr.startsWith("/") ? pathStr.slice(1) : pathStr);
    }
    mapPath(outerPath) {
        const resultPathParts = [];
        const tokens = this.removeLastInnerTextToken(this.splitToPathTokens(outerPath));
        const isCandyPath = this.isCandyPath(outerPath);
        const tokenStep = isCandyPath ? 1 : 2;
        for (let tokenIndex = 0; tokenIndex < tokens.length; tokenIndex += tokenStep) {
            const pathPart = tokens[tokenIndex] ?? (0, TypingUtils_1.reject)(`Invalid path ${outerPath}`);
            const filterOrIndex = tokens[tokenIndex + 1];
            resultPathParts.push(pathPart);
            const currentPath = PathUtils_1.PathUtils.join(resultPathParts);
            if (this.dataDeclarationHelper.isNodeHasChildrenEntry(currentPath)) {
                if (filterOrIndex != undefined && this.isInstanceIndex(filterOrIndex)) {
                    const highlanderInstanceIndex = this.parseInstanceIndex(filterOrIndex);
                    const instances = this.modelAccessor.getChildren(currentPath);
                    const isLastToken = tokenIndex === tokens.length - 1 || tokenIndex === tokens.length - 2;
                    const mappedInstance = isCandyPath
                        ? instances?.[highlanderInstanceIndex]
                        : instances?.[highlanderInstanceIndex - 1];
                    if (mappedInstance != undefined) {
                        resultPathParts.push(mappedInstance);
                        if (isCandyPath) {
                            tokenIndex += 1;
                        }
                    }
                    else if (!isLastToken && !isCandyPath) {
                        resultPathParts.push((highlanderInstanceIndex - 1).toString());
                    }
                }
                else if (filterOrIndex != undefined) {
                    const [instanceFilters, indexAfterFilter] = this.extractFilters(filterOrIndex);
                    const candyInstanceIndex = this.findIndexMatchTo(currentPath, instanceFilters, indexAfterFilter);
                    resultPathParts.push(candyInstanceIndex).toString();
                }
            }
        }
        return resultPathParts.join("/");
    }
    async mapPathWithLazyLoading(outerPath) {
        const tokens = this.removeLastInnerTextToken(this.splitToPathTokens(outerPath));
        const correctedPath = tokens.join("/");
        const isCandyPath = this.isCandyPath(outerPath);
        const mappedPath = isCandyPath
            ? correctedPath
            : (await this.candyApiDraftClient.resolveInnerPath({ errorLogPath: correctedPath })).innerPath;
        await this.lazyLoadingManager.requestFullInstanceLoadForCustomPath((0, ModelPath_1.createAbsolute)(mappedPath));
        return mappedPath;
    }
    splitToPathTokens(path) {
        const tokens = [];
        let openBracketCount = 0;
        let currentToken = "";
        let isPrevBackSlash = false;
        for (const currentChar of path) {
            if (currentChar != undefined) {
                if (currentChar === "/" && openBracketCount === 0 && !isPrevBackSlash) {
                    tokens.push(currentToken);
                    currentToken = "";
                }
                else {
                    if (currentChar === "[") {
                        openBracketCount++;
                    }
                    if (currentChar === "]") {
                        openBracketCount--;
                    }
                    if (currentChar === "\\") {
                        isPrevBackSlash = true;
                    }
                    else {
                        if (isPrevBackSlash && currentChar !== "/") {
                            currentToken += "\\";
                        }
                        currentToken = currentToken + currentChar;
                        isPrevBackSlash = false;
                    }
                }
            }
        }
        tokens.push(currentToken);
        return tokens.filter(TypingUtils_1.isNotNullOrEmpty);
    }
    getTextsBetweenBrackets(str) {
        const allWordsBetweenBracketsRegex = /\[([^\]]+)\]/g;
        const results = [];
        while (true) {
            const nextMatch = allWordsBetweenBracketsRegex.exec(str.toString());
            if (nextMatch == undefined || nextMatch[1] == undefined) {
                break;
            }
            results.push(nextMatch[1]);
        }
        return results;
    }
    removeLastInnerTextToken(tokens) {
        if (tokens[tokens.length - 1] === "innerText") {
            return tokens.slice(0, -1);
        }
        return tokens;
    }
    isInstanceIndex(filterOrIndex) {
        return /^\d+$/.test(filterOrIndex);
    }
    parseInstanceIndex(filterOrIndex) {
        return Number(filterOrIndex);
    }
    extractFilters(filterOrIndex) {
        const filters = this.getTextsBetweenBrackets(filterOrIndex);
        if (!(0, TypingUtils_1.arrayHasAtLeastOneItem)(filters)) {
            throw new Error(`'${filterOrIndex}' must be expression in brackets`);
        }
        const lastFilter = IterableUtils_1.IterUtils.last(filters);
        let index = "1";
        if (!this.isPathFilterExpression(lastFilter)) {
            index = lastFilter;
            filters.pop();
        }
        return [filters.map(x => this.createPathFilter(x)), index];
    }
    isPathFilterExpression(pathFilterOrIndex) {
        return pathFilterOrIndex.includes("=");
    }
    createPathFilter(pathFilterExpression) {
        const [path, value] = pathFilterExpression.split("=");
        if (path == undefined || value == undefined) {
            throw new Error(`'${pathFilterExpression}' must be a path filter expression`);
        }
        const patchedpath = this.simplePathMap(path);
        return new PathFilter(patchedpath, value);
    }
    simplePathMap(path) {
        return this.removeLastInnerTextToken(this.splitToPathTokens(path))
            .filter(t => t !== "1")
            .join("/");
    }
    *filter(currentPath, instanceFilters) {
        const instances = this.modelAccessor.getChildren(currentPath) ?? [];
        for (const instance of instances) {
            const instancePath = PathUtils_1.PathUtils.join([currentPath, instance]);
            if (instanceFilters.every(filter => filter.execute(this.modelAccessor, instancePath))) {
                yield instance;
            }
        }
    }
    elementAt(items, index) {
        let currentIndex = 0;
        for (const item of items) {
            if (currentIndex === index) {
                return item;
            }
            currentIndex++;
        }
        return undefined;
    }
    findIndexMatchTo(currentPath, instanceFilters, highlanderTargetIndex) {
        const filteredInstances = this.filter(currentPath, instanceFilters);
        return this.elementAt(filteredInstances, this.parseInstanceIndex(highlanderTargetIndex) - 1) ?? "0";
    }
}
exports.NavigationPathMapper = NavigationPathMapper;
class PathFilter {
    path;
    value;
    constructor(path, value) {
        this.path = path;
        this.value = value;
    }
    execute(modelAccessor, instancePath) {
        const path = instancePath + "/" + this.path;
        return modelAccessor.getValue(path) === this.value;
    }
}
