"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomValueValidator = void 0;
const IValueValidator_1 = require("./Bases/IValueValidator");
const ValueValidatorBase_1 = require("./Bases/ValueValidatorBase");
class CustomValidationErrorManagerWithEvents {
    #errorManager;
    innerProxy;
    model;
    events = [];
    eventCollectingEnabled = true;
    targetPath;
    result;
    constructor(errorManager, innerProxy, model, targetPath) {
        this.model = model;
        this.#errorManager = errorManager;
        this.innerProxy = innerProxy;
        this.targetPath = targetPath;
    }
    getEvents() {
        return this.events;
    }
    getResult() {
        return this.result;
    }
    stopEventCollecting() {
        this.eventCollectingEnabled = false;
    }
    addError(nodePath, message) {
        if (this.eventCollectingEnabled) {
            if (this.targetPath === nodePath) {
                this.result = IValueValidator_1.ValidationResults.Invalid(message, "error");
            }
            else {
                this.#errorManager.addErrorCached(this.model, nodePath, message);
            }
        }
        else {
            this.innerProxy.addError(nodePath, message);
        }
    }
    addWarning(nodePath, message) {
        if (this.eventCollectingEnabled) {
            if (this.targetPath === nodePath) {
                this.result = IValueValidator_1.ValidationResults.Invalid(message, "warning");
            }
            else {
                this.#errorManager.addWarningCached(this.model, nodePath, message);
            }
        }
        else {
            this.innerProxy.addWarning(nodePath, message);
        }
    }
    clearErrors(nodePath) {
        if (this.eventCollectingEnabled) {
            if (this.targetPath === nodePath) {
                this.result = IValueValidator_1.ValidationResults.Valid;
            }
            else {
                this.#errorManager.clearErrorsCached(this.model, nodePath);
            }
        }
        else {
            this.innerProxy.clearErrors(nodePath);
        }
    }
    clearWarnings(nodePath) {
        if (this.eventCollectingEnabled) {
            if (this.targetPath === nodePath) {
                this.result = IValueValidator_1.ValidationResults.Valid;
            }
            else {
                this.#errorManager.clearWarningsCached(this.model, nodePath);
            }
        }
        else {
            this.innerProxy.clearWarnings(nodePath);
        }
    }
    removeWarning(nodePath, message) {
        if (this.eventCollectingEnabled) {
            if (this.targetPath === nodePath) {
                this.result = IValueValidator_1.ValidationResults.Valid;
            }
            else {
                this.#errorManager.clearWarningsCached(this.model, nodePath);
            }
        }
        else {
            this.innerProxy.removeWarning(nodePath, message);
        }
    }
}
class CustomValueValidator extends ValueValidatorBase_1.ValueValidatorBase {
    customValidationFunction;
    message;
    checkType;
    constructor(customValidationFunction, checkType, message) {
        super();
        this.message = message;
        this.customValidationFunction = customValidationFunction;
        this.checkType = checkType;
    }
    validate(context, nodeViewModel) {
        const callContext = context.customValidationFunctionCallContextProvider.getCustomValidationFunctionCallContext(context.model);
        const providerWithEvents = new CustomValidationErrorManagerWithEvents(context.errorManager, callContext.errorManager, context.model, context.path);
        const result = this.customValidationFunction(nodeViewModel, {}, context.path, {
            ...callContext,
            errorManager: providerWithEvents,
        });
        providerWithEvents.stopEventCollecting();
        const additionalEvents = providerWithEvents.getEvents();
        const implicityAppliedResult = providerWithEvents.getResult();
        if (implicityAppliedResult != undefined) {
            return IValueValidator_1.ValidationResults.withEvents(implicityAppliedResult, additionalEvents);
        }
        if (result === false) {
            return IValueValidator_1.ValidationResults.ForceValidEvents(additionalEvents);
        }
        if (result !== true) {
            const message = typeof result === "string" ? result : undefined;
            return IValueValidator_1.ValidationResults.Invalid(this.getMessage(context, this.message) ?? message, this.checkType, additionalEvents);
        }
        return IValueValidator_1.ValidationResults.ValidWithEvents(additionalEvents);
    }
}
exports.CustomValueValidator = CustomValueValidator;
