"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ActiveLogHelper = void 0;
const IterableUtils_1 = require("../../../../../Common/IterableUtils");
const ExternalEvents_1 = require("../../Core/EventManagement/ExternalEvents/ExternalEvents");
const ModelPath_1 = require("../../../../../Common/ModelPath/ModelPath");
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const UnitId_1 = require("../../../Controls/FormParts/Unit/UnitId/UnitId");
const SectionNames_1 = require("../../../../../Common/CommonConstants/SectionNames");
const PathUtils_1 = require("../../Core/PathUtils/PathUtils");
const Utils_1 = require("../../Core/ErrorManager/Utils/Utils");
class ActiveLogHelper {
    form;
    constructor(form) {
        this.form = form;
    }
    async navigate(navigationPath) {
        return this.form.lazyLoadingManager.isLazyLoadingEnabled()
            ? this.navigateByHighlanderPaths(Iterator.from(navigationPath.split(Utils_1.splitPathWithComma)))
            : this.navigateByBindingPaths(Iterator.from(navigationPath.split(Utils_1.splitPathWithComma)).map(x => this.form.navigationPathMapper.mapPath(x)));
    }
    async navigateByBindingPaths(paths) {
        for (const path of paths) {
            await this.findUnitAndScroll(path);
            const result = await this.findControlAndFocus(path);
            if ((0, TypingUtils_1.isNotNullOrUndefined)(result)) {
                return result;
            }
        }
        return undefined;
    }
    async navigateByHighlanderPaths(paths) {
        const bindingPaths = await Promise.all(Iterator.from(paths)
            .map(path => this.form.navigationPathMapper.mapPathWithLazyLoading(path))
            .toArray());
        return this.navigateByBindingPaths(bindingPaths);
    }
    findShowOnDemandUnit(sectionNames, path) {
        const unitInfos = this.form.store.getState().unitInfos;
        if (unitInfos == undefined) {
            return undefined;
        }
        for (const sectionName of sectionNames) {
            if (sectionName === "default") {
                continue;
            }
            const unitId = (0, UnitId_1.sectionNameToUnitIdForNavigation)(sectionName);
            const result = this.findUnitInfoById(unitInfos, unitId);
            if ((0, TypingUtils_1.isNotNullOrUndefined)(result)) {
                if (!result.showOnDemand) {
                    continue;
                }
                const canNavigateFromErrorLog = result.onNavigateFromErrorLog?.(this.form.getHelperFunctionsCallContext(), unitId, path);
                if (typeof canNavigateFromErrorLog === "boolean" && !canNavigateFromErrorLog) {
                    continue;
                }
                return unitId;
            }
        }
        return undefined;
    }
    async findUnitAndScroll(path) {
        const sections = this.form.dataDeclarationHelper.getSections(path);
        let unitId = IterableUtils_1.IterUtils.last(sections
            .filter(x => x != SectionNames_1.SectionNames.defaultSection && x != SectionNames_1.SectionNames.requisitesSection)
            .map(UnitId_1.sectionNameToUnitId));
        if ((0, TypingUtils_1.isNotNullOrUndefined)(unitId)) {
            await this.findParentUnitAndScroll(path, unitId);
        }
        unitId = this.findShowOnDemandUnit(sections, path) ?? unitId;
        if ((0, TypingUtils_1.isNotNullOrUndefined)(unitId)) {
            await this.form.unitNavigator?.scrollToUnit(unitId);
        }
    }
    async findParentUnitAndScroll(path, unitId) {
        if (unitId?.parentInstances?.length) {
            const potentialOtherSectionToken = Array.from(unitId?.parentInstances).shift();
            if ((0, TypingUtils_1.isNotNullOrEmpty)(potentialOtherSectionToken)) {
                const splitArray = PathUtils_1.PathUtils.split(path);
                const otherSectionPath = splitArray.slice(0, splitArray.indexOf(potentialOtherSectionToken) + 1);
                const modelPath = (0, ModelPath_1.createAbsoluteFromTokens)(otherSectionPath).toLegacyPath();
                const potentialOtherSection = this.form.dataDeclarationHelper.getSections(modelPath);
                const parentUnitId = IterableUtils_1.IterUtils.last(potentialOtherSection
                    .filter(x => x != SectionNames_1.SectionNames.defaultSection && x != SectionNames_1.SectionNames.requisitesSection)
                    .map(UnitId_1.sectionNameToUnitId));
                if ((0, TypingUtils_1.isNotNullOrUndefined)(parentUnitId)) {
                    const preparedUnit = this.prepareParentUnitIdToNavigate(parentUnitId, potentialOtherSectionToken);
                    await this.form.unitNavigator?.scrollToUnit(preparedUnit);
                }
            }
        }
    }
    prepareParentUnitIdToNavigate(unitId, sectionInstanceToken) {
        return { pageId: unitId.pageId, instance: unitId.instance ?? sectionInstanceToken };
    }
    async findControlAndFocus(path) {
        // /0 - тут в качестве #innertext у некоторых компонент, например picklist.
        // Его надо отсечь для мэтчина с componentHandleRegistry
        const normalizedPath = path.endsWith("/0") ? path.slice(0, -2) : path;
        const { multiplicityPath, resultPaths: navigationPaths } = this.getPathParts(normalizedPath);
        if (multiplicityPath) {
            await this.navigateByMask(multiplicityPath, path);
        }
        let lastComponent;
        for (const currPath of navigationPaths) {
            const componentPath = this.cutLastInstanceFromPath(currPath, path);
            let components = this.form.componentHandleRegistry.getComponentHandles(componentPath);
            if (components == undefined || components.length === 0) {
                await this.waitMountComponent(componentPath);
                components = this.form.componentHandleRegistry.getComponentHandles(componentPath);
            }
            if (navigationPaths[navigationPaths.length - 1] === currPath &&
                (components == undefined || components.length === 0)) {
                if (currPath.startsWith("/")) {
                    components = this.form.componentHandleRegistry.getComponentHandles(currPath.substring(1));
                }
                else {
                    components = this.form.componentHandleRegistry.getComponentHandles(currPath);
                }
            }
            components = [...(components ?? [])];
            lastComponent = IterableUtils_1.IterUtils.first(components.filter(TypingUtils_1.isNotNullOrUndefined));
            for (const component of components) {
                await component.makePathAvailableForFocus((0, ModelPath_1.createModelPath)(currPath, true).toAbsolute());
                component.focus(currPath);
            }
        }
        return lastComponent;
    }
    async navigateByMask(currentPath, navigationPath) {
        const componentPath = this.cutLastInstanceFromPath(currentPath, navigationPath);
        let maskPath = this.form.dataDeclarationHelper.getMask(componentPath);
        if (maskPath) {
            maskPath = PathUtils_1.PathUtils.trimLastStarIfLastToken(maskPath);
            const componentByMask = this.form.componentHandleRegistry.getComponentHandles(maskPath) ?? [];
            for (const component of componentByMask) {
                await this.waitMountComponent(navigationPath);
                await component.makePathAvailableForFocus((0, ModelPath_1.createModelPath)(currentPath, true).toAbsolute());
                component.focus(currentPath);
            }
        }
    }
    getPathParts(path) {
        const emptyReturn = {
            multiplicityPath: "",
            resultPaths: [],
        };
        const allPartsPath = Iterator.from((0, ModelPath_1.createAbsolute)(path).getPathParts()).toArray();
        const partsPathBeforeMultiplicity = [];
        allPartsPath.forEach((item, index, array) => {
            const itemAbsolute = (0, ModelPath_1.createAbsolute)(item);
            if (index === 0) {
                partsPathBeforeMultiplicity.push(itemAbsolute);
            }
            else {
                const lastIndex = partsPathBeforeMultiplicity.length - 1;
                if (!isNaN(Number(array[index - 1]))) {
                    partsPathBeforeMultiplicity.push(itemAbsolute);
                }
                else {
                    const lastItem = partsPathBeforeMultiplicity[lastIndex];
                    if (lastItem !== undefined) {
                        partsPathBeforeMultiplicity[lastIndex] = lastItem.joinWith(item);
                    }
                }
            }
        });
        const multiplicityPathArray = partsPathBeforeMultiplicity;
        const lastMultiplicity = multiplicityPathArray.pop();
        const multiplicityPath = multiplicityPathArray.map(i => i.toLegacyPath()).join("/");
        if (lastMultiplicity == undefined) {
            return emptyReturn;
        }
        const tokensInLastMultiplicity = Iterator.from(lastMultiplicity.getPathParts()).toArray();
        if (tokensInLastMultiplicity == undefined) {
            return emptyReturn;
        }
        const partsPathToBeJoined = partsPathBeforeMultiplicity.concat(tokensInLastMultiplicity.map(i => (0, ModelPath_1.createAbsolute)(i)));
        let currentJoinedPath = "";
        const resultPaths = [];
        partsPathToBeJoined.forEach(item => {
            currentJoinedPath = `${currentJoinedPath}${item}`;
            resultPaths.push(currentJoinedPath);
        });
        return {
            multiplicityPath: multiplicityPath,
            resultPaths: resultPaths,
        };
    }
    cutLastInstanceFromPath(currentPath, navigationPath) {
        const match = /([^/]*)$/.exec(currentPath);
        if (match == undefined) {
            throw new Error("Invalid path");
        }
        const componentIndex = Number(match[0]);
        let componentPath = isNaN(componentIndex) || currentPath === navigationPath
            ? currentPath
            : currentPath.replace(/\/[^/]+$/g, "");
        if (componentPath.startsWith("/")) {
            componentPath = componentPath.substring(1);
        }
        return componentPath;
    }
    waitMountComponent(componentPath) {
        return new Promise(resolve => {
            const timeout = setTimeout(() => {
                this.form.offExternalEvent((0, ExternalEvents_1.formComponentDidMountEventPath)(componentPath), focusEvent);
                resolve();
            }, 100);
            const focusEvent = () => {
                this.form.offExternalEvent((0, ExternalEvents_1.formComponentDidMountEventPath)(componentPath), focusEvent);
                clearTimeout(timeout);
                resolve();
            };
            this.form.onExternalEvent((0, ExternalEvents_1.formComponentDidMountEventPath)(componentPath), focusEvent);
        });
    }
    findUnitInfoById(unitInfos, unitId) {
        const needMultipleInfo = (0, TypingUtils_1.isNotNullOrUndefined)(unitId.instance);
        return unitInfos.filter(x => x.multiple == needMultipleInfo).find(x => x.id === unitId.pageId);
    }
}
exports.ActiveLogHelper = ActiveLogHelper;
