"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AttachmentsManager = void 0;
const ViewModelSelect_1 = require("../../../Helpers/StateAccessors/ViewModelSelect");
const UUID_1 = require("../../../../../Common/UUID");
const AllActionCreators_1 = require("../../ActionCreators/AllActionCreators");
const ModelHunk_1 = require("../../Core/ModelStoring/ModelHunk");
const PathConstants_1 = require("../../../../../Common/PathConstants");
const AttachmentsActions_1 = require("./Store/AttachmentsActions");
const ConvertTask_1 = require("./Tasks/ConvertTask");
const TaskStorage_1 = require("./Tasks/TaskStorage");
const UploadToDriveTask_1 = require("./Tasks/UploadToDriveTask");
class AttachmentsManager {
    dataDeclarationHelper;
    errorManager;
    store;
    modelAccessor;
    changeSet;
    candyApiDraftClient;
    taskStorage;
    constructor(dataDeclarationHelper, errorManager, store, changeSet, settings, candyApiDraftClient, modelAccessor) {
        this.modelAccessor = modelAccessor;
        this.candyApiDraftClient = candyApiDraftClient;
        this.dataDeclarationHelper = dataDeclarationHelper;
        this.errorManager = errorManager;
        this.store = store;
        this.changeSet = changeSet;
        this.taskStorage = new TaskStorage_1.TaskStorage();
    }
    async addFiles(bindingPath, files) {
        const attachments = await this.createAttachments(bindingPath, files);
        for (const attachment of attachments) {
            if (this.checkAttachmentError(attachment.id)) {
                continue;
            }
            await this.tryUploadToDrive(attachment, bindingPath);
            await this.tryConvertFile(attachment.id, bindingPath);
        }
    }
    async addFile(bindingPath, file) {
        const attachment = this.createAttachment(file, bindingPath);
        await this.createSingleAttachment(attachment, bindingPath);
        if (this.checkAttachmentError(attachment.id)) {
            return;
        }
        await this.tryUploadToDrive(attachment, bindingPath);
        await this.tryConvertFile(attachment.id, bindingPath);
    }
    async removeFileByInstance(bindingPath, instance) {
        const id = this.modelAccessor.getValue(bindingPath.joinWith(instance).joinWith(PathConstants_1.AttachmentInfoPath));
        const task = this.taskStorage.getTask(id);
        if (task) {
            task.cancel();
        }
        if (id != undefined) {
            await this.candyApiDraftClient.removeAttachments([id]);
        }
        this.store.dispatch((0, AllActionCreators_1.removeInstance)(bindingPath, instance));
        if (id != undefined) {
            this.store.dispatch((0, AttachmentsActions_1.removeAttachment)(id));
        }
        await this.changeSet.send();
    }
    async removeFile(pathToAttachment) {
        let id = this.modelAccessor.getValue(pathToAttachment);
        if (id === undefined) {
            const attachInfo = this.store.getState().attachments.byId;
            for (const attachmentId of Object.keys(attachInfo)) {
                if (attachInfo[attachmentId]?.meta.bindingPath === pathToAttachment.toLegacyPath()) {
                    id = attachmentId;
                }
            }
        }
        const task = this.taskStorage.getTask(id);
        if (task) {
            task.cancel();
        }
        if (id != undefined) {
            await this.candyApiDraftClient.removeAttachments([id]);
        }
        this.store.dispatch((0, AllActionCreators_1.removeSingleAttachment)(pathToAttachment));
        if (id != undefined) {
            this.store.dispatch((0, AttachmentsActions_1.removeAttachment)(id));
        }
        await this.changeSet.send();
    }
    async initTasks() {
        const state = this.store.getState();
        const attachmentIds = state.attachments.allIds;
        await Promise.all(attachmentIds.map((attachmentId) => {
            const attachment = state.attachments.byId[attachmentId];
            if (!attachment) {
                return Promise.resolve();
            }
            const { bindingPath } = attachment.meta;
            return this.tryConvertFile(attachmentId, bindingPath);
        }));
    }
    async openAsDraft(gfv, bindingPath) {
        const createResponse = await this.candyApiDraftClient.createDraftAttachment(gfv, bindingPath.toLegacyPath());
        createResponse.meta.pathToAttachment = bindingPath.joinWith(PathConstants_1.AttachmentInfoPath).toLegacyPath();
        createResponse.meta.bindingPath = bindingPath.toLegacyPath();
        createResponse.meta.progress = 100;
        this.store.dispatch((0, AttachmentsActions_1.addAttachments)([createResponse]));
        await this.candyApiDraftClient.createOrUpdateAttachments([createResponse]);
        this.setAttachmentToModel(bindingPath, createResponse);
        await this.changeSet.send();
    }
    async openFufAsDraft(attachInfo) {
        const createResponse = await this.candyApiDraftClient.openLoadedAttachmentAsDraft(attachInfo.id);
        const newAttach = {
            ...attachInfo,
            nsId: createResponse.nsId,
            draftId: createResponse.draftId,
            meta: {
                ...attachInfo.meta,
                attachmentDraftId: createResponse.meta.attachmentDraftId,
                progress: 100,
            },
        };
        this.store.dispatch((0, AttachmentsActions_1.updateAttachment)(newAttach.id, newAttach));
    }
    async loadAsFile(loadedFileProps, gfv, bindingPath) {
        const attachment = this.createAttachment(loadedFileProps.uploadFile, bindingPath, undefined, gfv, loadedFileProps.ndsTotalSum);
        await this.createSingleAttachment(attachment, bindingPath);
        if (this.checkAttachmentError(attachment.id)) {
            return;
        }
        await this.tryUploadToDrive(attachment, bindingPath);
        await this.tryConvertFile(attachment.id, bindingPath);
    }
    validateAttachments() {
        const state = this.store.getState();
        const attachmentIds = state.attachments.allIds;
        for (const attachmentId of attachmentIds) {
            const attachInfo = state.attachments.byId[attachmentId];
            if (attachInfo == undefined) {
                continue;
            }
            const { pathToAttachment } = attachInfo.meta;
            if (pathToAttachment != undefined && !attachInfo.contentUri && !this.checkAttachmentError(attachmentId)) {
                this.modelAccessor.setError(pathToAttachment, "Загрузка была прервана. Загрузите ещё раз");
            }
        }
    }
    async createAttachments(bindingPath, files) {
        const instances = this.modelAccessor.getChildren(bindingPath) ?? [];
        const lastInstances = Number(instances.slice(-1)[0]);
        const attachments = files.map((file, index) => {
            const instance = String(lastInstances + 1 + index || index);
            return this.createAttachment(file, bindingPath, instance);
        });
        this.store.dispatch((0, AttachmentsActions_1.addAttachments)(attachments));
        await this.candyApiDraftClient.createOrUpdateAttachments(attachments);
        this.setAttachmentsToModel(bindingPath, instances, attachments);
        await this.changeSet.send();
        return attachments;
    }
    async createSingleAttachment(attachment, bindingPath) {
        this.store.dispatch((0, AttachmentsActions_1.addAttachments)([attachment]));
        await this.candyApiDraftClient.createOrUpdateAttachments([attachment]);
        this.setAttachmentToModel(bindingPath, attachment);
        await this.changeSet.send();
        return attachment;
    }
    async tryUploadToDrive(attachment, bindingPath) {
        if (attachment.contentUri) {
            return;
        }
        const task = this.taskStorage.getTask(attachment.id);
        if (task) {
            await task.taskPromise;
            return;
        }
        const state = this.store.getState();
        const attachInfo = state.attachments.byId[attachment.id];
        if (!attachInfo || !attachInfo.file) {
            this.modelAccessor.setError(bindingPath, "Загрузка была прервана. Загрузите ещё раз");
            return;
        }
        const { fileName } = attachInfo.meta;
        const isConvert = this.dataDeclarationHelper.isFnsConvertFile(bindingPath, fileName);
        const uploadTask = new UploadToDriveTask_1.UploadToDriveTask(this.store, this.modelAccessor, attachment.id, bindingPath, attachInfo.file, isConvert, (contentUri) => this.writeResult(contentUri, attachment.id, attachInfo.file));
        this.taskStorage.setTask(attachment.id, uploadTask);
        await uploadTask.taskPromise;
        this.taskStorage.deleteTask(attachment.id);
    }
    async writeResult(contentUri, attachmentId, file) {
        const attachment = this.store.getState().attachments.byId[attachmentId];
        if (attachment == undefined) {
            // eslint-disable-next-line no-console
            console.error(`AttachInfo with id ${attachmentId} dont exist in uploadToDrive`, file);
            return Promise.resolve();
        }
        const nextAttachment = {
            ...attachment,
            contentUri: contentUri,
        };
        this.store.dispatch((0, AttachmentsActions_1.addAttachments)([nextAttachment]));
        return await this.candyApiDraftClient.createOrUpdateAttachments([nextAttachment]);
    }
    async tryConvertFile(attachmentId, bindingPath) {
        const attachInfo = this.store.getState().attachments.byId[attachmentId];
        // Загрузка файла отменена
        if (!attachInfo) {
            return;
        }
        const { fileName, isConverted, pathToAttachment } = attachInfo.meta;
        const isConvert = this.dataDeclarationHelper.isFnsConvertFile(bindingPath, fileName);
        if (!isConvert || !attachInfo.contentUri || isConverted || pathToAttachment == undefined) {
            return;
        }
        const task = this.taskStorage.getTask(attachmentId);
        if (task) {
            await task.taskPromise;
            return;
        }
        const convertTask = new ConvertTask_1.ConvertTask(this.store, this.modelAccessor, this.errorManager, this.candyApiDraftClient, attachmentId, pathToAttachment);
        this.taskStorage.setTask(attachmentId, convertTask);
        await convertTask.taskPromise;
        this.taskStorage.deleteTask(attachmentId);
    }
    setAttachmentsToModel(bindingPath, instances, attachments) {
        const modelChunk = ModelHunk_1.ModelHunk.create();
        ModelHunk_1.ModelHunk.set(modelChunk, bindingPath, {
            children: [...instances, ...attachments.map(({ meta }) => meta.instance ?? "")],
        });
        for (const attachment of attachments) {
            if (attachment.meta.pathToAttachment != undefined) {
                ModelHunk_1.ModelHunk.set(modelChunk, attachment.meta.pathToAttachment, { value: attachment.id });
            }
        }
        this.modelAccessor.setBatch(modelChunk);
    }
    setAttachmentToModel(bindingPath, attachment) {
        if (attachment.meta.pathToAttachment == undefined) {
            return;
        }
        const modelChunk = ModelHunk_1.ModelHunk.createFromValuesBatch({
            [attachment.meta.pathToAttachment]: { value: attachment.id },
        });
        this.modelAccessor.setBatch(modelChunk);
    }
    checkAttachmentError(attachmentId) {
        const state = this.store.getState();
        const attachInfo = state.attachments.byId[attachmentId];
        if (attachInfo == undefined) {
            throw new Error(`Attachment with ${attachmentId} does not exist`);
        }
        const pathToAttachment = attachInfo.meta.pathToAttachment;
        if (pathToAttachment == undefined) {
            return false;
        }
        const error = ViewModelSelect_1.currentViewModelSelect.pathViewModel(state, pathToAttachment).error ?? [];
        return Boolean(error.length);
    }
    createAttachment(file, bindingPath, instance, gfv, ndsTotalSum) {
        const id = (0, UUID_1.uuid)();
        const attachmentPath = instance != undefined
            ? bindingPath.joinWith(instance).joinWith(PathConstants_1.AttachmentInfoPath)
            : bindingPath.joinWith(PathConstants_1.AttachmentInfoPath);
        return {
            id: id,
            contentUri: file.contentUri,
            file: file,
            meta: {
                bindingPath: bindingPath.toLegacyPath(),
                pathToAttachment: attachmentPath.toLegacyPath(),
                instance: instance,
                fileName: file.name,
                fileSize: file.size,
                gfv: gfv,
                progress: file.contentUri ? 100 : 0,
                ndsTotalSum: ndsTotalSum,
            },
        };
    }
    getAttachmentIdByPath(path) {
        const state = this.store.getState();
        const id = Object.values(state.attachments.byId)?.find(attachment => attachment?.meta.bindingPath === path)?.id;
        if (!id) {
            throw new Error(`attachment id (${id}) not found`);
        }
        return id;
    }
}
exports.AttachmentsManager = AttachmentsManager;
