"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CandyClientExports = void 0;
exports.isEngineVersionGreaterThanOrEqual = isEngineVersionGreaterThanOrEqual;
const whatwg_fetch_1 = require("whatwg-fetch");
const TypingUtils_1 = require("../../../../Common/TypingUtils");
const LoadIntoDocument_1 = require("../../Helpers/LoadIntoDocument");
const GetClientDraftInfoUrl_1 = require("../../../../Common/CandyApiClient/ClientDraftInfo/GetClientDraftInfoUrl");
const ClientSettings_1 = require("./ClientSettings");
const CreateLegacyPublicForm_1 = require("./LegacyPublicForm/CreateLegacyPublicForm");
function defaultCallback() {
    /* empty */
}
function parseBool(value) {
    if (typeof value === "string") {
        if (value === "false") {
            return false;
        }
        if (value === "true") {
            return true;
        }
        return false;
    }
    return Boolean(value);
}
function url(relativeUrl = "") {
    const baseUrl = global.CF.keformsUrl || "";
    return `${baseUrl}/${relativeUrl}`.replace(/([^:])\/{2,}/g, "$1/");
}
function buildUrl(path, params) {
    if (params === undefined) {
        return path;
    }
    const queryParts = [];
    for (const paramName of Object.keys(params)) {
        const paramValue = params[paramName];
        if (paramValue != undefined) {
            queryParts.push(`${paramName}=${encodeURIComponent(paramValue)}`);
        }
    }
    if (queryParts.length === 0) {
        return path;
    }
    return path + "?" + queryParts.join("&");
}
function isAllowed(domains) {
    return new RegExp(`^(?:${domains.replace(/\./g, "\\.").replace(/\*/g, "[a-z0-9-._]+")})$`).test(global.location.host);
}
function getParameterByName(name) {
    return (global.location.search
        .trim()
        .replace(/^(\?|#|&)/, "")
        .split("&")
        .map(param => param.replace(/\+/g, " ").split("=").map(decodeURIComponent))
        .find(([key]) => key == name) || [])[1];
}
function runParallel(tasks, callback) {
    let completed = 0;
    tasks.forEach(task => {
        task(error => {
            if (completed === tasks.length) {
                return;
            }
            if (error) {
                completed = tasks.length;
                callback(error);
                return;
            }
            completed += 1;
            if (completed === tasks.length) {
                callback();
            }
        });
    });
}
function isEngineVersionGreaterThanOrEqual(left, right) {
    try {
        const leftNumbers = left
            .split(".")
            .slice(0, 3)
            .map(x => parseInt(x, 10));
        const rightNumbers = right
            .split(".")
            .slice(0, 3)
            .map(x => parseInt(x, 10));
        return ((leftNumbers[0] ?? 0) > (rightNumbers[0] ?? 0) ||
            ((leftNumbers[0] ?? 0) === (rightNumbers[0] ?? 0) &&
                ((leftNumbers[1] ?? 0) > (rightNumbers[1] ?? 0) ||
                    ((leftNumbers[1] ?? 0) === (rightNumbers[1] ?? 0) &&
                        (leftNumbers[2] ?? 0) >= (rightNumbers[2] ?? 0)))));
    }
    catch {
        return false;
    }
}
function loadForm(settings, callback) {
    const { gfv, engineVersion } = settings;
    if (global.CF.entryPoint[gfv]) {
        callback();
        return;
    }
    // @ts-ignore
    const isNeedEngine = engineVersion && !global[engineVersion];
    const noStylesForGfv = false; // engineVersion != undefined && (isEngineVersionGreaterThanOrEqual(engineVersion, "4.214.0") || engineVersion === "local");
    const needLoadStylesForGfv = !noStylesForGfv;
    const min = getParameterByName("debug") === "true" ? "" : ".min";
    const tasks = [];
    const params = { resourceZone: settings.resourceZone };
    if (needLoadStylesForGfv) {
        tasks.push(cb => LoadIntoDocument_1.LoadIntoDocument.styleSheet(url(buildUrl(`static/public/styles/${gfv}${min}.css`, params)), true, cb));
    }
    tasks.push(cb => LoadIntoDocument_1.LoadIntoDocument.script(url(buildUrl(`static/public/scripts/${gfv}${min}.js`, params)), cb));
    if (isNeedEngine) {
        tasks.push(cb => LoadIntoDocument_1.LoadIntoDocument.styleSheet(url(buildUrl(`static/public/styles/candy_${engineVersion}${min}.css`, params)), false, cb), cb => LoadIntoDocument_1.LoadIntoDocument.script(url(buildUrl(`static/public/scripts/candy_${engineVersion}${min}.js`, params)), cb));
    }
    runParallel(tasks, callback);
}
function renderForm(pageContent, extendedSettings, callback) {
    const formEntryPoint = global.CF.entryPoint[extendedSettings.gfv] ?? (0, TypingUtils_1.reject)(`Form entry point ${extendedSettings.gfv} not found.`);
    formEntryPoint(pageContent, extendedSettings)
        .then(form => {
        const publicForm = form.publicForm || (0, CreateLegacyPublicForm_1.createLegacyPublicForm)(form);
        callback(null, publicForm);
    })
        .catch(callback);
}
function getEngineEntryPoint(engineVersion) {
    if (engineVersion == undefined) {
        return undefined;
    }
    // @ts-ignore
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return
    return global[engineVersion] && global[engineVersion].default;
}
function start(pageContent, options, callback = defaultCallback) {
    let error = null;
    const { ns, draftId } = options;
    if (!isAllowed(ClientSettings_1.clientSettings.konturDomains)) {
        error = new Error("domain.untrusted");
    }
    else if (!ClientSettings_1.clientSettings.authorized) {
        error = new Error("user.unauthenticated");
    }
    else if (!draftId) {
        error = new Error("options.draftId is required");
    }
    else if (!ns) {
        error = new Error("options.ns is required");
    }
    if (error) {
        callback(error);
        return;
    }
    const resourceZone = options.resourceZone ?? getParameterByName("resourceZone");
    const clientDraftUrl = buildUrl(url((0, GetClientDraftInfoUrl_1.getClientDraftInfoUrl)({ ns: ns, draftId: draftId })), {
        resourceZone: resourceZone,
    });
    const requestOptions = { method: "GET" };
    if (options.authentication == undefined || options.authentication === "browser-credentials-include") {
        requestOptions.credentials = "include";
    }
    else if ("authSid" in options.authentication && options.authentication.authSid != undefined) {
        requestOptions.credentials = "omit";
        requestOptions.headers = requestOptions.headers ?? {};
        // @ts-ignore
        requestOptions.headers["Authorization"] = `AUTH.SID ${options.authentication.authSid}`;
    }
    else {
        requestOptions.credentials = "omit";
    }
    const fetchImpl = options.fetch ?? whatwg_fetch_1.fetch;
    fetchImpl(clientDraftUrl, requestOptions)
        .then(x => x.json())
        .then((clientDraftInfo) => {
        const gfv = clientDraftInfo.gfv;
        const engineVersion = clientDraftInfo.engineVersion;
        const extendedSettings = {
            authorized: ClientSettings_1.clientSettings.authorized,
            cauldronApiUrl: ClientSettings_1.clientSettings.cauldronApiUrl,
            isTestServer: ClientSettings_1.clientSettings.isTestServer,
            renderCachingForIE8: ClientSettings_1.clientSettings.renderCachingForIE8,
            normalizationOnServer: ClientSettings_1.clientSettings.normalizationOnServer,
            normativeUrl: ClientSettings_1.clientSettings.normativeUrl,
            konturDomains: ClientSettings_1.clientSettings.konturDomains,
            showNavigationSidebar: options.showNavigationSidebar ?? true,
            onRemoveAttachment: options.onRemoveAttachment,
            onClearAttachment: options.onClearAttachment,
            ...options,
            ...clientDraftInfo,
            gfv: gfv,
            engineVersion: engineVersion,
            feedbackEnabled: options.feedbackEnabled ?? clientDraftInfo.feedbackEnabled,
            isDisabledCleanForm: options.isDisabledCleanForm ?? false,
            isDisabledCloseButton: options.isDisabledCloseButton ?? false,
            isDisabledCloseAndSaveButton: options.isDisabledCloseAndSaveButton ?? false,
            resourceZone: resourceZone,
        };
        loadForm({
            gfv: extendedSettings.gfv,
            engineVersion: extendedSettings.engineVersion,
            resourceZone: extendedSettings.resourceZone,
            theme: extendedSettings.theme,
        }, loadError => {
            if (loadError || !global.CF.entryPoint[gfv]) {
                callback(new Error(`dynamic loading is not supported for gfv ${gfv}\n${loadError}`));
                return;
            }
            const render = getEngineEntryPoint(engineVersion);
            renderForm(pageContent, {
                ...extendedSettings,
                templatesApi: options.templatesApi,
                renderForm: render,
            }, callback);
        });
    })
        .catch(callback);
}
const CandyClientExports = {
    start: start,
    entryPoint: {},
    partialRenderingForIE8: parseBool("{partialRenderingForIE8}"),
    keformsUrl: "{keformsUrl}",
    staffServiceUrl: "{staffServiceUrl}",
    picklistUrl: "{picklistApiUrl}",
    nidusUrl: "{nidusApiUrl}",
    fsPrinterUrl: "{fsprinterApiUrl}",
};
exports.CandyClientExports = CandyClientExports;
global.CF = CandyClientExports;
