"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HelperFunctionsDateHelpers = exports.comparator = void 0;
exports.parseDateString = parseDateString;
exports.isValidDate = isValidDate;
exports.tryGetValidDateShape = tryGetValidDateShape;
exports.isValidStringDate = isValidStringDate;
exports.isLessOrEqualStringDates = isLessOrEqualStringDates;
exports.getLastDateOfMonth = getLastDateOfMonth;
exports.differenceInMonths = differenceInMonths;
exports.differenceInDays = differenceInDays;
exports.addDays = addDays;
exports.addMonths = addMonths;
exports.addYears = addYears;
exports.dateToString = dateToString;
exports.isDate = isDate;
exports.convertDate = convertDate;
const ValidationHelper_1 = require("../Engine/ValidationHelper");
const TypingUtils_1 = require("../../../Common/TypingUtils");
function parseDateString(value) {
    const [date, month, year] = (value || "").split(".").map(x => (x ? Number(x) : undefined));
    return { date: date, month: month ? month - 1 : undefined, year: year };
}
function isValidDate({ date, month, year }) {
    if (typeof date !== "number" ||
        typeof month !== "number" ||
        typeof year !== "number" ||
        year.toString().length !== 4) {
        return false;
    }
    const dateObj = new Date(Date.UTC(year, month, date));
    if (dateObj.getUTCDate() === date && dateObj.getUTCMonth() === month && dateObj.getUTCFullYear() === year) {
        return true;
    }
    return false;
}
function tryGetValidDateShape(x) {
    if (isValidDate(x)) {
        return x;
    }
    return undefined;
}
const comparator = (a, b) => {
    if (a.year < b.year) {
        return -1;
    }
    else if (a.year > b.year) {
        return 1;
    }
    else if (a.month < b.month) {
        return -1;
    }
    else if (a.month > b.month) {
        return 1;
    }
    else if (a.date < b.date) {
        return -1;
    }
    else if (a.date > b.date) {
        return 1;
    }
    return 0;
};
exports.comparator = comparator;
function isLess(left, right) {
    return (0, exports.comparator)(left, right) === -1;
}
function isEqual(a, b) {
    return ((a == undefined && b == undefined) ||
        (a != undefined && b != undefined && a.year === b.year && a.month === b.month && a.date === b.date));
}
function isLessOrEqual(left, right) {
    return isLess(left, right) || isEqual(left, right);
}
function isValidStringDate(date) {
    return isValidDate(parseDateString(date));
}
function isLessOrEqualStringDates(a, b) {
    const dateA = tryGetValidDateShape(parseDateString(a));
    const dateB = tryGetValidDateShape(parseDateString(b));
    if (dateA && dateB) {
        return isLessOrEqual(dateA, dateB);
    }
    return false;
}
function getLastDateOfMonth(date) {
    const lastDateOfMonth = new Date(date.getUTCFullYear(), date.getUTCMonth() + 1, 0);
    return lastDateOfMonth.getDate();
}
function differenceInMonths(leftDate, rightDate) {
    const sign = rightDate > leftDate ? 1 : -1;
    const leastDate = rightDate >= leftDate ? leftDate : rightDate;
    const mostDate = rightDate >= leftDate ? rightDate : leftDate;
    const isFirstDay = leastDate.getDate() === 1;
    const isLastDay = mostDate.getDate() === getLastDateOfMonth(mostDate);
    let difference = (mostDate.getFullYear() - leastDate.getFullYear()) * 12;
    difference -= leastDate.getMonth();
    difference += mostDate.getMonth();
    const sameMonth = difference === 0;
    if (sameMonth) {
        if (isFirstDay && isLastDay) {
            return sign;
        }
        return 0;
    }
    difference--;
    if (isFirstDay) {
        difference++;
    }
    if (isLastDay) {
        difference++;
    }
    return difference * sign;
}
function differenceInDays(leftDate, rightDate) {
    const difference = (rightDate.getTime() - leftDate.getTime()) / (1000 * 3600 * 24);
    return Math.floor(difference);
}
function addDays(date, amount) {
    date.setDate(date.getDate() + amount);
    return date;
}
function addMonths(date, amount) {
    date.setMonth(date.getMonth() + amount);
    return date;
}
function addYears(date, amount) {
    date.setFullYear(date.getFullYear() + amount);
    return date;
}
function dateToString(date, format) {
    const months = {
        "1": "января",
        "2": "февраля",
        "3": "марта",
        "4": "апреля",
        "5": "мая",
        "6": "июня",
        "7": "июля",
        "8": "августа",
        "9": "сентября",
        "10": "октября",
        "11": "ноября",
        "12": "декабря",
    };
    return format
        .replace("HH", date.getHours().toString().padStart(2, "0"))
        .replace("mm", date.getMinutes().toString().padStart(2, "0"))
        .replace("dd", date.getDate().toString().padStart(2, "0"))
        .replace("d", date.getDate().toString())
        .replace("MMMM", months[date.getMonth() + 1] ?? "")
        .replace("MM", (date.getMonth() + 1).toString().padStart(2, "0"))
        .replace("yyyy", date.getFullYear().toString());
}
function isDate(date) {
    return date instanceof Date && !isNaN(date.getTime());
}
function convertDate(dateValue) {
    // dd.MM.yyyy
    if (ValidationHelper_1.ValidationHelper.isDateFormat(dateValue)) {
        const v = dateValue.split(".");
        const d = parseInt(v[0] ?? (0, TypingUtils_1.reject)(), 10);
        const m = parseInt(v[1] ?? (0, TypingUtils_1.reject)(), 10);
        const y = parseInt(v[2] ?? (0, TypingUtils_1.reject)(), 10);
        return new Date(Date.UTC(y, m - 1, d));
    }
    // yyyy-MM-dd
    if (ValidationHelper_1.ValidationHelper.isDateFormatIso(dateValue)) {
        return new Date(dateValue);
    }
    return null;
}
exports.HelperFunctionsDateHelpers = {
    isValidDate: isValidDate,
    parseDateString: parseDateString,
};
