"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FilterExpressionTransduceVisitor = void 0;
exports.compileFilterExpression = compileFilterExpression;
const TypingUtils_1 = require("../../../../Common/TypingUtils");
const DateHelpers_1 = require("../DateHelpers");
const FilterExpression_1 = require("./FilterExpression");
function compileFilterExpression(expression) {
    const compileVisitor = new CompileFilterExpressionTransduceVisitor();
    return compileVisitor.transformExpression(expression);
}
class FilterExpressionTransduceVisitor {
    transformExpression(expression) {
        switch (expression.operator) {
            case FilterExpression_1.OperatorType.Or:
                return this.transformOrExpression(expression);
            case FilterExpression_1.OperatorType.And:
                return this.transformAndExpression(expression);
            case FilterExpression_1.OperatorType.Equal:
                return this.transformEqualExpression(expression);
            case FilterExpression_1.OperatorType.StartDate:
                return this.transformStartDateExpression(expression);
            case FilterExpression_1.OperatorType.EndDate:
                return this.transformEndDateExpression(expression);
            case FilterExpression_1.OperatorType.Substring:
                return this.transformSubstringExpression(expression);
            case FilterExpression_1.OperatorType.UnaryNot:
                return this.transformUnaryNotExpression(expression);
            case FilterExpression_1.OperatorType.ByInstances:
                return this.transformByInstancesExpression(expression);
            default:
                (0, TypingUtils_1.ensureNever)(expression);
                return this.createDefaultResult();
        }
    }
}
exports.FilterExpressionTransduceVisitor = FilterExpressionTransduceVisitor;
class CompileFilterExpressionTransduceVisitor extends FilterExpressionTransduceVisitor {
    createDefaultResult() {
        return () => true;
    }
    transformOrExpression(expression) {
        const conditions = expression.conditions.map(x => this.transformExpression(x));
        return (getModelValuesFn, GetInstanceFn) => conditions.some(condition => condition(getModelValuesFn, GetInstanceFn));
    }
    transformAndExpression(expression) {
        const conditions = expression.conditions.map(x => this.transformExpression(x));
        return (getModelValuesFn, GetInstanceFn) => conditions.every(condition => condition(getModelValuesFn, GetInstanceFn));
    }
    transformEqualExpression(expression) {
        const conditionValue = this.transformConditionValueExpression(expression.condition);
        return (getModelValuesFn) => getModelValuesFn(expression.condition.dataPath).some(x => x === conditionValue);
    }
    transformStartDateExpression(expression) {
        const conditionValue = this.transformConditionValueExpression(expression.condition);
        return (getModelValuesFn) => getModelValuesFn(expression.condition.dataPath).some(x => (0, DateHelpers_1.isLessOrEqualStringDates)(conditionValue, x));
    }
    transformEndDateExpression(expression) {
        const conditionValue = this.transformConditionValueExpression(expression.condition);
        return (getModelValuesFn) => getModelValuesFn(expression.condition.dataPath).some(x => (0, DateHelpers_1.isLessOrEqualStringDates)(x, conditionValue));
    }
    transformSubstringExpression(expression) {
        const conditionValue = this.transformConditionValueExpression(expression.condition);
        return (getModelValuesFn) => getModelValuesFn(expression.condition.dataPath).some(x => stringNormalize(x).includes(stringNormalize(conditionValue)));
    }
    transformUnaryNotExpression(expression) {
        const condition = this.transformExpression(expression.condition);
        return (getModelValuesFn, getPathsFn) => !condition(getModelValuesFn, getPathsFn);
    }
    transformConditionValueExpression(condition) {
        return condition.value;
    }
    transformByInstancesExpression(expression) {
        return (getModelValuesFn, getInstanceFn) => {
            const instance = getInstanceFn(expression.condition.dataPath);
            return expression.condition.instances.some(x => x === instance);
        };
    }
}
function stringNormalize(str) {
    return str.trim().toLocaleLowerCase();
}
