"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.currentViewModelSelect = exports.ViewModelSelect = void 0;
const IterableUtils_1 = require("../../../../Common/IterableUtils");
const ModelPath_1 = require("../../../../Common/ModelPath/ModelPath");
const viewModelUtils_1 = require("../../Controls/viewModelUtils");
const PathUtils_1 = require("../../Engine/Core/PathUtils/PathUtils");
const ReduxStateHelpers_1 = require("../ReduxStateHelpers");
class ViewModelSelect {
    redefinedViewModelIndex;
    useControlContext;
    constructor(useContext, redefinedViewModelIndex) {
        this.useControlContext = useContext;
        this.redefinedViewModelIndex = redefinedViewModelIndex;
    }
    withNextViewModel() {
        return new ViewModelSelect(true, this.redefinedViewModelIndex === undefined ? 0 : this.redefinedViewModelIndex + 1);
    }
    controlModel(state, bindingPath) {
        const modelValue = this.pathViewModel(state, bindingPath);
        let autoFlag = modelValue.autoFlag;
        if (typeof autoFlag === "string") {
            autoFlag = autoFlag === "true";
        }
        const bindingPathWithoutStar = PathUtils_1.PathUtils.trimLastStarIfLastToken(bindingPath);
        const tooltipEnabled = !state.errorLogTooltip?.bindingPaths.some(bp => bp.includes(bindingPathWithoutStar));
        return {
            isTooltipEnabled: tooltipEnabled,
            value: modelValue.value,
            description: modelValue.description,
            id: modelValue.id,
            error: modelValue.error?.[0],
            warning: modelValue.warning?.[0],
            children: modelValue.children,
            autoValue: modelValue.autoValue,
            disabled: typeof modelValue.disabled === "string" ? modelValue.disabled === "true" : modelValue.disabled,
            visible: modelValue.visible,
            disabledSending: modelValue.disabledSending,
            autoFlag: autoFlag,
            ignoreAutoFlag: modelValue.ignoreAutoFlag,
        };
    }
    pathViewModel(state, bindingPath) {
        if ((0, ModelPath_1.isModelPath)(bindingPath)) {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return this.viewModelGetIn(state, IterableUtils_1.IterUtils.concat(bindingPath.normalize().getSimpleTokensStrict(), ["."]), {});
        }
        else {
            const path = bindingPath.startsWith("/") ? bindingPath.split("/").slice(1) : bindingPath.split("/");
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return this.viewModelGetIn(state, [...path, "."], {});
        }
    }
    pathViewModelOrUndefined(state, bindingPath) {
        if ((0, ModelPath_1.isModelPath)(bindingPath)) {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return this.viewModelGetIn(state, IterableUtils_1.IterUtils.concat(bindingPath.normalize().getSimpleTokensStrict(), ["."]));
        }
        else {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return this.viewModelGetIn(state, [...bindingPath.split("/"), "."]);
        }
    }
    node(state, bindingPath) {
        const keys = this.splitBindingPathToIterableTokens(bindingPath);
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return this.viewModelGetIn(state, keys, {});
    }
    splitBindingPathToIterableTokens(bindingPath) {
        if ((0, ModelPath_1.isModelPath)(bindingPath)) {
            return bindingPath.normalize().getSimpleTokensStrict();
        }
        return bindingPath.split("/");
    }
    customFieldValue(state, bindingPath, customValueFieldName, defaultValue = "") {
        if (PathUtils_1.PathUtils.isPicklistFieldValueName(customValueFieldName)) {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return this.viewModelGetIn(state, [
                ...((0, ModelPath_1.isModelPath)(bindingPath)
                    ? bindingPath.normalize().getSimpleTokensStrict()
                    : (0, ModelPath_1.createAbsolute)(bindingPath).normalize().getSimpleTokensStrict()),
                ".",
                "picklist",
                PathUtils_1.PathUtils.getPicklistFieldName(customValueFieldName),
            ], defaultValue);
        }
        if (customValueFieldName === "$errorCount") {
            return this.getErrorCount(state, (0, ModelPath_1.isModelPath)(bindingPath) ? bindingPath : (0, ModelPath_1.createAbsolute)(bindingPath)).toString();
        }
        if (customValueFieldName === "$warningCount") {
            return this.getWarningCount(state, (0, ModelPath_1.isModelPath)(bindingPath) ? bindingPath : (0, ModelPath_1.createAbsolute)(bindingPath)).toString();
        }
        const fieldValue = this.viewModelGetIn(state, [
            ...((0, ModelPath_1.isModelPath)(bindingPath)
                ? bindingPath.normalize().getSimpleTokensStrict()
                : (0, ModelPath_1.createAbsolute)(bindingPath).normalize().getSimpleTokensStrict()),
            ".",
            customValueFieldName,
        ], defaultValue);
        if (customValueFieldName === "errorsCount" || customValueFieldName === "warningsCount") {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return fieldValue == undefined ? "0" : typeof fieldValue === "number" ? fieldValue.toString() : fieldValue;
        }
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return fieldValue;
    }
    getErrorCount(state, modelPath) {
        let result = 0;
        for (const viewModel of (0, viewModelUtils_1.iterateViewModels)(this.node(state, modelPath))) {
            if (viewModel.error != undefined && viewModel.error.length > 0) {
                result++;
            }
        }
        return result;
    }
    getWarningCount(state, modelPath) {
        let result = 0;
        for (const viewModel of (0, viewModelUtils_1.iterateViewModels)(this.node(state, modelPath))) {
            if (viewModel.warning != undefined && viewModel.warning.length > 0) {
                result++;
            }
        }
        return result;
    }
    customFieldValueOrUndefined(state, bindingPath, customValueFieldName) {
        if (PathUtils_1.PathUtils.isPicklistFieldValueName(customValueFieldName)) {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return this.viewModelGetIn(state, [
                ...((0, ModelPath_1.isModelPath)(bindingPath)
                    ? bindingPath.normalize().getSimpleTokensStrict()
                    : (0, ModelPath_1.createAbsolute)(bindingPath).normalize().getSimpleTokensStrict()),
                ".",
                "picklist",
                PathUtils_1.PathUtils.getPicklistFieldName(customValueFieldName),
            ]);
        }
        if (customValueFieldName === "$errorCount") {
            return this.getErrorCount(state, (0, ModelPath_1.isModelPath)(bindingPath) ? bindingPath : (0, ModelPath_1.createAbsolute)(bindingPath)).toString();
        }
        if (customValueFieldName === "$warningCount") {
            return this.getWarningCount(state, (0, ModelPath_1.isModelPath)(bindingPath) ? bindingPath : (0, ModelPath_1.createAbsolute)(bindingPath)).toString();
        }
        const fieldValue = this.viewModelGetIn(state, [
            ...((0, ModelPath_1.isModelPath)(bindingPath)
                ? bindingPath.normalize().getSimpleTokensStrict()
                : (0, ModelPath_1.createAbsolute)(bindingPath).normalize().getSimpleTokensStrict()),
            ".",
            customValueFieldName,
        ]);
        if (customValueFieldName === "errorsCount" || customValueFieldName === "warningsCount") {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return fieldValue == undefined ? "0" : typeof fieldValue === "number" ? fieldValue.toString() : fieldValue;
        }
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return fieldValue;
    }
    children(state, bindingPath, defaultValue = []) {
        if ((0, ModelPath_1.isModelPath)(bindingPath)) {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return this.viewModelGetIn(state, IterableUtils_1.IterUtils.concat(bindingPath.normalize().getSimpleTokensStrict(), [".", "children"]), defaultValue);
        }
        else {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return this.viewModelGetIn(state, [...bindingPath.split("/"), ".", "children"], defaultValue);
        }
    }
    draftMeta(state) {
        const rootElement = this.getRootElement(state);
        if (rootElement == undefined) {
            return undefined;
        }
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return this.viewModelGetIn(state, [rootElement, "."]);
    }
    getRootElement(state) {
        return IterableUtils_1.IterUtils.first((0, IterableUtils_1.iterateKeys)(state.viewModel));
    }
    viewModelGetIn(state = {}, keys, 
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    defaultValue
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    ) {
        let viewModel = state.viewModel;
        if (state.redefinedViewModels) {
            if (this.useControlContext) {
                if (this.redefinedViewModelIndex !== undefined) {
                    viewModel = state.redefinedViewModels[this.redefinedViewModelIndex];
                }
            }
            else {
                if (state.redefinedViewModels.length > 0) {
                    viewModel = state.redefinedViewModels[state.redefinedViewModels.length - 1];
                }
            }
        }
        // На всякий случай
        if (!viewModel) {
            // eslint-disable-next-line no-console
            console.error("viewModelGetIn пытается обратиться к redefinedViewModels равной undefined");
            viewModel = state.viewModel;
        }
        const result = (0, ReduxStateHelpers_1.getIn)(viewModel, keys, defaultValue);
        return result ?? defaultValue;
    }
}
exports.ViewModelSelect = ViewModelSelect;
exports.currentViewModelSelect = new ViewModelSelect(false);
