"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SugarAttributeReadError = exports.SugarNodeReadingError = exports.SugarReadingError = exports.ASTNode = exports.XmlNode = void 0;
exports.getNodeSource = getNodeSource;
const generator_1 = __importDefault(require("@babel/generator"));
const IterableUtils_1 = require("../../../../Common/IterableUtils");
const TypingUtils_1 = require("../../../../Common/TypingUtils");
const ExtendedError_1 = require("../../../../Common/Errors/ExtendedError");
function getNodeSource(node) {
    if (node.source == undefined) {
        return "N/A";
    }
    try {
        const sourceLines = node.source.sourceText.split("\n");
        return sourceLines.slice(node.source.location.start.line - 1, node.source.location.end.line).join("\n");
    }
    catch {
        return "N/A";
    }
}
class XmlNode {
    attributes;
    children;
    name;
    parent;
    source;
    attributeSources;
    constructor(createParams) {
        this.name = createParams.name;
        this.parent = createParams.parent ?? undefined;
        this.attributes = createParams.attributes ?? {};
        this.source = createParams.source;
        this.attributeSources = createParams.attributeSources;
        this.children = (createParams.children ?? []).map(child => {
            child.setParent(this);
            return child;
        });
    }
    clone() {
        return new XmlNode({
            name: this.name,
            parent: this.parent,
            attributes: this.attributes,
            source: this.source,
            attributeSources: this.attributeSources,
            children: this.children.map(x => x.clone()),
        });
    }
    getAllAttributeNames() {
        return (0, IterableUtils_1.iterateKeys)(this.attributes);
    }
    getChildren(childrenName) {
        if (childrenName == undefined) {
            return this.children;
        }
        return this.children.filter(node => node.name === childrenName);
    }
    getFirstChild(childName) {
        if (childName == undefined) {
            return this.children[0];
        }
        const arr = this.getChildren(childName);
        return arr[0];
    }
    getRoot() {
        if (this.parent == undefined) {
            return this;
        }
        return this.parent?.getRoot() ?? this;
    }
    getParent() {
        return this.parent;
    }
    getParentStrict() {
        return this.getParent() ?? (0, TypingUtils_1.reject)(`Node ${this.name} has no parent`);
    }
    getParentPageStrict() {
        return this.findAncestor("page") ?? (0, TypingUtils_1.reject)("Not parent page element");
    }
    getParentPageOrUndefined() {
        return this.findAncestor("page");
    }
    findAncestor(parentName) {
        if (this.name === parentName) {
            return this;
        }
        if (!this.parent) {
            return undefined;
        }
        return this.parent.findAncestor(parentName);
    }
    setChildren(inputChildren) {
        const children = Array.isArray(inputChildren) ? inputChildren : [inputChildren];
        for (const child of children) {
            child.setParent(this);
            this.children.push(child);
        }
    }
    replaceChildren(inputChildren) {
        const children = Array.isArray(inputChildren) ? inputChildren : [inputChildren];
        for (const child of children) {
            child.setParent(this);
        }
        this.children = children;
    }
    setParent(parentNode) {
        this.parent = parentNode;
    }
    hasAttribute(attrName) {
        return Object.prototype.hasOwnProperty.call(this.attributes ?? {}, attrName);
    }
    attr(attrName, defaultValue = "") {
        // eslint-disable-next-line @typescript-eslint/no-base-to-string
        return (this.attributes[attrName] || defaultValue).toString().toLowerCase();
    }
    getAttributeAsNumber(attrName, defaultValue) {
        if (!this.attributes[attrName]) {
            return defaultValue;
        }
        const value = Number(this.attributes[attrName]);
        if (Number.isNaN(value)) {
            throw new SugarAttributeReadError(`Error: Attibute ${attrName} in ${this.name} is not a number!`, this, attrName);
        }
        return value;
    }
    getAttributeAsNumberStrict(attrName) {
        const value = this.getAttributeAsNumber(attrName);
        if (value == undefined) {
            throw new SugarAttributeReadError(`Error: Attibute ${attrName} in ${this.name} is required`, this, attrName);
        }
        return value;
    }
    getAttributeAsArray(attrName, type) {
        const attributeValue = this.attributes[attrName];
        if (attributeValue === undefined) {
            return [];
        }
        if (Array.isArray(attributeValue)) {
            // @ts-ignore i dont know how fix it
            return attributeValue;
        }
        if (type === "number") {
            return [Number(attributeValue)];
        }
        return [attributeValue];
    }
    attrAsLengthUnit(attrName) {
        const value = this.attributes[attrName];
        if (value == undefined) {
            return undefined;
        }
        if (typeof value === "number") {
            return value;
        }
        if (typeof value === "string") {
            if (/^[+-]?[0-9]+$/.test(value)) {
                return parseInt(value, 10);
            }
            if (/^[+-]?([0-9]+)?\.?[0-9]+(px|em|ex|%|in|cm|mm|pt|pc)$/.test(value)) {
                return value;
            }
        }
        throw new SugarAttributeReadError(`Attribute ${attrName} of node ${this.name} cannot be conveterd to unit of length\n` +
            `Value: \n${JSON.stringify(value)}`, this, attrName);
    }
    attrAsBoolean(attrName) {
        const attribute = this.attributes[attrName];
        const hasAttribute = Object.prototype.hasOwnProperty.call(this.attributes, attrName);
        if (attribute == undefined) {
            if (hasAttribute) {
                return true;
            }
            return attribute;
        }
        if (typeof attribute === "string") {
            if (attribute === "true" || attribute === "True") {
                return true;
            }
            if (attribute === "false" || attribute === "False") {
                return false;
            }
        }
        if (typeof attribute === "boolean") {
            return attribute;
        }
        throw new SugarAttributeReadError(`Expected value to be on of ('true', 'false', 'True', 'False')`, this, attrName);
    }
    attrAsString(attrName, defaultValue) {
        const value = this.attributes[attrName];
        if (value == undefined) {
            return defaultValue;
        }
        if (typeof value === "string" || typeof value === "number") {
            return value.toString();
        }
        else {
            throw new SugarAttributeReadError(`Attribute ${attrName} of node ${this.name} cannot be conveterd to string\n` +
                `Value: \n${JSON.stringify(value)}`, this, attrName);
        }
    }
    attrAsStringStrict(attrName) {
        const result = this.attrAsString(attrName);
        if (result == undefined) {
            throw new SugarAttributeReadError(`Attribute ${attrName} of node ${this.name} should exists and be a string`, this, attrName);
        }
        return result;
    }
    boolAttr(attrName, defaultValue) {
        return this.attr(attrName) ? this.attr(attrName) === "true" : !!defaultValue;
    }
    setAttribute(attrName, value) {
        if (attrName === undefined || value === undefined) {
            return;
        }
        this.attributes[attrName] = value;
    }
    removeAttribute(attrName) {
        Reflect.deleteProperty(this.attributes, attrName);
    }
    attrAsStringLocalized(attrName) {
        const value = this.attributes[attrName];
        if (value instanceof ASTNode) {
            return { type: "Expression", expression: value.toString() };
        }
        if (typeof value === "string" || typeof value === "number") {
            return value.toString();
        }
        else if (value == undefined) {
            return undefined;
        }
        throw new SugarAttributeReadError(`Attribute ${attrName} of node ${this.name} cannot be conveterd to string\n` +
            `Value: \n${JSON.stringify(value)}`, this, attrName);
    }
}
exports.XmlNode = XmlNode;
class ASTNode {
    node;
    constructor(node) {
        this.node = node;
    }
    toString() {
        const result = (0, generator_1.default)(this.node, { jsescOption: { minimal: true } }).code;
        return result.slice(-1) === ";" ? result.slice(0, -1) : result;
    }
}
exports.ASTNode = ASTNode;
class SugarReadingError extends ExtendedError_1.ExtendedError {
}
exports.SugarReadingError = SugarReadingError;
class SugarNodeReadingError extends SugarReadingError {
    sourceXmlNode;
    constructor(message, sourceXmlNode, innerError) {
        super(message, innerError);
        this.sourceXmlNode = sourceXmlNode instanceof XmlNode ? sourceXmlNode : sourceXmlNode.sourceXmlNode;
    }
}
exports.SugarNodeReadingError = SugarNodeReadingError;
class SugarAttributeReadError extends SugarReadingError {
    sourceXmlNode;
    attributeName;
    constructor(message, sourceXmlNode, attributeName, innerError) {
        super(message, innerError);
        this.sourceXmlNode = sourceXmlNode instanceof XmlNode ? sourceXmlNode : sourceXmlNode.sourceXmlNode;
        this.attributeName = attributeName;
    }
}
exports.SugarAttributeReadError = SugarAttributeReadError;
