"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConverterResult = void 0;
const TypingUtils_1 = require("../../../../../Common/TypingUtils");
const IterableUtils_1 = require("../../../../../Common/IterableUtils");
const ModuleDescriptor_1 = require("./ModuleDescriptor");
const defaultContextName = "default";
class ConverterResult {
    contextsData = new Map();
    contexts = new Map();
    currentCtxName;
    constructor(body = "", deps) {
        this.contextsData.set(defaultContextName, {
            body: body,
            deps: [],
        });
        this.currentCtxName = defaultContextName;
        this.addDependencies(deps);
    }
    static empty() {
        return ConverterResult.fromConverterResults([], "");
    }
    static fromConverterResults(source, jsx, dependencies) {
        if (jsx == undefined) {
            throw new Error('"jsx" arg in fromConverterResults is absent');
        }
        const sourceConverterResults = Array.isArray(source) ? source : [source];
        const result = this.mergeConverterResults(sourceConverterResults);
        if (jsx) {
            result.setRenderBody(jsx);
        }
        result.addDependencies(dependencies);
        return result;
    }
    static mergeConverterResults(sourceConversionResults) {
        const result = new ConverterResult();
        for (const sourceConverterResult of sourceConversionResults) {
            result.appendConverterResult(sourceConverterResult);
        }
        return result;
    }
    appendConverterResult(source) {
        for (const [key, value] of source.contexts.entries()) {
            this.contexts.set(key, value);
        }
        const sourceContextNames = [...source.contexts.keys()];
        for (const sourceContextName of sourceContextNames) {
            if (!source.hasContextWithName(sourceContextName)) {
                break;
            }
            if (this.hasContextWithName(sourceContextName)) {
                throw new Error(`Cannot merge different contexts with the same name: '${sourceContextName}'.`);
            }
            const sourceContextData = source.getContextData(sourceContextName);
            this.contextsData.set(sourceContextName, sourceContextData);
            const sourceContext = source.getContext(sourceContextName);
            const depPath = `../${sourceContext.dirName}/${sourceContextName}`;
            const depName = `${sourceContextName}Creator`;
            this.addDependencies(new ModuleDescriptor_1.ModuleDescriptor(depPath, depName));
        }
        const defaultContextData = source.tryGetContextData(defaultContextName);
        if (defaultContextData != undefined) {
            this.addDependencies(defaultContextData.deps);
        }
    }
    addDependencies(deps) {
        if (deps == undefined) {
            return;
        }
        const depsArray = deps instanceof ModuleDescriptor_1.ModuleDescriptor ? [deps] : deps;
        const context = this.getCurrentCtxData();
        context.deps = [...IterableUtils_1.IterUtils.distinctBy(context.deps.concat(depsArray), x => x.toHash())];
    }
    getDependencies(ctxName) {
        return this.getContextData(ctxName).deps;
    }
    getDependenciesContent(ctxName, builder) {
        const context = this.getContext(ctxName);
        const contextPath = context.dirName;
        const dependencyFilesWithContent = this.getDependencies(ctxName)
            .map(dependency => dependency.toFile())
            .filter(TypingUtils_1.isNotNullOrUndefined);
        if (context.isBase) {
            builder.addComponent(`${contextPath}/index.js`, `export default from './${ctxName}'\n`, true);
        }
        for (const dependencyFile of dependencyFilesWithContent) {
            const dependencyFilePath = `${contextPath}/${dependencyFile.getName()}`;
            builder.addComponent(dependencyFilePath, dependencyFile.getContent(), true);
        }
    }
    setRenderBody(body = "") {
        this.getCurrentCtxData().body = body;
    }
    getRenderBody(ctxName) {
        return this.getContextData(ctxName).body;
    }
    getContexts() {
        return this.contexts.values().toArray();
    }
    setCurrentCtx(convertationContext) {
        const ctxName = convertationContext.name;
        this.contextsData.set(ctxName, this.getCurrentCtxData());
        this.contextsData.delete(this.currentCtxName);
        this.currentCtxName = ctxName;
        this.contexts.set(ctxName, convertationContext);
    }
    getContext(ctxName) {
        const result = this.contexts.get(ctxName);
        if (result == undefined) {
            throw new Error("InvalidProgramState");
        }
        return result;
    }
    hasContextWithName(sourceContextName) {
        return this.contextsData.has(sourceContextName);
    }
    getContextData(ctxName) {
        const context = this.tryGetContextData(ctxName);
        if (context == undefined) {
            throw new Error("InvalidProgramState");
        }
        return context;
    }
    tryGetContextData(ctxName) {
        return this.contextsData.get(ctxName || this.currentCtxName);
    }
    getCurrentCtxData() {
        return this.getContextData();
    }
}
exports.ConverterResult = ConverterResult;
