"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CandyServiceTestContext = void 0;
const https_1 = require("https");
const node_fetch_1 = __importDefault(require("node-fetch"));
const uuid_1 = require("uuid");
const KonturApi_1 = require("../Core/Api/KonturApi");
const EndPoints_1 = require("../../../../Common/EndPoints");
const CandyApiClient_1 = require("../../../../Common/CandyApiClient/CandyApiClient");
const NodeFetch_1 = require("../../../../Common/HttpClient/FetchImplementation/NodeFetch");
const DraftPresentationType_1 = require("../../../../Common/CandyApiClient/Types/Presentations/DraftPresentationType");
const CommonUtils_1 = require("../PageObjects/CommonUtils");
class CandyServiceTestContext {
    konturApi;
    candyApi;
    testUserSid;
    externSession;
    constructor(konturApi, candyApi, testUserAuthInfo) {
        this.konturApi = konturApi;
        this.candyApi = candyApi;
        this.testUserSid = testUserAuthInfo.authSid;
        this.externSession = testUserAuthInfo.externSession;
    }
    static async initialize(options = { domain: ".testkontur.ru" }) {
        const konturApi = new KonturApi_1.KonturApi(EndPoints_1.testKonturApi);
        const testUserAuthInfo = await konturApi.getAuthInfoFromAuth("forms@tests.ru", "asasas");
        const candyApi = new CandyApiClient_1.CandyApiClient(NodeFetch_1.nodeFetch, EndPoints_1.localCandyApi, options?.resourceZone, {
            authSid: testUserAuthInfo.authSid,
        });
        await this.ensureAliveApi(candyApi);
        return new CandyServiceTestContext(konturApi, candyApi, testUserAuthInfo);
    }
    static async ensureAliveApi(candyApi) {
        let lastError;
        for (let i = 0; i < 60; i++) {
            try {
                await candyApi.checkAlive();
                return;
            }
            catch (e) {
                // eslint-disable-next-line no-console
                console.log(e);
                await (0, CommonUtils_1.delay)(1000);
                if (e instanceof Error) {
                    lastError = e;
                }
                else {
                    lastError = new Error(`${e}`);
                }
            }
        }
        // eslint-disable-next-line @typescript-eslint/only-throw-error
        throw lastError;
    }
    async downloadFile(url) {
        const cookie = this.getCookie();
        const response = await (0, node_fetch_1.default)(url, {
            method: "GET",
            headers: {
                ["cookie"]: cookie,
            },
            agent: new https_1.Agent({ rejectUnauthorized: false }),
        });
        if (!response.ok) {
            throw new Error(`Unable to download file. Code: ${response.status}. File url: '${url}'`);
        }
        return response.buffer();
    }
    async getInner(draftFullId) {
        return await this.candyApi.forDraft(draftFullId).getInner(undefined);
    }
    async createDefaultDraft(gfv) {
        const ns = (0, uuid_1.v4)();
        const result = await this.candyApi.createDraft(ns, gfv);
        return result;
    }
    async createDraftFromFuf(fufContent) {
        const ns = (0, uuid_1.v4)();
        const result = await this.candyApi.createDraft(ns, undefined, fufContent);
        return result;
    }
    async setPresentations(draftInfo, presentations) {
        if (presentations?.clientRequisites != undefined) {
            await this.candyApi
                .forDraft(draftInfo)
                .setPresentation(DraftPresentationType_1.DraftPresentationType.clientRequisites, presentations.clientRequisites);
        }
        if (presentations?.formsClientInfo != undefined) {
            await this.candyApi
                .forDraft(draftInfo)
                .setPresentation(DraftPresentationType_1.DraftPresentationType.formsClientInfo, presentations.formsClientInfo);
        }
        if (presentations?.inner != undefined) {
            await this.candyApi.forDraft(draftInfo).setPresentation(DraftPresentationType_1.DraftPresentationType.inner, presentations.inner);
        }
        if (presentations?.outer != undefined) {
            await this.candyApi.forDraft(draftInfo).setPresentation(DraftPresentationType_1.DraftPresentationType.outer, presentations.outer);
        }
        if (presentations?.fuf != undefined) {
            await this.candyApi.forDraft(draftInfo).setPresentation(DraftPresentationType_1.DraftPresentationType.fuf, presentations.fuf);
        }
    }
    async sendChangeSet(draftInfo, changeSet) {
        const dataVersion = await this.candyApi.forDraft(draftInfo).getDataVersion();
        await this.candyApi.forDraft(draftInfo).sendChangeSet(changeSet, dataVersion);
    }
    getCookie() {
        const sid = `auth.sid=${this.testUserSid};`;
        return this.externSession ? `${sid} ExternSession=${this.externSession};` : sid;
    }
}
exports.CandyServiceTestContext = CandyServiceTestContext;
